@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 """
 This file implements the Job class for pegasus-monitord.
 """
@@ -22,6 +20,7 @@ This file implements the Job class for pegasus-monitord.
 
 # Import Python modules
 import os
+import sys
 import re
 import logging
 
@@ -32,6 +31,7 @@ logger = logging.getLogger()
 
 # Global variables
 good_rsl = {"maxcputime": 1, "maxtime":1, "maxwalltime": 1}
+MAX_OUTPUT_LENGTH = 2**16-1  # Only keep stdout to 64K
 
 # Used in parse_sub_file
 re_rsl_string = re.compile(r"^\s*globusrsl\W", re.IGNORECASE)
@@ -104,6 +104,8 @@ class Job:
                                        # line for pegasus-plan and subdax_ jobs
         self._kickstart_parsed = False # Flag indicating if the kickstart
                                        # output for this job was parsed or not
+        self._has_rotated_stdout_err_files = False #Flag indicating whether we detected that job stdout|stderr
+                                                  #was rotated or not, as is the default case.
 
     def set_job_state(self, job_state, sched_id, timestamp, status):
         """
@@ -295,7 +297,8 @@ class Job:
         my_invocation_found = False
         my_task_number = 0
         self._stdout_text = "" # Initialize stdout
-
+        stdout_text_list = []
+        stdout_size=0
         for my_record in kickstart_output:
             if not "invocation" in my_record:
                 # Not this one... skip to the next
@@ -321,14 +324,36 @@ class Job:
                 # We are done with this part
                 my_invocation_found = True
 
+            #PM-641 optimization Modified string concatenation to a list join 
             if "stdout" in my_record:
-                self._stdout_text = self._stdout_text + "#@ %d stdout\n" % (my_task_number) + my_record["stdout"] + "\n"
-            if "stderr" in my_record:
-                self._stdout_text = self._stdout_text + "#@ %d stderr\n" % (my_task_number) + my_record["stderr"] + "\n"
+                if len(my_record["stdout"])<= MAX_OUTPUT_LENGTH - stdout_size:
+                    try:
+                        stdout_text_list.append(utils.quote("#@ %d stdout\n" % (my_task_number)))
+                        stdout_text_list.append(utils.quote(my_record["stdout"]))
+                        stdout_text_list.append(utils.quote("\n"))
+                        stdout_size+=len(my_record["stdout"])+20
+                    except KeyError:
+                        logger.exception( "Unable to parse stdout section from kickstart record for task %s from file %s " %(my_task_number, self.get_rotated_out_filename() ))
 
+            if "stderr" in my_record:
+                if len(my_record["stderr"]) <= MAX_OUTPUT_LENGTH - stdout_size :
+                    try:
+                        stdout_text_list.append(utils.quote("#@ %d stderr\n" % (my_task_number)))
+                        stdout_text_list.append(utils.quote(my_record["stderr"]))
+                        stdout_text_list.append(utils.quote("\n"))
+                        stdout_size+=len(my_record["stderr"])+20
+                    except KeyError:
+                        logger.exception( "Unable to parse stderr section from kickstart record for task %s from file %s " %(my_task_number, self.get_rotated_out_filename() ))
+
+        if len(stdout_text_list) > 0 :
+            self._stdout_text = "".join(stdout_text_list)
+
+
+            #PM-641 optimization merged encoding above
         # Now, we encode it!
-        if self._stdout_text != "":
-            self._stdout_text = utils.quote(self._stdout_text)
+#        if self._stdout_text != "":
+#            self._stdout_text = utils.quote(self._stdout_text)
+
 
         if not my_invocation_found:
             logger.debug("cannot find invocation record in output")
@@ -374,20 +399,52 @@ class Job:
         # Done populating Job class with information from the output file
         return my_invocation_found
 
+    def get_rotated_out_filename(self):
+        """
+        Returns the name of the rotated .out file for the job on the basis
+        of the current counter
+        """
+
+        basename = self._output_file
+        if self._has_rotated_stdout_err_files:
+            basename += ".%03d" % ( self._job_output_counter)
+
+        return basename
+
+    def get_rotated_err_filename(self ):
+        """
+        Returns the name of the rotated .err file for the job on the basis
+        of the current counter
+        """
+
+        basename = self._error_file
+        if self._has_rotated_stdout_err_files:
+            basename += ".%03d" % ( self._job_output_counter)
+
+        return basename
+
     def read_stdout_stderr_files(self, run_dir):
         """
         This function reads both stdout and stderr files and populates
         these fields in the Job class.
         """
+        my_max_encoded_length = MAX_OUTPUT_LENGTH - 2000
         if self._output_file is None:
             # This is the case for SUBDAG jobs
             self._stdout_text = None
         else:
-            my_out_file = os.path.join(run_dir, self._output_file)
+            basename = self._output_file
+            if self._has_rotated_stdout_err_files:
+                basename += ".%03d" % ( self._job_output_counter)
+
+            my_out_file = os.path.join(run_dir, basename)
 
             try:
                 OUT = open(my_out_file, 'r')
-                self._stdout_text = utils.quote("#@ 1 stdout\n" + OUT.read())
+                buffer = OUT.read()
+                if len( buffer ) > my_max_encoded_length :
+                    buffer = buffer[:my_max_encoded_length]
+                self._stdout_text = utils.quote("#@ 1 stdout\n" + buffer)
             except IOError:
                 self._stdout_text = None
                 logger.warning("unable to read output file: %s, continuing..." % (my_out_file))
@@ -398,11 +455,18 @@ class Job:
             # This is the case for SUBDAG jobs
             self._stderr_text = None
         else:
-            my_err_file = os.path.join(run_dir, self._error_file)
+            basename = self._error_file
+            if self._has_rotated_stdout_err_files:
+                basename += ".%03d" % ( self._job_output_counter)
+
+            my_err_file = os.path.join(run_dir, basename)
 
             try:
                 ERR = open(my_err_file, 'r')
-                self._stderr_text = utils.quote(ERR.read())
+                buffer = ERR.read()
+                if len( buffer ) > my_max_encoded_length :
+                    buffer = buffer[:my_max_encoded_length]
+                self._stderr_text = utils.quote(buffer)
             except IOError:
                 self._stderr_text = None
                 logger.warning("unable to read error file: %s, continuing..." % (my_err_file))
