@@ -1,8 +1,5 @@
-#!/usr/bin/env python
 """
-Pegasus utility for generating workflow execution gantt chart
-
-Usage: pegasus-gantt [options] submit directory
+Pegasus library for generating workflow execution gantt chart
 
 """
 
@@ -42,7 +39,6 @@ from datetime import datetime
 
 
 #Global variables----
-prog_base = os.path.split(sys.argv[0])[1]	# Name of this program
 output_dir = None
 
 
@@ -1079,10 +1075,3 @@ def generate_chart(workflow_info):
 	create_gantt_plot_page(workflow_info , output_dir)
 
 
-# ---------main----------------------------------------------------------------------------
-def main():
-	sys.exit(0)
-
-
-if __name__ == '__main__':
-	main()
