@@ -59,6 +59,7 @@ class Parser:
         self._parsing_data = False
         self._parsing_cwd = False
         self._line_number = 0
+        self._record_number = 0
         self._arguments = ""
         self._stdout = ""
         self._stderr = ""
@@ -104,6 +105,12 @@ class Parser:
         """
         buffer = ""
 
+        #valid token that is parsed
+        token = ""
+
+        self._record_number += 1
+        logger.debug( "Started reading record number %d from kickstart file %s" %( self._record_number, self._kickstart_output_file))
+
         # First, we find the beginning <invocation xmlns....
         while True:
             line = self._fh.readline()
@@ -111,14 +118,26 @@ class Parser:
                 # End of file, record not found
                 return None
             if line.find("<invocation") != -1:
+                token = "<invocation"
                 break
-            if line.find("[seqexec-task") != -1:
+            if ( line.find("[cluster-task") != -1 ):
+                token = "[cluster-task"
                 break
-            if line.find("[seqexec-summary") != -1:
+            if ( line.find("[cluster-summary") != -1 ):
+                token = "[cluster-summary"
+                break
+            if ( line.find("[seqexec-task") != -1 ):
+                #deprecated token
+                token = "[seqexec-task"
+                break
+            if ( line.find("[seqexec-summary") != -1 ):
+                #deprecated token
+                token = "[seqexec-summary"
                 break
 
-        # Found something!
-        if line.find("<invocation") >= 0:
+        # Found something!        
+        #if line.find("<invocation") >= 0:
+        if token == "<invocation" :
             # Found invocation record
             start = line.find("<invocation")
             buffer = line[start:]
@@ -128,9 +147,10 @@ class Parser:
             if end >= 0:
                 end = end + len("</invocation>")
                 return buffer[:end]
-        elif line.find("[seqexec-summary") >= 0:
+        #elif line.find("[seqexec-summary") >= 0:
+        elif ( token == "[cluster-summary" or token == "[seqexec-summary" ):
             # Found line with cluster jobs summary
-            start = line.find("[seqexec-summary")
+            start = line.find(token)
             buffer = line[start:]
             end = buffer.find("]")
 
@@ -139,41 +159,51 @@ class Parser:
                 return buffer[:end]
 
             # clustered record should be in a single line!
-            logger.warning("%s: seqexec-summary line is malformed... ignoring it..." % (self._kickstart_output_file))
+            logger.warning("%s: %s line is malformed... ignoring it..." % (self._kickstart_output_file, token ))
             return ""
-        elif line.find("[seqexec-task") >= 0:
+        #elif line.find("[seqexec-task") >= 0:
+        elif ( token == "[cluster-task" or token == "[seqexec-task" ):
             # Found line with task information
-            start = line.find("[seqexec-task")
+            start = line.find( token )
             buffer = line[start:]
             end = buffer.find("]")
 
             if end >= 0:
-                end = end + len("]")
+                end = end + len("]")                
                 return buffer[:end]
 
             # task record should be in a single line!
-            logger.warning("%s: seqexec-task line is malformed... ignoring it..." % (self._kickstart_output_file))
+            logger.warning("%s: %s line is malformed... ignoring it..." % (self._kickstart_output_file, token))
             return ""
         else:
             return ""
 
         # Ok, now continue reading the file until we get a full record
+        buffer = [buffer]
+
         while True:
             line = self._fh.readline()
             if line == '':
                 # End of file, record not found
                 return None
-            buffer = buffer + line
-            if buffer.find("</invocation>") > 0:
+            #buffer = buffer + line
+            buffer.append( line )
+            if line.find("</invocation>") >= 0:
                 break
 
         # Now, we got it, let's make sure
-        end = buffer.find("</invocation>")
+        end = line.find("</invocation>")
         if end == -1:
             return ""
 
-        end = end + len("</invocation>")
-        return buffer[:end]
+
+        #end = end + len("</invocation>")
+        invocation = "".join(buffer)
+        #print invocation
+        logger.debug( "Finished reading record number %d from kickstart file %s" %( self._record_number, self._kickstart_output_file))
+        return invocation
+        #return buffer[:end]
+
 
     def is_invocation_record(self, buffer=''):
         """
@@ -187,17 +217,17 @@ class Parser:
         """
         Returns True if buffer contains a task record.
         """
-        if buffer.find("[seqexec-task") == -1:
-            return False
-        return True
+        if ( buffer.find("[seqexec-task") != -1 or buffer.find( "[cluster-task" ) != -1 ):
+            return True
+        return False
 
     def is_clustered_record(self, buffer=''):
         """
         Returns True if buffer contains a clustered record.
         """
-        if buffer.find("[seqexec-summary") == -1:
-            return False
-        return True
+        if ( buffer.find("[seqexec-summary") != -1 or buffer.find( "[cluster-summary" ) != -1):
+            return True
+        return False
 
     def start_element(self, name, attrs):
         """
@@ -461,9 +491,12 @@ class Parser:
         if self.open() == False:
             return my_reply
 
+        logger.debug( "Started reading records from kickstart file %s" %(self._kickstart_output_file))
+
+        self._record_number = 0
         # Read first record
         my_buffer = self.read_record()
-        
+
         # Loop while we still have record to read
         while my_buffer is not None:
             if self.is_invocation_record(my_buffer) == True:
