@@ -32,20 +32,9 @@ import commands
 import datetime
 import traceback
 import subprocess
-
-# The unquote routine comes from urllib
-from urllib import unquote
+import urllib
 
 __all__ = ['quote', 'unquote']
-_mapping = {}
-
-# Initialize _mapping
-for i, c in zip(xrange(256), ''.join([chr(x) for x in xrange(256)])):
-    if (i >= 32 and i < 127 and c not in '"%\''):
-        _mapping[c] = c
-    else:
-        _mapping[c] = ('%%%02X'%i)
-del i; del c
 
 # Compile our regular expressions
 
@@ -65,28 +54,84 @@ logger = logging.getLogger()
 
 def quote(s):
     """
-    Encodes a string using a partial URL encoding. The encoding
-    replaces the following elements with their URL-encoded
-    equivalents:
-    1. Non-printing and control characters (characters < 0x20 and 0x7F [DEL])
-    2. Extended ASCII characters (characters >= 0x80)
-    3. Percent (0x25), quote (0x27), and double quote (0x22)
+    Encodes a byte string using URL encoding. This replaces bytes with their
+    URL-encoded equivalent %XX where XX is the hexadecimal value of the byte.
+    In particular, the bytes that are encoded include:
+
+    1. Bytes < 0x20 or >= 0x7F (C0 and C1 control codes)
+    2. Quote/apostrophe (0x27), double quote (0x22), and percent (0x25)
+
+    This will always return a byte string. If the argument is a unicode string
+    then it will be utf-8 encoded before being quoted.
     """
-    return ''.join(map(_mapping.__getitem__, s))
+    if not isinstance(s, basestring):
+        raise TypeError("Not a string: %s" % str(s))
+
+    if isinstance(s, unicode):
+        # We need to utf-8 encode unicode strings
+        s = s.encode('utf-8')
+
+    buf = []
+    for c in s:
+        i = ord(c)
+        if i < 0x20 or i >= 0x7F:
+            # Any byte less than 0x20 is a control character
+            # any byte >= 0x7F is either a control character
+            # or a multibyte unicode character
+            buf.append("%%%02X" % i)
+        elif c == '%':
+            buf.append('%25')
+        elif c == "'":
+            buf.append('%27')
+        elif c == '"':
+            buf.append('%22')
+        else:
+            # These are regular bytes
+            buf.append(c)
+
+    return ''.join(buf)
+
+def unquote(s):
+    """
+    Unquote a URL-quoted string.
+
+    This will always return a byte string with an unknown encoding. If the
+    argument is a unicode string then it will be encoded in latin-1 before being
+    unquoted. Latin-1 is used because it doesn't modify bytes <= 0xFF, which
+    is all we expect a quoted string to contain.
+
+    Unfortunately, this cannot return a unicode string because we have no
+    way of knowing what encoding was used for the original string that was
+    passed to quote().
+    """
+    if not isinstance(s, basestring):
+        raise TypeError("Not a string: %s" % str(s))
+
+    if isinstance(s, unicode):
+        # Technically, this should not happen because
+        # quote always returns a byte string, but if it was
+        # passed through a database or something then it might happen
+        # Latin-1 is used because it has the nice property that every
+        # unicode code point <= 0xFF has the same value in latin-1
+        # We ignore anything else (i.e. >0xFF) because, tecnically, it
+        # should have been removed by quote()
+        s = s.encode('latin-1', 'ignore')
+
+    return urllib.unquote(s)
 
 def isodate(now=int(time.time()), utc=False, short=False):
     """
     This function converts seconds since epoch into ISO 8601 timestamp
     """
-    my_time_u = time.gmtime(now)
     if utc:
+        my_time_u = time.gmtime(now)
         if short:
             return time.strftime("%Y%m%dT%H%M%SZ", my_time_u)
         else:
             return time.strftime("%Y-%m-%dT%H:%M:%SZ", my_time_u)
     else:
         my_time_l = time.localtime(now)
-        my_offset = int( time.mktime(my_time_l) - time.mktime(my_time_u) )
+        my_offset = int( calendar.timegm(my_time_l) - time.mktime(my_time_l) )
         offset = "%+03d%02d" % ( my_offset / 3600, (abs(my_offset) % 3600) / 60)
         if short:
             return time.strftime("%Y%m%dT%H%M%S", my_time_l) + offset
@@ -126,7 +171,7 @@ def epochdate(timestamp):
                 my_time = my_time + my_offset
             else:
                 my_time = my_time - my_offset
-
+        
         # Turn my_time into Epoch format
         return int(calendar.timegm(my_time.timetuple()))
 
@@ -134,6 +179,60 @@ def epochdate(timestamp):
         logger.warn("unable to parse timestamp \"%s\"" % timestamp)
         return None
 
+def get_path_dashboard_db( props ):
+    """
+    Utility method for creating appropriate directory and the path to the
+    dashboard database file
+    @param props  properties passed by user
+
+    # Returns fully qualified path to binary, None if not found
+    """
+    db_file = props.property("pegasus.dashboard.output")
+
+    if db_file is not None:
+        return db_file
+
+    #construct the default path
+    home = os.getenv("HOME")
+    
+    if home == None:
+        logger.error('Environment variable HOME not defined, set pegasus.dashboard.output property to point to the Dashboard database')
+        return None
+        
+    dir = os.path.join( home, ".pegasus" );
+
+    # check for writability and create directory if required
+    if not os.path.isdir( dir ):
+        try:
+            os.mkdir( dir  )
+        except OSError:
+            logger.error("Unable to create directory." + dir)
+            return None
+    elif not os.access( dir, os.W_OK ):
+        logger.warning( "unable to write to directory " + dir )
+        return None
+
+    #directory exists, touch the file and set permissions
+    filename =  os.path.join( dir, "workflow.db" )
+    if not os.access(filename, os.F_OK):
+        try:
+            # touch the file
+            open(filename, 'w').close()
+            os.chmod( filename, 0600)
+        except:
+            logger.warning("unable to initialize dashboard db %s. ..." % (filename))
+            logger.warning(traceback.format_exc())
+            return None
+    elif not os.access( filename, os.W_OK ):
+        logger.warning( "no read access for file " + filename )
+        return None
+
+    db_file = "sqlite:///" + filename
+
+    return db_file
+
+
+
 def create_directory(dir_name, delete_if_exists=False):
     """
     Utility method for creating directory
