@@ -23,6 +23,8 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.PrintStream;
 import java.util.Enumeration;
+import java.util.LinkedList;
+import java.util.List;
 import java.util.MissingResourceException;
 import java.util.Properties;
 import java.util.regex.Matcher;
@@ -38,9 +40,9 @@ import java.util.regex.Pattern;
  * @author Yong Zhao
  * @author Karan Vahi
  * @author Mats Rynge
- * @version $Revision: 4651 $
+ * @version $Revision$
  * */
-public class CommonProperties
+public class CommonProperties implements Cloneable
 {
     /**
      * implements the singleton access via class variable.
@@ -428,8 +430,10 @@ public class CommonProperties
     {
         //set in internal properties object also
         //else prefix option does not work. Karan Oct 1, 2008
-        this.m_props.setProperty( key, value );
-        return System.setProperty( key, value );
+        return this.m_props.setProperty( key, value );
+        //we don't set System properties, else it makes the clone method
+        //unusable
+        //return System.setProperty( key, value );
     }
 
     /**
@@ -483,19 +487,31 @@ public class CommonProperties
         // Remark 2: We need to give priority to System properties. This is done
         // automatically by calling this class's getProperty method.
         String key;
-        for ( Enumeration e = propertyNames(); e.hasMoreElements(); ) {
-            key = (String) e.nextElement();
 
-            if ( keepPrefix ) {
-                // keep full prefix in result, also copy direct matches
-                if ( key.startsWith(prefixMatch) || key.equals(prefixSelf) )
-                    result.setProperty( key,
-                            getProperty(key) );
-            } else {
-                // remove full prefix in result, dont copy direct matches
-                if ( key.startsWith(prefixMatch) )
-                    result.setProperty( key.substring( prefixMatch.length() ),
-                            getProperty(key) );
+        List<Enumeration> enumerations = new LinkedList();
+
+        //take cares of only the properties specified in the properties file
+        enumerations.add( propertyNames() );
+        //user may have specified profiles as properties in the system
+        //fix for PM-581
+        enumerations.add( System.getProperties().propertyNames() );
+
+        for( Enumeration e: enumerations ){
+            while( e.hasMoreElements() ){
+                key = (String) e.nextElement();
+
+                if ( keepPrefix ) {
+                    // keep full prefix in result, also copy direct matches
+                    if ( key.startsWith(prefixMatch) || key.equals(prefixSelf) )
+                        result.setProperty( key,
+                                getProperty(key) );
+                } else {
+                    // remove full prefix in result, dont copy direct matches
+                    if ( key.startsWith(prefixMatch) )
+                        result.setProperty( key.substring( prefixMatch.length() ),
+                                getProperty(key) );
+                        
+                }
             }
         }
 
@@ -589,4 +605,32 @@ public class CommonProperties
 
         cp.list( System.out );
     }
+
+    /**
+     * Returns the clone of the object.
+     *
+     * @return the clone
+     */
+    public Object clone(){
+        CommonProperties props;
+        try{
+
+            //this will do a shallow clone for all member variables
+            //that is fine for the string variables
+            props = ( CommonProperties ) super.clone();
+
+            //do a deep clone for the underlying properties
+            //can be expensive
+            props.m_props  = new Properties();
+            for( Object key: this.m_props.keySet() ){
+                props.m_props.put(key, this.m_props.get(key));
+            }
+        }
+        catch( CloneNotSupportedException e ){
+            //somewhere in the hierarch chain clone is not implemented
+            throw new RuntimeException("Clone not implemented in the base class of " + this.getClass().getName(),
+                                       e );
+        }
+        return props;
+    }
 }
