@@ -49,7 +49,7 @@ import java.io.IOException;
  * This data class describes a site in the site catalog.
  *
  * @author Karan Vahi
- * @version $Revision: 5078 $
+ * @version $Revision$
  */
 public class SiteCatalogEntry extends AbstractSiteData{
 
@@ -87,13 +87,18 @@ public class SiteCatalogEntry extends AbstractSiteData{
     /**
      * The handle to the head node filesystem.
      */
-    private HeadNodeFS mHeadFS;
+//    private HeadNodeFS mHeadFS;
     
     /**
      * The handle to the worker node filesystem.
      */
-    private WorkerNodeFS mWorkerFS;
-    
+//    private WorkerNodeFS mWorkerFS;
+
+    /**
+     * A Map of different directories indexed by Directory.TYPE associated
+     * with the site catalog entry
+     */
+    private Map<Directory.TYPE, Directory> mDirectories;
     
     /**
      * Map of grid gateways at the site for submitting different job types.
@@ -156,6 +161,7 @@ public class SiteCatalogEntry extends AbstractSiteData{
     public void initialize( String id ){        
         mID       = id;
         mSysInfo  = new SysInfo();
+        mDirectories = new HashMap<Directory.TYPE, Directory>();
         mProfiles        = new Profiles();
         mGridGateways    = new HashMap();
         mReplicaCatalogs = new LinkedList();
@@ -315,54 +321,204 @@ public class SiteCatalogEntry extends AbstractSiteData{
     public String getGlibc( ){
         return mSysInfo.getGlibc();
     }
-    
+
     /**
-     * Sets the headnode filesystem.
-     * 
-     * @param system   the head node filesystem.
+     * Adds a directory to the site catalog entry object.
+     * Adding a directory automatically will add a head node filesystem
+     * and a worker node filesystem to the SiteCatalogEntry if it does not exist
+     * already.
+     * The mapping followed is
+     * <pre>
+     *  shared-scratch  ->  HeadNodeFS shared scratch
+     *  shared-storage  ->  HeadNodeFS shared storage
+     *  local-scratch   ->  WorkerNodeFS local scratch
+     *  local-storage   ->  HeadNodeFS local storage
+     * </pre>
+     *
+     * @param directory  the directory to be added
+     *//*
+    public void addDirectory( Directory directory ) {
+        Directory.TYPE type = directory.getType();
+
+        if( type == Directory.TYPE.shared_scratch ){
+            HeadNodeFS headnode = this.getHeadNodeFS();
+            if( headnode == null ){
+                headnode = new HeadNodeFS();
+                HeadNodeScratch scratch = new HeadNodeScratch();
+                scratch.setSharedDirectory( directory );
+                headnode.setScratch( scratch );
+                this.setHeadNodeFS( headnode );
+            }
+            else{
+                //retrive from existing
+                HeadNodeScratch scratch = headnode.getScratch();
+                if( scratch == null ){
+                    scratch = new HeadNodeScratch();
+                    headnode.setScratch(scratch);
+                }
+                //get the shared filesystem
+               scratch.setSharedDirectory( directory );
+            }
+        }
+        else if( type == Directory.TYPE.shared_storage ){
+            HeadNodeFS headnode = this.getHeadNodeFS();
+            if( headnode == null ){
+                headnode = new HeadNodeFS();
+                HeadNodeStorage storage = new HeadNodeStorage();
+                storage.setSharedDirectory( directory );
+                headnode.setStorage( storage );
+                this.setHeadNodeFS( headnode );
+            }
+            else{
+                //retrieve from existing
+                HeadNodeStorage storage = headnode.getStorage();
+                if( storage == null ){
+                    storage = new HeadNodeStorage();
+                    headnode.setStorage( storage );
+                }
+                //set the shared filesystem
+               storage.setSharedDirectory( directory );
+            }
+        }
+        else if( type == Directory.TYPE.local_scratch ){
+            WorkerNodeFS workernode = this.getWorkerNodeFS();
+            if( workernode == null ){
+                workernode = new WorkerNodeFS();
+                WorkerNodeScratch scratch = new WorkerNodeScratch();
+                scratch.setLocalDirectory( directory );
+                workernode.setScratch( scratch );
+                this.setWorkerNodeFS( workernode );
+            }
+            else{
+                //retrieve from existing
+                WorkerNodeScratch scratch = workernode.getScratch();
+                if( scratch == null ){
+                    scratch = new WorkerNodeScratch();
+                    workernode.setScratch(scratch);
+                }
+                //set the shared filesystem
+               scratch.setLocalDirectory( directory );
+            }
+        }
+        /// we now map HeadNode local storage
+        else if( type == Directory.TYPE.local_storage ){
+            WorkerNodeFS workernode = this.getWorkerNodeFS();
+            if( workernode == null ){
+                workernode = new WorkerNodeFS();
+                WorkerNodeStorage storage = new WorkerNodeStorage();
+                storage.setLocalDirectory( directory );
+                workernode.setStorage( storage );
+                this.setWorkerNodeFS( workernode );
+            }
+            else{
+                //retrieve from existing
+                WorkerNodeStorage storage = workernode.getStorage();
+                if( storage == null ){
+                    storage = new WorkerNodeStorage();
+                    workernode.setStorage( storage );
+                }
+                //set the shared filesystem
+               storage.setLocalDirectory( directory );
+            }
+        }
+         
+        else if( type == Directory.TYPE.local_storage ){
+            HeadNodeFS headnode = this.getHeadNodeFS();
+            if( headnode == null ){
+                headnode = new HeadNodeFS();
+                HeadNodeStorage storage = new HeadNodeStorage();
+                storage.setLocalDirectory( directory );
+                headnode.setStorage( storage );
+                this.setHeadNodeFS( headnode );
+            }
+            else{
+                //retrieve from existing
+                HeadNodeStorage storage = headnode.getStorage();
+                if( storage == null ){
+                    storage = new HeadNodeStorage();
+                    headnode.setStorage( storage );
+                }
+                //set the shared filesystem
+               storage.setLocalDirectory( directory );
+            }
+        }
+
+    }*/
+
+    /**
+     * Adds a directory internally. Complains if directory of same type already
+     * exists
+     *
+     * @param directory   the directory to be added.
      */
-    public void setHeadNodeFS( HeadNodeFS system ){
-        mHeadFS = system;
+    public void addDirectory(Directory directory) {
+        //check for existence
+        if( this.mDirectories.containsKey( directory.getType() ) ){
+            StringBuffer error = new StringBuffer();
+            error.append( "Unable to add Directory " ).append( directory.getInternalMountPoint() )
+                 .append( " for site " ).append(  this.getSiteHandle() );
+            throw new RuntimeException( error.toString() );
+
+        }
+
+        this.mDirectories.put( directory.getType(), directory);
     }
-    
-    
+
     /**
-     * Returns the headnode filesystem.
-     * 
-     * @return   the head node filesystem.
+     * Sets a directory corresponding to a particular type
+     *
+     * @param directory the directory to be set
      */
-    public HeadNodeFS getHeadNodeFS(  ){
-        return mHeadFS;
+    public void setDirectory( Directory directory ){
+         this.mDirectories.put( directory.getType(), directory );
+    }
+
+    /**
+     * Returns a directory corresponding to a particular type
+     *
+     * @return the iterator
+     */
+    public Iterator<Directory> getDirectoryIterator(   ){
+        return this.mDirectories.values().iterator();
+    }
+
+    /**
+     * Returns a directory corresponding to a particular type
+     *
+     * @param the type the directory type
+     */
+    public Directory getDirectory( Directory.TYPE type ){
+        return this.mDirectories.get( type );
     }
     
     /**
-     * Sets the worker node filesystem.
+     * Returns the local-storage directory.
+     * If it is not specified, then returns shared-storage
+     * If none is associated, then returns null
      * 
-     * @param system   the head node filesystem.
+     * @return  the appropriate  directory
      */
-    public void setWorkerNodeFS( WorkerNodeFS system ){
-        mWorkerFS = system;
-    }
-    
-    
-    /**
-     * Returns the worker node filesystem.
-     * 
-     * @return   the worker node filesystem.
-     */
-    public WorkerNodeFS getWorkerNodeFS(  ){
-        return mWorkerFS;
+    public Directory getHeadNodeStorageDirectory() {
+        Directory result = this.getDirectory( Directory.TYPE.local_storage );
+        if( result == null ){
+            result = this.getDirectory(Directory.TYPE.shared_storage );
+        }
+        return result;
     }
+
+   
     
     /**
      * Returns the work directory for the compute jobs on a site. 
      * 
      * Currently, the work directory is picked up from the head node shared filesystem.
      * 
-     * @return the internal mount point.
+     * @return the internal mount point, else null
      */
     public String getInternalMountPointOfWorkDirectory() {
-        return this.getHeadNodeFS().getScratch().getSharedDirectory().getInternalMountPoint().getMountPoint();
+        Directory dir = this.getDirectory( Directory.TYPE.shared_scratch );
+        return ( dir == null ) ? null : dir.getInternalMountPoint().getMountPoint();
+
     }
     
     /**
@@ -514,6 +670,52 @@ public class SiteCatalogEntry extends AbstractSiteData{
         return g;
     }
     
+    
+    /**
+     * A convenience method to select the URL Prefix for the FileServer for 
+     * the shared scratch space on the HeadNode matching a particular operation.
+     *
+     * For get and put operations, the results default back to searching for an
+     * ALL operation server.
+     *
+     *
+     * @param operation  the operation for which the file server is required
+     * 
+     * @return  URL Prefix for the FileServer for the shared scratch space , else null
+     * @deprecated should be removed
+     */
+    public String selectHeadNodeScratchSharedFileServerURLPrefix( FileServer.OPERATION operation ){
+        FileServer server = this.selectHeadNodeScratchSharedFileServer( operation );
+        return ( server == null )?
+               null:
+               server.getURLPrefix();
+    }
+    
+    /**
+     * A convenience method to select the FileServer for the shared scratch
+     * space on the HeadNode.
+     *
+     * For get and put operations, the results default back to searching for an
+     * ALL operation server.
+     *
+     *
+     * @param operation  the operation for which the file server is required
+     * 
+     * @return  FileServer for the shared scratch space , else null
+     */
+    public FileServer selectHeadNodeScratchSharedFileServer( FileServer.OPERATION operation ){
+        Directory dir = this.getDirectory( Directory.TYPE.shared_scratch );
+
+        //sanity check
+        if( dir == null ){
+            return null;
+        }
+
+        return dir.selectFileServer( operation );
+    }
+    
+    
+    
     /**
      * A convenience method that selects a file server for staging the data out to 
      * a site. It returns the file server to which the generated data is staged
@@ -521,12 +723,26 @@ public class SiteCatalogEntry extends AbstractSiteData{
      * 
      * The <code>FileServer</code> selected is associated with the HeadNode Filesystem.
      * 
+     * For get and put operations, the results default back to searching for an
+     * ALL server.
+     * 
+     * 
+     * @param operation  the operation for which the file server is required
+     *
      * @return the <code>FileServer</code> else null.
      */
-    public FileServer selectStorageFileServerForStageout(){
-        return ( this.getHeadNodeFS() == null )?
-               null:
-               this.getHeadNodeFS().selectStorageFileServerForStageout();
+    public FileServer selectStorageFileServerForStageout( FileServer.OPERATION operation  ){
+        Directory dir = this.getDirectory( Directory.TYPE.local_storage );
+        if( dir == null ){
+            dir = this.getDirectory( Directory.TYPE.shared_storage );
+        }
+
+        //sanity check
+        if( dir == null ){
+            return null;
+        }
+
+        return dir.selectFileServer( operation );
     }
     
     /**
@@ -642,16 +858,10 @@ public class SiteCatalogEntry extends AbstractSiteData{
         for( Iterator<GridGateway> it = this.getGridGatewayIterator(); it.hasNext(); ){
             it.next().toXML( writer, newIndent );
         }
-        
-        HeadNodeFS fs = null;
-        if( (fs = this.getHeadNodeFS()) != null ){
-            fs.toXML( writer, newIndent );
-        }
-        
-        
-        WorkerNodeFS wfs = null;
-        if( ( wfs = this.getWorkerNodeFS() ) != null ){
-            wfs.toXML( writer, newIndent );
+
+        //list all the directories
+        for( Directory directory : this.mDirectories.values() ){
+            directory.toXML(writer, newIndent);
         }
         
         //list all the replica catalogs associate
@@ -682,16 +892,11 @@ public class SiteCatalogEntry extends AbstractSiteData{
             for( Iterator<GridGateway> it = this.getGridGatewayIterator(); it.hasNext(); ){
                 obj.addGridGateway( (GridGateway)it.next().clone() );
             }   
-        
-            HeadNodeFS fs = null;
-            if( (fs = this.getHeadNodeFS()) != null ){
-                obj.setHeadNodeFS( (HeadNodeFS)fs.clone() );
-            }
-        
-            WorkerNodeFS wfs = null;
-            if( ( wfs = this.getWorkerNodeFS() ) != null ){
-                obj.setWorkerNodeFS( (WorkerNodeFS)wfs.clone() );
+
+            for( Directory directory : this.mDirectories.values() ){
+                obj.setDirectory( (Directory)directory.clone() );
             }
+            
         
             //list all the replica catalogs associate
             for( Iterator<ReplicaCatalog> it = this.getReplicaCatalogIterator(); it.hasNext(); ){
@@ -710,6 +915,39 @@ public class SiteCatalogEntry extends AbstractSiteData{
         return obj;
     }
 
+    /**
+     * Accept method for the visitor interface
+     *
+     * @param visitor   the visitor
+     *
+     * @throws  IOException  in case of error
+     */
+    public void accept(SiteDataVisitor visitor) throws IOException {
+
+        visitor.visit( this );
+
+       //list all the gridgateways
+        for( Iterator<GridGateway> it = this.getGridGatewayIterator(); it.hasNext(); ){
+            it.next().accept(visitor);
+        }
+
+        for( Directory directory: this.mDirectories.values() ){
+            directory.accept( visitor );
+        }
+        
+        //list all the replica catalogs associate
+        for( Iterator<ReplicaCatalog> it = this.getReplicaCatalogIterator(); it.hasNext(); ){
+            it.next().accept(visitor);
+        }
+
+        //profiles are handled in the depart method
+        visitor.depart(this);
+    }
+
+
+
+
+
    
     
     
