@@ -28,37 +28,24 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import edu.isi.pegasus.common.util.Currently;
 import edu.isi.pegasus.planner.catalog.classes.SysInfo;
 import edu.isi.pegasus.planner.catalog.transformation.classes.VDSSysInfo;
 import edu.isi.pegasus.planner.classes.Job;
 import edu.isi.pegasus.planner.classes.PlannerOptions;
 import edu.isi.pegasus.planner.common.PegasusProperties;
 import edu.isi.pegasus.planner.namespace.Pegasus;
+import edu.isi.pegasus.planner.transfer.mapper.OutputMapperFactory;
 
 /**
  * The site store contains the collection of sites backed by a HashMap.
  * 
  * @author Karan Vahi
  * @author Mats Rynge
- * @version $Revision: 4915 $
+ * @version $Revision$
  */
 public class SiteStore extends AbstractSiteData{
     
-    /**
-     * The "official" namespace URI of the site catalog schema.
-     */
-    public static final String SCHEMA_NAMESPACE = "http://pegasus.isi.edu/schema/sitecatalog";
-
-    /**
-     * The "not-so-official" location URL of the DAX schema definition.
-     */
-    public static final String SCHEMA_LOCATION = "http://pegasus.isi.edu/schema/sc-3.0.xsd";
-
-    /**
-     * The version to report.
-     */
-    public static final String SCHEMA_VERSION = "3.0";
+    
     
     /**
      * The internal map that maps a site catalog entry to the site handle.
@@ -70,17 +57,8 @@ public class SiteStore extends AbstractSiteData{
      */
     private String mWorkDir;
     private PlannerOptions mPlannerOptions;
-    
-    
-    /**
-     * This contains the storage directory relative to the se mount point of the
-     * pool. It is populated from the pegasus.dir.storage property from the properties
-     * file. If not specified then the storage directory is the se mount point
-     * from the pool.config file.
-     */
-    protected String mStorageDir;
-
 
+    
     /**
      * A boolean indicating whether to have a deep directory structure for
      * the storage directory or not.
@@ -94,9 +72,6 @@ public class SiteStore extends AbstractSiteData{
         initialize();
     }
 
-    
-    
-    
     /**
      * The intialize method.
      */
@@ -112,9 +87,10 @@ public class SiteStore extends AbstractSiteData{
      */
     public void setForPlannerUse( PegasusProperties properties, PlannerOptions options ){
         mPlannerOptions = options;
-        mWorkDir              = properties.getExecDirectory();
-        mStorageDir           = properties.getStorageDirectory();        
-        mDeepStorageStructure = properties.useDeepStorageDirectoryStructure();
+        mWorkDir              = properties.getExecDirectory();  
+        mDeepStorageStructure = properties.useDeepStorageDirectoryStructure() ;
+                                //||hashedOutputMapperUsed( properties );
+                                  
     }
     
     /**
@@ -359,10 +335,11 @@ public class SiteStore extends AbstractSiteData{
      * mount point ).
      *
      * @param siteHandle    the site handle.
+     * @param operation     the operation for which we need the server.
      *
      * @return the url
      */
-    public String getExternalWorkDirectoryURL( String siteHandle ){
+    public String getExternalWorkDirectoryURL( String siteHandle,  FileServer.OPERATION operation ){
         String url = null;
 
         SiteCatalogEntry site = this.lookup( siteHandle );
@@ -371,9 +348,28 @@ public class SiteStore extends AbstractSiteData{
         }
 
         //select a file server
-        FileServer fs = site.getHeadNodeFS().selectScratchSharedFileServer();
+        FileServer fs = site.selectHeadNodeScratchSharedFileServer( operation );
 
-        url = fs.getURLPrefix() + this.getExternalWorkDirectory( fs, siteHandle );
+        return this.getExternalWorkDirectoryURL( fs, siteHandle );
+    }
+
+    /**
+     * Returns a  URL to the work directory as seen externally ( including external
+     * mount point ).
+     *
+     * @param server        the FileServer to use
+     * @param siteHandle    the site handle.
+     *
+     * @return the url else null
+     */
+    public String getExternalWorkDirectoryURL( FileServer server, String siteHandle ){
+        String url = null;
+
+        if( server == null ){
+            return null;
+        }
+
+        url = server.getURLPrefix() + this.getExternalWorkDirectory( server, siteHandle );
 
         return url;
     }
@@ -382,7 +378,7 @@ public class SiteStore extends AbstractSiteData{
      * Return the work directory as seen externally (including external mount point)
      *
      * @param fs          the FileServer with the file system
-     * @param siteHanlde  the site for which you want the directory
+     * @param siteHandle  the site for which you want the directory
      *
      * @return    String corresponding to the mount point
      */    
@@ -430,6 +426,36 @@ public class SiteStore extends AbstractSiteData{
         return path.toString();
     }
 
+    /**
+     * Return the relative directory that needs to be appended to the storage
+     * directory for the workflow.
+     *
+     *
+     * @return    String corresponding to the mount point if the pool is found.
+     *            null if pool entry is not found.
+     * 
+     */
+    public String getRelativeStorageDirectoryAddon(  ) {
+        
+        String mount_point = "";
+        //check if we need to replicate the submit directory
+        //structure on the storage directory
+        if( mDeepStorageStructure ){
+            String leaf = ( this.mPlannerOptions.partOfDeferredRun() )?
+                             //if a deferred run then pick up the relative random directory
+                             //this.mUserOpts.getOptions().getRandomDir():
+                             this.mPlannerOptions.getRelativeDirectory():
+                             //for a normal run add the relative submit directory
+                             this.mPlannerOptions.getRelativeDirectory();
+            File f = new File( mount_point, leaf );
+            mount_point = f.getAbsolutePath();
+        }
+
+
+        return mount_point;
+
+    }
+ 
      
     /**
      * Return the storage mount point for a particular pool.
@@ -438,7 +464,10 @@ public class SiteStore extends AbstractSiteData{
      *
      * @return    String corresponding to the mount point if the pool is found.
      *            null if pool entry is not found.
+     * 
+     * @deprecated
      */
+/*
     public String getExternalStorageDirectory( String site ) {
         
         String mount_point = mStorageDir;
@@ -451,7 +480,7 @@ public class SiteStore extends AbstractSiteData{
         
         FileServer server = null;
         if ( mStorageDir.length() == 0 || mStorageDir.charAt( 0 ) != '/' ) {
-            server = entry.selectStorageFileServerForStageout();
+            server = entry.selectStorageFileServerForStageout( FileServer.OPERATION.put );
             mount_point = server.getMountPoint();
 
             //removing the trailing slash if there
@@ -483,6 +512,7 @@ public class SiteStore extends AbstractSiteData{
         return mount_point;
 
     }
+ */ 
     
     /**
      * This determines the working directory on remote execution pool on the
@@ -641,9 +671,6 @@ public class SiteStore extends AbstractSiteData{
                  "";
         String newIndent = indent + "\t";
         
-        //write out the xml header first.
-        this.writeXMLHeader( writer, indent );
-        
         //iterate through all the entries and spit them out.
         for( Iterator<SiteCatalogEntry> it = this.entryIterator(); it.hasNext(); ){
             it.next().toXML( writer, newIndent );
@@ -655,63 +682,7 @@ public class SiteStore extends AbstractSiteData{
         writer.write( newLine );
     }
     
-    /**
-     * Writes the header of the XML output. The output contains the special
-     * strings to start an XML document, some comments, and the root element.
-     * The latter points to the XML schema via XML Instances.
-     *
-     * @param stream is a stream opened and ready for writing. This can also
-     *               be a string stream for efficient output.
-     * @param indent is a <code>String</code> of spaces used for pretty
-     *               printing. The initial amount of spaces should be an empty
-     *               string. The parameter is used internally for the recursive 
-     *               traversal.
-     * 
-     * @exception IOException if something fishy happens to the stream.
-     */
-    public void writeXMLHeader( Writer stream, String indent )
-                                                    throws IOException  {
-        
-        String newline = System.getProperty( "line.separator", "\r\n" );
-        indent = (indent != null && indent.length() > 0 ) ?
-                 indent:
-                 "";
-        
-        stream.write( indent );
-        stream.write( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" );
-        stream.write( newline );
-
-        stream.write( indent );        
-        stream.write( "<!-- generated: " );
-        stream.write( Currently.iso8601(false) );
-        stream.write( " -->" );
-        stream.write( newline );
-
-        // who generated this document
-        stream.write( indent );
-        stream.write( "<!-- generated by: " );
-        stream.write( System.getProperties().getProperty("user.name", "unknown") );
-        stream.write( " [" );
-        stream.write( System.getProperties().getProperty("user.region","??") );
-        stream.write( "] -->" );
-        stream.write( newline );
-
-        // root element with elementary attributes
-        stream.write( indent );
-        stream.write( '<' );
-        
-        stream.write( "sitecatalog xmlns" );       
-        stream.write( "=\"");
-        stream.write( SCHEMA_NAMESPACE );
-        stream.write( "\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"" );
-        stream.write( SCHEMA_NAMESPACE );
-        stream.write( ' ' );
-        stream.write( SCHEMA_LOCATION );
-        stream.write( '"' );
-        writeAttribute( stream, "version", SCHEMA_VERSION );
-        stream.write( '>' );
-        stream.write( newline );
-   }
+   
 
     /**
      * Returns the clone of the object.
@@ -737,6 +708,39 @@ public class SiteStore extends AbstractSiteData{
         return obj;
     }
 
+    /**
+     * Accept method for the SiteStore object
+     *
+     * @param visitor that goes through it
+     */
+    public void accept(SiteDataVisitor visitor) throws IOException {
+        visitor.visit( this );
+
+        for( Iterator<SiteCatalogEntry> it = this.entryIterator(); it.hasNext(); ){
+            it.next().accept(visitor);
+        }
+
+        visitor.depart( this );
+    }
+
+    /**
+     * Returns a boolean indicating whether the Hashed Output Mapper was used or
+     * not
+     * 
+     * @param properties
+     * 
+     * @return boolean 
+     */
+    private boolean hashedOutputMapperUsed(PegasusProperties properties) {
+        
+        String mapper = properties.getProperty( OutputMapperFactory.PROPERTY_KEY );
+        if( mapper == null ){
+            mapper = OutputMapperFactory.DEFAULT_OUTPUT_MAPPER_IMPLEMENTATION;
+        }
+        
+        return mapper.equals( OutputMapperFactory.HASHED_OUTPUT_MAPPER_IMPLEMENTATION );
+    }
+
 
 
 
