@@ -20,7 +20,7 @@ package edu.isi.pegasus.planner.catalog.transformation.impl;
  * This is the database implementation for the TC.
  *
  * @author Gaurang Mehta gmehta@isi.edu
- * @version $Revision: 4090 $
+ * @version $Revision$
  */
 
 import java.io.IOException;
@@ -1766,7 +1766,7 @@ if(!write) return false;
      * @return String
      */
     public String getDescription() {
-        return new String(PegasusProperties.nonSingletonInstance().getTCMode());
+        return PegasusProperties.nonSingletonInstance().getTCMode();
     }
 
     public void close() {
