@@ -39,7 +39,7 @@ import java.util.Map;
  *
  * @author Karan Vahi
  * @author Gaurang Mehta
- * @version $Revision: 4797 $
+ * @version $Revision$
  */
 public class PlannerOptions extends Data implements Cloneable{
     
@@ -53,7 +53,11 @@ public class PlannerOptions extends Data implements Cloneable{
      */
     public static final int DEFAULT_NUMBER_OF_RESCUE_TRIES = 999;
 
-
+    /**
+     * The various cleanup options supported by the planner
+     */
+    public enum CLEANUP_OPTIONS{ none, inplace, leaf};
+    
     /**
      * The base submit directory.
      */
@@ -125,9 +129,9 @@ public class PlannerOptions extends Data implements Cloneable{
     private boolean mForce;
 
     /**
-     * A boolean indicating whether to do cleanup or not.
+     * An enum tracking what type of cleanup needs to be done.
      */
-    private boolean mCleanup;
+    private CLEANUP_OPTIONS mCleanup;
 
     /**
      * To Display help or not.
@@ -198,12 +202,6 @@ public class PlannerOptions extends Data implements Cloneable{
      */
     private boolean mDeferredRun;
 
-    /**
-     * Boolean indicating whether to spawn off a monitoring process or not
-     * for the workflow.
-     */
-    private boolean mMonitor;
-
     /**
      * The VOGroup to which the user belongs to.
      */
@@ -259,6 +257,29 @@ public class PlannerOptions extends Data implements Cloneable{
      * A map that maps an execution site to a staging site.
      */
     private Map<String,String> mStagingSitesMap;
+    
+    /**
+     * the path to the shiwa bundle
+     */
+    private String mShiwaBundle;
+    
+    /**
+     * the input directory 
+     */
+    private String mInputDir;
+    
+    
+    /**
+     * the output directory 
+     */
+    private String mOutputDir;
+    
+     
+    /**
+     * The conf option passed to the planner pointing to the properties file.
+     */
+    private String mConfFile;
+    
 
     /**
      * Default Constructor.
@@ -288,8 +309,7 @@ public class PlannerOptions extends Data implements Cloneable{
         mVDSProps         = null;
         mClusterer        = null;
         mBasenamePrefix   = null;
-        mMonitor          = false;
-        mCleanup          = true;
+        mCleanup          = CLEANUP_OPTIONS.none;
         mVOGroup          = "pegasus";
         mDeferredRun      = false;
         mDate             = new Date();
@@ -301,6 +321,10 @@ public class PlannerOptions extends Data implements Cloneable{
         mForceReplan      = false;
         mOriginalArgumentString = null;
         mStagingSitesMap  = new HashMap<String,String>();
+        mShiwaBundle      = null;
+        mInputDir         = null;
+        mOutputDir        = null;
+        mConfFile         = null;
     }
 
     /**
@@ -411,9 +435,9 @@ public class PlannerOptions extends Data implements Cloneable{
     /**
      * Returns the option indicating whether to do cleanup or not.
      *
-     * @return the boolean value indicating the cleanup option.
+     * @return the cleanup strategy to be used 
      */
-    public boolean getCleanup(){
+    public CLEANUP_OPTIONS getCleanup(){
         return mCleanup;
     }
 
@@ -448,6 +472,15 @@ public class PlannerOptions extends Data implements Cloneable{
         return mDisplayHelp;
     }
 
+    /**
+     * Returns the Shiwa Bundle set
+     * 
+     * @return the shiwa bundle specified by the user
+     */
+    public String getShiwaBundle() {
+        return this.mShiwaBundle;
+    }
+
     /**
      * Increments the logging level by 1.
      */
@@ -481,6 +514,24 @@ public class PlannerOptions extends Data implements Cloneable{
         return this.mMegadag;
     }
 
+    /**
+     * Returns the input directory.
+     * 
+     * @return the input directory for the workflow
+     */
+    public String getInputDirectory(){
+        return this.mInputDir;
+    }
+    
+    /**
+     * Returns the output directory
+     * 
+     * @return the output directory for the workflow
+     */
+    public String getOutputDirectory(){
+        return this.mOutputDir;
+    }
+    
     /**
      * Returns the output site where to stage the output .
      *
@@ -568,6 +619,15 @@ public class PlannerOptions extends Data implements Cloneable{
         mProperties.setProperty( args[0], args[1] );
         
     }
+
+    /**
+     * Sets the path to the Shiwa Bundle to be used.
+     * 
+     * @param bundle    the shiwa bundle
+     */
+    public void setShiwaBundle(String bundle ) {
+        this.mShiwaBundle = bundle;
+    }
     
     /**
      * Returns whether to submit the workflow or not.
@@ -668,6 +728,16 @@ public class PlannerOptions extends Data implements Cloneable{
 
     }
 
+    /**
+     * Returns the property file pointed to by the --conf option passed to the 
+     * planner
+     * 
+     * @return the conf option if passed, else null 
+     */
+    public String getConfFile(){
+        return mConfFile;
+    }
+    
     /**
      * Sets the authenticate flag to the value passed.
      *
@@ -719,24 +789,6 @@ public class PlannerOptions extends Data implements Cloneable{
     }
 
 
-    /**
-     * Sets the flag to denote whether we want to monitor the workflow or not.
-     *
-     * @param value boolean.
-     */
-    public void setMonitoring(boolean value){
-        this.mMonitor = value;
-    }
-
-    /**
-     * Returns boolean indicating whether we want to monitor or not.
-     *
-     * @return boolean indicating whether monitoring was set or not.
-     */
-    public boolean monitorWorkflow(){
-        return this.mMonitor;
-    }
-
     /**
      * Sets the partitioning type in case of partition and plan.
      *
@@ -951,10 +1003,19 @@ public class PlannerOptions extends Data implements Cloneable{
     /**
      * Sets the cleanup option for the planner.
      *
-     * @param cleanup  boolean value.
+     * @param cleanup  the cleanup option
      */
-    public void setCleanup( boolean cleanup ){
-        mCleanup = cleanup;
+    public void setCleanup( String cleanup ){
+        this.setCleanup( CLEANUP_OPTIONS.valueOf(cleanup) );
+    }
+    
+    /**
+     * Sets the cleanup option for the planner.
+     *
+     * @param cleanup  the cleanup option
+     */
+    public void setCleanup( CLEANUP_OPTIONS cleanup ){
+        mCleanup =  cleanup;
     }
 
 
@@ -1001,6 +1062,26 @@ public class PlannerOptions extends Data implements Cloneable{
         mPDAXFile = pdax;
     }
 
+    /**
+     * Set the input directory.
+     * 
+     * @param input the input directory for the workflow
+     */
+    public void setInputDirectory( String input ){
+        this.mInputDir = sanitizePath( input );
+    }
+    
+    
+    /**
+     * Set the output directory.
+     * 
+     * @param output the input directory for the workflow
+     */
+    public void setOutputDirectory( String output ){
+        this.mOutputDir =  output ;
+    }
+    
+    
     /**
      * Sets the output site specified by the user.
      *
@@ -1265,6 +1346,16 @@ public class PlannerOptions extends Data implements Cloneable{
     }
 
     
+    /**
+     * Sets the property file pointed to by the --conf option passed to the 
+     * planner
+     * 
+     * @param conf  the conf option if passed, else null 
+     */
+    public void setConfFile( String conf ){
+        mConfFile = conf;
+    }
+    
     /**
      * Returns the textual description of all the options that were set for
      * the planner.
@@ -1285,7 +1376,9 @@ public class PlannerOptions extends Data implements Cloneable{
                     "\n Staging Sites        " + this.stagingSiteMappingToString() +
                     "\n Cache Files          " + this.setToString(mCacheFiles,",") +
                     "\n Inherited RC Files   " + this.setToString(mInheritedRCFiles,",") +
-                    "\n Output Pool          " + mOutputPool +
+                    "\n Input Directory      " + this.mInputDir + 
+                    "\n Output Directory     " + this.mOutputDir +
+                    "\n Output Site          " + mOutputPool +
                     "\n Submit to CondorG    " + mSubmit +
                     "\n Display Help         " + mDisplayHelp +
                     "\n Logging Level        " + mLoggingLevel +
@@ -1296,7 +1389,7 @@ public class PlannerOptions extends Data implements Cloneable{
                     "\n Random Direct Name   " + mRandomDirName +
                     "\n Authenticate         " + mAuthenticate +
                     "\n Clustering Technique " + mClusterer +
-                    "\n Monitor Workflow     " + mMonitor +
+                    "\n Cleanup    "           + mCleanup +
                     "\n VO Group             " + mVOGroup +
                     "\n Rescue Tries         " + mNumOfRescueTries +
                     "\n VDS Properties       " + mVDSProps + 
@@ -1313,6 +1406,11 @@ public class PlannerOptions extends Data implements Cloneable{
      */
     public String toOptions(){
         StringBuffer sb = new StringBuffer();
+        
+        //the conf option if set always has to be the first one!
+        if( mConfFile != null ){
+            sb.append( " --conf " ).append( mConfFile );
+        }
 
         //the submit file dir
 //        if( mSubmitFileDir != null){ sb.append(" --dir ").append(mSubmitFileDir);}
@@ -1344,6 +1442,11 @@ public class PlannerOptions extends Data implements Cloneable{
             sb.append( " --staging-site " ).
                append( this.stagingSiteMappingToString() );
         }
+        
+        if( this.mShiwaBundle != null ){
+            sb.append( " --shiwa-bundle " ).
+               append( this.mShiwaBundle );
+        }
 
         //cache files
         if(!mCacheFiles.isEmpty()){
@@ -1358,8 +1461,17 @@ public class PlannerOptions extends Data implements Cloneable{
         //collapse option
         if( mClusterer != null ){ sb.append(" --cluster ").append(mClusterer);}
 
-        //specify the output pool
-        if(mOutputPool != null){ sb.append(" --output ").append(mOutputPool);}
+        //specify the input directory
+        if( this.mInputDir != null ){
+            sb.append(" --input-dir ").append( this.mInputDir );
+        }
+
+        //specify the output directory
+        if( this.mOutputDir != null ){
+            sb.append(" --output-dir ").append( this.mOutputDir );
+        }
+        //specify the output site
+        if(mOutputPool != null){ sb.append(" --output-site ").append(mOutputPool);}
 
         //the condor submit option
         if(mSubmit){ sb.append(" --run "); }
@@ -1371,7 +1483,8 @@ public class PlannerOptions extends Data implements Cloneable{
         if( mForceReplan ){ sb.append( " --force-replan " ); }
 
         //the cleanup option
-        if( !mCleanup ){ sb.append(" --nocleanup "); }
+        sb.append( " --cleanup " ).append( mCleanup.name() );
+        //if( !mCleanup ){ sb.append(" --nocleanup "); }
 
 
         //the verbose option
@@ -1383,8 +1496,6 @@ public class PlannerOptions extends Data implements Cloneable{
             sb.append(" --quiet " );
         }
 
-        //the monitor option
-        if( mMonitor ) { sb.append(" --monitor "); }
 
         //the deferred run option
         if( mDeferredRun ) { sb.append( " --deferred "); }
@@ -1464,8 +1575,8 @@ public class PlannerOptions extends Data implements Cloneable{
     public String getCompleteOptions(){
         StringBuffer sb = new StringBuffer();
         sb./*append( this.toJVMOptions() ).append( " " ).*/
-           append( "--dax" ).append( " " ).append( this.getDAX() ).
-           append( this.toOptions() );
+           append( this.toOptions() ).
+           append( "--dax" ).append( " " ).append( this.getDAX() );
         
         return sb.toString();
     }
@@ -1517,6 +1628,8 @@ public class PlannerOptions extends Data implements Cloneable{
         pOpt.mCacheFiles     = cloneSet(this.mCacheFiles);
         pOpt.mInheritedRCFiles       = cloneSet(this.mInheritedRCFiles);
         pOpt.mNonStandardJavaOptions = cloneSet( this.mNonStandardJavaOptions );
+        pOpt.mInputDir       = this.mInputDir;
+        pOpt.mOutputDir      = this.mOutputDir;
         pOpt.mOutputPool     = this.mOutputPool;
         pOpt.mDisplayHelp    = this.mDisplayHelp;
         pOpt.mLoggingLevel   = this.mLoggingLevel;
@@ -1526,7 +1639,6 @@ public class PlannerOptions extends Data implements Cloneable{
         pOpt.mSubmit         = this.mSubmit;
         pOpt.mGenRandomDir   = this.mGenRandomDir;
         pOpt.mOptArg         = this.mOptArg;
-        pOpt.mMonitor        = this.mMonitor;
         pOpt.mRandomDirName  = this.mRandomDirName;
         pOpt.mAuthenticate   = this.mAuthenticate;
         pOpt.mClusterer      = this.mClusterer;
@@ -1534,10 +1646,12 @@ public class PlannerOptions extends Data implements Cloneable{
         pOpt.mJobPrefix      = this.mJobPrefix;
         pOpt.mVOGroup        = this.mVOGroup;
         pOpt.mDeferredRun    = this.mDeferredRun;
+        pOpt.mShiwaBundle    = this.mShiwaBundle;
         pOpt.mDate           = (Date)this.mDate.clone();
         pOpt.mPartitioningType = this.mPartitioningType;
         pOpt.mNumOfRescueTries = this.mNumOfRescueTries;
         pOpt.mOriginalArgumentString = this.mOriginalArgumentString;
+        pOpt.mConfFile       = this.mConfFile;
 
         //a shallow clone for forward options
         pOpt.mForwardOptions  = this.mForwardOptions;
