@@ -50,6 +50,7 @@ import java.util.List;
 import java.util.Set;
 import java.util.Iterator;
 import edu.isi.pegasus.common.util.Separator;
+import edu.isi.pegasus.planner.partitioner.graph.GraphNode;
 import java.util.HashSet;
 
 /**
@@ -57,7 +58,7 @@ import java.util.HashSet;
  * implementations can choose to extend.
  *
  * @author Karan Vahi vahi@isi.edu
- * @version $Revision: 4737 $
+ * @version $Revision$
  *
  */
 public abstract class Abstract implements JobAggregator {
@@ -362,8 +363,7 @@ public abstract class Abstract implements JobAggregator {
      */
     public void makeAbstractAggregatedJobConcrete( AggregatedJob job ){
 
-        String stdIn  = null;
-
+        
         //containers for the input and output
         //files of fat job. Set insures no duplication
         //The multiple transfer ensures no duplicate transfer of
@@ -373,68 +373,9 @@ public abstract class Abstract implements JobAggregator {
   
 //        mergedJob = enable( mergedJob, jobs  );
 
-        
-        Job firstJob = (Job)job.getConstituentJob( 0 );
-        try {
-            BufferedWriter writer;
-            stdIn = job.getID() + ".in";
-            writer = new BufferedWriter(new FileWriter(
-                                                       new File(mDirectory,stdIn)));
-
-            //traverse throught the jobs to determine input/output files
-            //and merge the profiles for the jobs
-            int taskid = 1;
-            for( Iterator it = job.constituentJobsIterator(); it.hasNext(); taskid++ ) {
-                Job constitutentJob = (Job) it.next();
 
 
-
-                //handle stdin
-                if( constitutentJob instanceof AggregatedJob ){
-                    //slurp in contents of it's stdin
-                    File file = new File ( mDirectory, job.getStdIn() );
-                    BufferedReader reader = new BufferedReader(
-                                                             new FileReader( file )
-                                                               );
-                    String line;
-                    while( (line = reader.readLine()) != null ){
-                        //ignore comment out lines
-                        if( line.startsWith( "#" ) ){
-                            continue;
-                        }
-                        writer.write( line );
-                        writer.write( "\n" );
-                        taskid++;
-                    }
-                    reader.close();
-                    //delete the previous stdin file
-                    file.delete();
-                }
-                else{
-                    //write out the argument string to the
-                    //stdin file for the fat job
-
-                    //genereate the comment string that has the 
-                    //taskid transformation derivation
-                    writer.write( getCommentString( constitutentJob, taskid ) + "\n" );
-
-                    // the arguments are no longer set as condor profiles
-                    // they are now set to the corresponding profiles in
-                    // the Condor Code Generator only.
-                    writer.write( constitutentJob.getRemoteExecutable()  + " " +
-                                   constitutentJob.getArguments() + "\n");
-                }
-            }
-
-            //closing the handle to the writer
-            writer.close();
-        }
-        catch(IOException e){
-            mLogger.log("While writing the stdIn file " + e.getMessage(),
-                        LogManager.ERROR_MESSAGE_LEVEL);
-            throw new RuntimeException( "While writing the stdIn file " + stdIn, e );
-        }
-
+        File stdIn = writeOutInputFileForJobAggregator( job );
         
         
 /*      JIRA PM-277
@@ -450,7 +391,7 @@ public abstract class Abstract implements JobAggregator {
 
         //stdin file is the file containing the arguments
         //for the jobs being collapsed
-        job.setStdIn( stdIn );
+        job.setStdIn( stdIn.getName() );
 
         //explicitly set stdout to null overriding any stdout
         //that might have been inherited in the clone operation.
@@ -458,9 +399,7 @@ public abstract class Abstract implements JobAggregator {
         job.setStdOut( "" );
         job.setStdErr( "" );
 
-        //set the arguments for the clustered job
-        job.setArguments( this.aggregatedJobArguments( job ) );
-
+        
         //get hold of one of the jobs and suck init's globus namespace
         //info into the the map.
 
@@ -493,6 +432,12 @@ public abstract class Abstract implements JobAggregator {
         //catalog.
         job.updateProfiles( mProps );
 
+        //set the arguments for the clustered job
+        //they are set in the end to ensure that profiles can
+        //be used to specify the argumetns
+        job.setArguments( this.aggregatedJobArguments( job ) );
+
+
         return ;
 
     }
@@ -929,4 +874,80 @@ public abstract class Abstract implements JobAggregator {
 
     }
 
+    /**
+     * Writes out the input file for the aggregated job
+     *
+     * @param job   the aggregated job
+     *
+     * @return path to the input file
+     */
+    protected File writeOutInputFileForJobAggregator(AggregatedJob job) {
+        File stdin = null;
+        try {
+            BufferedWriter writer;
+            String name = job.getID() + ".in";
+            stdin = new File(mDirectory, name );
+            writer = new BufferedWriter(new FileWriter( stdin ) );
+
+            //traverse throught the jobs to determine input/output files
+            //and merge the profiles for the jobs
+            int taskid = 1;
+            
+            for(  Iterator it = this.topologicalOrderingRequired() ?
+                            job.topologicalSortIterator()://we care about order
+                            job.nodeIterator();//dont care about order
+                                                it.hasNext(); taskid++ ) {
+                GraphNode node = ( GraphNode )it.next();
+                Job constitutentJob = (Job) node.getContent();
+
+                //handle stdin
+                if( constitutentJob instanceof AggregatedJob ){
+                    //slurp in contents of it's stdin
+                    File file = new File ( mDirectory, job.getStdIn() );
+                    BufferedReader reader = new BufferedReader(
+                                                             new FileReader( file )
+                                                               );
+                    String line;
+                    while( (line = reader.readLine()) != null ){
+                        //ignore comment out lines
+                        if( line.startsWith( "#" ) ){
+                            continue;
+                        }
+                        writer.write( line );
+                        writer.write( "\n" );
+                        taskid++;
+                    }
+                    reader.close();
+                    //delete the previous stdin file
+                    file.delete();
+                }
+                else{
+                    //write out the argument string to the
+                    //stdin file for the fat job
+
+                    //genereate the comment string that has the
+                    //taskid transformation derivation
+                    writer.write( getCommentString( constitutentJob, taskid ) + "\n" );
+
+                    // the arguments are no longer set as condor profiles
+                    // they are now set to the corresponding profiles in
+                    // the Condor Code Generator only.
+                    writer.write( constitutentJob.getRemoteExecutable()  + " " +
+                                   constitutentJob.getArguments() + "\n");
+                }
+            }
+
+            //closing the handle to the writer
+            writer.close();
+        }
+        catch(IOException e){
+            mLogger.log("While writing the stdIn file " + e.getMessage(),
+                        LogManager.ERROR_MESSAGE_LEVEL);
+            throw new RuntimeException( "While writing the stdIn file " + stdin, e );
+        }
+
+        return stdin;
+
+    }
+
 }
