@@ -17,8 +17,6 @@
 package edu.isi.pegasus.planner.code;
 
 import edu.isi.pegasus.planner.code.gridstart.*;
-import edu.isi.pegasus.planner.code.GridStart;
-import edu.isi.pegasus.planner.code.POSTScript;
 
 import edu.isi.pegasus.planner.classes.ADag;
 import edu.isi.pegasus.planner.classes.Job;
@@ -44,7 +42,7 @@ import edu.isi.pegasus.planner.classes.PegasusBag;
  *
  *
  * @author Karan Vahi
- * @version $Revision: 4627 $
+ * @version $Revision$
  */
 
 public class GridStartFactory {
@@ -64,7 +62,7 @@ public class GridStartFactory {
                                            "none",
                                            "seqexec"
                                           };
-
+    
     /**
      * The index in the constant arrays for NoGridStart.
      */
@@ -117,8 +115,6 @@ public class GridStartFactory {
         POSTSCRIPT_IMPLEMENTING_CLASS_TABLE = new HashMap( 8 );
         //not really the best way. should have avoided creating objects
         //but then too many constants everywhere.
-        associate( new ExitPOST() );
-        associate( new ExitCode() );
         associate( new UserPOSTScript() );
         associate( new NoPOSTScript() );
         associate( new NetloggerPostScript() );
@@ -273,29 +269,39 @@ public class GridStartFactory {
                 "GridStartFactory needs to be initialized first before using" );
         }
         GridStart gs = null;
-        if ( /*gridStartPath == null ||*/ job.isMPIJob()){
-            //return NoGridStart implementation
-            gs = (GridStart)this.gridStart( GRIDSTART_SHORT_NAMES[ NO_GRIDSTART_INDEX ] );
-        }
-        else{
-            //determine the short name of GridStart implementation
-            //on the basis of any profile associated or from the properties file
-            String shortName = ( job.vdsNS.containsKey( Pegasus.GRIDSTART_KEY) ) ?
-                                //pick the one associated in profiles
-                                ( String ) job.vdsNS.get( Pegasus.GRIDSTART_KEY ):
-                                //pick the one in the properties file
-                                mProps.getGridStart();
 
-            //try loading on the basis of short name from the cache
-            Object obj = this.gridStart( shortName );
-
-            if( obj == null ){
-                //load via reflection and register in the cache
-                obj = this.loadGridStart( mBag, mDAG, shortName );
-                this.registerGridStart( shortName, (GridStart)obj );
-            }
-            gs = (GridStart) obj;
+        if ( job.isMPIJob() && !(job instanceof AggregatedJob )){
+            
+            //for only MPI jobs that are not PMC, we associate exitcode postscript
+            //with the rotation of logs option  and explicity associate
+            //NoGridStart with them
+            job.vdsNS.construct(Pegasus.GRIDSTART_KEY, "None" );
+            
+            //no empty postscript but arguments to exitcode to add -r $RETURN
+            job.dagmanVariables.construct( Dagman.POST_SCRIPT_KEY,
+                                                  PegasusExitCode.SHORT_NAME );
+            job.dagmanVariables.construct( Dagman.POST_SCRIPT_ARGUMENTS_KEY,
+                                                  PegasusExitCode.POSTSCRIPT_ARGUMENTS_FOR_ONLY_ROTATING_LOG_FILE );
+            
         }
+
+        //determine the short name of GridStart implementation
+        //on the basis of any profile associated or from the properties file
+        String shortName = ( job.vdsNS.containsKey( Pegasus.GRIDSTART_KEY) ) ?
+                           //pick the one associated in profiles
+                           ( String ) job.vdsNS.get( Pegasus.GRIDSTART_KEY ):
+                           //pick the one in the properties file
+                           mProps.getGridStart();
+
+        //try loading on the basis of short name from the cache
+        Object obj = this.gridStart( shortName );
+        if( obj == null ){
+            //load via reflection and register in the cache
+            obj = this.loadGridStart( mBag, mDAG, shortName );
+            this.registerGridStart( shortName, (GridStart)obj );
+        }
+        gs = (GridStart) obj;
+
         return gs;
      }
 
