@@ -28,7 +28,6 @@ import java.util.Iterator;
 import java.util.Map;
 
 import edu.isi.pegasus.common.credential.CredentialHandler;
-import edu.isi.pegasus.common.credential.CredentialHandler.TYPE;
 import edu.isi.pegasus.common.credential.CredentialHandlerFactory;
 import edu.isi.pegasus.common.logging.LogManager;
 import edu.isi.pegasus.common.util.DefaultStreamGobblerCallback;
@@ -44,10 +43,11 @@ import edu.isi.pegasus.planner.code.CodeGeneratorException;
 import edu.isi.pegasus.planner.code.GridStart;
 import edu.isi.pegasus.planner.code.GridStartFactory;
 import edu.isi.pegasus.planner.code.POSTScript;
+import edu.isi.pegasus.planner.code.generator.condor.SUBDAXGenerator;
 import edu.isi.pegasus.planner.namespace.Dagman;
-import edu.isi.pegasus.planner.partitioner.graph.Adapter;
 import edu.isi.pegasus.planner.partitioner.graph.Graph;
 import edu.isi.pegasus.planner.partitioner.graph.GraphNode;
+import java.util.Set;
 
 /**
  * This code generator generates a shell script in the submit directory.
@@ -55,7 +55,7 @@ import edu.isi.pegasus.planner.partitioner.graph.GraphNode;
  * locally.
  *
  * @author Karan Vahi
- * @version $Revision: 4818 $
+ * @version $Revision$
  */
 public class Shell extends Abstract {
 
@@ -95,11 +95,16 @@ public class Shell extends Abstract {
      */
     protected GridStartFactory mGridStartFactory;
 
+    /**
+     * Handle to the Credential Factory
+     */
+    protected CredentialHandlerFactory mFactory;
+
     /**
      * A boolean indicating whether grid start has been initialized or not.
      */
     protected boolean mInitializeGridStart;
-
+    
     
     /**
      * The default constructor.
@@ -127,7 +132,9 @@ public class Shell extends Abstract {
 
         //get the handle to pool file
         mSiteStore = bag.getHandleToSiteStore();
-
+        
+        mFactory = new CredentialHandlerFactory();
+	mFactory.initialize( mBag );
     }
 
     /**
@@ -153,8 +160,8 @@ public class Shell extends Abstract {
         //write out the script header
         writeString(this.getScriptHeader( mSubmitFileDir ) );
 
-        //we first need to convert internally into graph format
-        Graph workflow =    Adapter.convert( dag );
+        //PM-747 no need for conversion as ADag now implements Graph interface
+        Graph workflow =  dag;
 
         //traverse the workflow in topological sort order
         for( Iterator<GraphNode> it = workflow.topologicalSortIterator(); it.hasNext(); ){
@@ -180,7 +187,7 @@ public class Shell extends Abstract {
         this.writeOutStampedeEvents( dag );
 
         //write out the metrics file
-        this.writeOutWorkflowMetrics(dag);
+//        this.writeOutWorkflowMetrics(dag);
 
         return result;
     }
@@ -203,17 +210,23 @@ public class Shell extends Abstract {
         if( !job.getSiteHandle().equals( "local" ) ){
             throw new CodeGeneratorException( "Shell Code generator only works for jobs scheduled to site local" );
         }
-
-	CredentialHandlerFactory factory = new CredentialHandlerFactory();
-	factory.initialize( mBag );
-
-	for (TYPE type : job.getCredentialTypes()) {
-	    CredentialHandler handler = factory.loadInstance( type );
-	    job.addProfile( new Profile( Profile.ENV, handler
-		    .getEnvironmentVariable(), handler.getPath() ) );
-	}
+        if ( job.getJobType () == Job.DAX_JOB ) {
+            SUBDAXGenerator subdax = new SUBDAXGenerator ();
+            subdax.initialize ( mBag, dag,  mWriteHandle );
+            subdax.generateCode ( job );
+        }
+        
+	//handle credentials for the job
+        for( Map.Entry<String,Set<CredentialHandler.TYPE>> entry : job.getCredentialTypes().entrySet()  ){
+            String site = entry.getKey();
+            for( CredentialHandler.TYPE cred: entry.getValue()){
+                CredentialHandler handler = mFactory.loadInstance( cred );
+                job.addProfile( new Profile( Profile.ENV,
+                                             handler.getEnvironmentVariable( site ), 
+                                             handler.getPath( site ) ) );
+            }
 	
-	factory = null;
+        }
 	
         //initialize GridStart if required.
         if ( mInitializeGridStart ){
@@ -233,7 +246,6 @@ public class Shell extends Abstract {
 
         //JIRA PM-491 . Path to kickstart should not be passed
         //to the factory.
-//        String gridStartPath = site.getKickstartPath();
         GridStart gridStart = mGridStartFactory.loadGridStart( job , null );
 
         //enable the job
@@ -292,7 +304,7 @@ public class Shell extends Abstract {
      */
     protected String generateCallToCheckExitcode( Job job,
                                                   String prefix ){
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
             sb.append( "check_exitcode" ).append( " " ).
                append(  job.getID() ).append( " " ).
                append(  prefix ).append( " " ).
@@ -312,13 +324,13 @@ public class Shell extends Abstract {
      */
     protected String generateCallToExecutePostScript( Job job,
                                                       String directory ){
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         
         //gridstart modules right now store the executable
         //and arguments as condor profiles. Should be fixed.
         //This setting should happen only in Condor Generator
         String executable = (String) job.dagmanVariables.get( Dagman.POST_SCRIPT_KEY );
-        StringBuffer args = new StringBuffer();
+        StringBuilder args = new StringBuilder();
         args.append( (String)job.dagmanVariables.get( Dagman.POST_SCRIPT_ARGUMENTS_KEY ) ).
              append( " " ).append( (String)job.dagmanVariables.get( Dagman.OUTPUT_KEY)  ); 
         
@@ -356,7 +368,7 @@ public class Shell extends Abstract {
     protected String generateCallToExecuteJob( Job job,
                                                String scratchDirectory,
                                                String submitDirectory ){
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         
         //gridstart modules right now store the executable
         //and arguments as condor profiles. Should be fixed.
@@ -366,9 +378,10 @@ public class Shell extends Abstract {
         String arguments = (String)job.condorVariables.get( Condor.ARGUMENTS_KEY );
  */
         String executable = job.getRemoteExecutable();
-        String arguments = job.getArguments();
+        String arguments = job.getJobType() == Job.DAX_JOB ? job.getPreScriptPath() + job.getPreScriptArguments()+ " --submit" : job.getArguments();
         arguments = ( arguments == null ) ? "" : arguments;
-
+        //arguments = job.getJobType() == Job.DAX_JOB ? arguments + " --submit" : arguments;
+        
         String directory = job.runInWorkDirectory() ? scratchDirectory : submitDirectory;
 
         //generate the call to execute job function
@@ -419,7 +432,7 @@ public class Shell extends Abstract {
      */
     protected String getScriptHeader( String submitDirectory ){
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         sb.append( "#!/bin/bash" ).append( "\n" ).
            append( "#" ).append( "\n" ).
            append( "# executes the workflow in shell mode " ).append( "\n" ).
@@ -458,7 +471,7 @@ public class Shell extends Abstract {
      * @return the path on the submit host.
      */
     protected String getSubmitHostPathToShellRunnerFunctions() {
-        StringBuffer path = new StringBuffer();
+        StringBuilder path = new StringBuilder();
 
         //first get the path to the share directory
         File share = mProps.getSharedDir();
@@ -480,7 +493,7 @@ public class Shell extends Abstract {
      */
     protected String getScriptFooter(){
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         sb.append( "echo \"INTERNAL *** SHELL_SCRIPT_FINISHED 0 ***\" >> $JOBSTATE_LOG" );
 
         return sb.toString();
@@ -494,9 +507,9 @@ public class Shell extends Abstract {
      * @return path
      */
     protected String getPathToShellScript(ADag dag) {
-        StringBuffer script = new StringBuffer();
+        StringBuilder script = new StringBuilder();
         script.append( this.mSubmitFileDir ).append( File.separator ).
-               append( dag.dagInfo.nameOfADag ).append( ".sh" );
+               append( dag.getLabel() ).append( ".sh" );
         return script.toString();
     }
 
