@@ -40,8 +40,6 @@ import edu.isi.pegasus.common.logging.LogManager;
 import edu.isi.pegasus.common.util.CondorVersion;
 
 import edu.isi.pegasus.planner.classes.ADag;
-import edu.isi.pegasus.planner.classes.DagInfo;
-import edu.isi.pegasus.planner.classes.PCRelation;
 import edu.isi.pegasus.planner.classes.Job;
 import edu.isi.pegasus.planner.classes.PegasusBag;
 import edu.isi.pegasus.planner.classes.DAGJob;
@@ -59,8 +57,6 @@ import edu.isi.pegasus.planner.namespace.Globus;
 import edu.isi.pegasus.planner.namespace.Pegasus;
 import edu.isi.pegasus.planner.namespace.ENV;
 
-import edu.isi.pegasus.planner.partitioner.graph.Adapter;
-import edu.isi.pegasus.planner.partitioner.graph.Graph;
 import edu.isi.pegasus.planner.partitioner.graph.GraphNode;
 import org.griphyn.vdl.euryale.VTorInUseException;
 
@@ -75,11 +71,9 @@ import java.io.InputStreamReader;
 import java.io.InputStream;
 
 import java.util.ArrayList;
-import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.Map;
 import java.util.StringTokenizer;
-import java.util.Vector;
 import java.util.Collection;
 import java.util.List;
 import java.util.Properties;
@@ -90,7 +84,7 @@ import java.util.HashMap;
  *
  * @author Gaurang Mehta
  * @author Karan Vahi
- * @version $Revision: 4782 $
+ * @version $Revision$
  */
 public class CondorGenerator extends Abstract {
     
@@ -106,7 +100,17 @@ public class CondorGenerator extends Abstract {
         "######################################################################";
     
     
-    
+    /**
+     * Default value for the periodic_release for a job
+     */
+    public static final String DEFAULT_PERIODIC_RELEASE_VALUE = "False";
+
+
+    /**
+     * Default value for the periodic_remove for a job
+     */
+    public static final String DEFAULT_PERIODIC_REMOVE_VALUE = "(JobStatus == 5) && ((CurrentTime - EnteredCurrentStatus) > 3600)";
+
 
     /**
      * The namespace to use for condor dagman.
@@ -231,7 +235,8 @@ public class CondorGenerator extends Abstract {
      * Boolean indicating whether to assign job priorities or not.
      */
     private boolean mAssignDefaultJobPriorities;
-    
+
+
     /**
      * The default constructor.
      */
@@ -294,16 +299,15 @@ public class CondorGenerator extends Abstract {
      * @throws CodeGeneratorException in case of any error occuring code generation.
      */
     public Collection<File> generateCode( ADag dag ) throws CodeGeneratorException{
-        DagInfo ndi        = dag.dagInfo;
-        Vector vSubInfo    = dag.vJobSubInfos;
+//        DagInfo ndi        = dag.dagInfo;
+//       Vector vSubInfo    = dag.vJobSubInfos;
 
         if ( mInitializeGridStart ){
             mConcreteWorkflow = dag;
             mGridStartFactory.initialize( mBag, dag );
             mInitializeGridStart = false;
         }
-
-
+        
         CodeGenerator storkGenerator = CodeGeneratorFactory.loadInstance( mBag, "Stork" );
 
         String className   = this.getClass().getName();
@@ -312,18 +316,10 @@ public class CondorGenerator extends Abstract {
 
         File dagFile = null;
         Collection<File> result = new ArrayList(1);
-        if (ndi.dagJobs.isEmpty()) {
+        if ( dag.isEmpty() ) {
             //call the callout before returns
             concreteDagEmpty( dagFileName, dag );
             return result ;
-        } else {
-            //initialize the file handle to the dag
-            //file and print it's header
-            dagFile = initializeDagFileWriter( dagFileName, ndi );
-            result.add( dagFile );
-
-            //write out any category based dagman knobs to the dagman file
-            printDagString( this.getCategoryDAGManKnobs( mProps ) );
         }
 
         
@@ -349,8 +345,8 @@ public class CondorGenerator extends Abstract {
             //and symlink it to the submit directory.
             try{
           
-                File f = File.createTempFile( dag.dagInfo.nameOfADag + "-" +
-                                              dag.dagInfo.index,".log", 
+                File f = File.createTempFile( dag.getLabel()  + "-" +
+                                              dag.getIndex(), ".log", 
                                               directory );
                 mTempLogFile=f.getAbsolutePath();
             } catch (IOException ioe) {
@@ -369,11 +365,24 @@ public class CondorGenerator extends Abstract {
   
         //convert the dax to a graph representation and walk it
         //in a top down manner
-        Graph workflow = Adapter.convert( dag );
+        //PM-747 no need for conversion as ADag now implements Graph interface
+        //Graph workflow = dag;
         SUBDAXGenerator subdaxGen = new SUBDAXGenerator();
-        subdaxGen.initialize( mBag, dag, workflow, mDagWriter );
+        subdaxGen.initialize( mBag, dag, mDagWriter );
+
+        //we should initialize the .dag file only when we are done
+        //with the conversion
+
+        //initialize the file handle to the dag
+        //file and print it's header
+        dagFile = initializeDagFileWriter( dagFileName, dag );
+        result.add( dagFile );
+
+        //write out any category based dagman knobs to the dagman file
+        printDagString( this.getCategoryDAGManKnobs( mProps ) );
+
                     
-        for( Iterator it = workflow.iterator(); it.hasNext(); ){
+        for( Iterator it = dag.iterator(); it.hasNext(); ){
             GraphNode node = ( GraphNode )it.next();
             Job job = (Job)node.getContent();
             
@@ -468,7 +477,7 @@ public class CondorGenerator extends Abstract {
 
         //writing the tail of .dag file
         //that contains the relation pairs
-        this.writeDagFileTail( ndi );
+        this.writeDagFileTail( dag );
         mLogger.log("Written Dag File : " + dagFileName.toString(),
                     LogManager.DEBUG_MESSAGE_LEVEL);
 
@@ -501,7 +510,7 @@ public class CondorGenerator extends Abstract {
         this.writeOutStampedeEvents( dag );
 
         //write out the metrics file
-        this.writeOutWorkflowMetrics(dag);
+//        this.writeOutWorkflowMetrics(dag);
 
         //write out the braindump file
         this.writeOutBraindump( dag );
@@ -522,9 +531,9 @@ public class CondorGenerator extends Abstract {
      * @throws CodeGeneratorException in case of any error occuring code generation.
      */
     public void generateCode( ADag dag, Job job ) throws CodeGeneratorException{
-        String dagname  = dag.dagInfo.nameOfADag;
-        String dagindex = dag.dagInfo.index;
-        String dagcount = dag.dagInfo.count;
+        String dagname  = dag.getLabel();
+        String dagindex = dag.getIndex();
+        String dagcount = dag.getCount();
         String subfilename = this.getFileBaseName( job );
         String envStr = null;
 
@@ -586,13 +595,25 @@ public class CondorGenerator extends Abstract {
         //for the job from various resources
         handleGlobusRSLForJob( job );
 
-        writer.println(this.mSeparator);
+        StringBuffer fragment = new StringBuffer();
+
+        //add the header to the fragment
+        fragment.append(CondorGenerator.mSeparator).append( "\n" );
+        fragment.append("# PEGASUS WMS GENERATED SUBMIT FILE").append( "\n" );
+        fragment.append("# DAG : " + dagname + ", Index = " + dagindex +
+                       ", Count = " + dagcount).append( "\n" );
+        fragment.append("# SUBMIT FILE NAME : " + subfilename).append( "\n" );
+        fragment.append(CondorGenerator.mSeparator);
+        writer.println( fragment );
+
+/*
+        writer.println(CondorGenerator.mSeparator);
         writer.println("# PEGASUS WMS GENERATED SUBMIT FILE");
         writer.println("# DAG : " + dagname + ", Index = " + dagindex +
                        ", Count = " + dagcount);
         writer.println("# SUBMIT FILE NAME : " + subfilename);
-        writer.println(this.mSeparator);
-
+        writer.println(CondorGenerator.mSeparator);
+*/
         //figure out the style to apply for a job
         applyStyle( job, writer );
 
@@ -617,11 +638,19 @@ public class CondorGenerator extends Abstract {
         ClassADSGenerator.generate( writer, dag, job );
 
         // DONE
+        fragment = new StringBuffer();
+
+        fragment.append("queue").append( "\n" );
+        fragment.append(CondorGenerator.mSeparator).append( "\n" );
+        fragment.append("# END OF SUBMIT FILE").append( "\n" );
+        fragment.append(CondorGenerator.mSeparator);
+        writer.println( fragment );
+/*
         writer.println("queue");
         writer.println(this.mSeparator);
         writer.println("# END OF SUBMIT FILE");
         writer.println(this.mSeparator);
-
+*/
         // close the print stream to the file (flush)
         writer.close();
         return;
@@ -981,13 +1010,13 @@ public class CondorGenerator extends Abstract {
      * Initializes the file handler to the dag file and writes the header to it.
      *
      * @param filename     basename of dag file to be written.
-     * @param dinfo        object containing daginfo of type DagInfo .
+     * @param dag        the workflow
      *
      * @return the File object for the DAG file.
      *
      * @throws CodeGeneratorException in case of any error occuring code generation.
      */
-    protected File initializeDagFileWriter(String filename, DagInfo dinfo)
+    protected File initializeDagFileWriter(String filename, ADag workflow )
                                                        throws CodeGeneratorException{
         // initialize file handler
 
@@ -1003,9 +1032,9 @@ public class CondorGenerator extends Abstract {
 
             printDagString(this.mSeparator);
             printDagString("# PEGASUS WMS GENERATED DAG FILE");
-            printDagString("# DAG " + dinfo.nameOfADag);
-            printDagString("# Index = " + dinfo.index + ", Count = " +
-                           dinfo.count);
+            printDagString("# DAG " + workflow.getLabel() );
+            printDagString("# Index = " + workflow.getIndex() + ", Count = " +
+                           workflow.getCount() );
             printDagString(this.mSeparator);
         } catch (Exception e) {
             throw new CodeGeneratorException( "While writing to DAG FILE " + filename,
@@ -1114,20 +1143,23 @@ public class CondorGenerator extends Abstract {
      * It writes the relations making up the  DAG in the dag file and and closes
      * the file handle to it.
      *
-     * @param dinfo   object containing daginfo of type DagInfo.
-     *
+     * @param dag the executable workflow
+     * 
      * @throws CodeGeneratorException
      */
-    protected void writeDagFileTail(DagInfo dinfo) throws CodeGeneratorException{
+    protected void writeDagFileTail( ADag dag ) throws CodeGeneratorException{
         try {
+            
+            for( Iterator<GraphNode> it = dag.jobIterator(); it.hasNext() ; ){
+                GraphNode gn = (GraphNode) it.next();
 
-            // read the contents from the Daginfo object and
-            //print out the parent child relations.
-
-            for (Enumeration dagrelationsenum = dinfo.relations.elements();
-                 dagrelationsenum.hasMoreElements(); ) {
-                PCRelation pcrl = (PCRelation) dagrelationsenum.nextElement();
-                printDagString("PARENT " + pcrl.parent + " CHILD " + pcrl.child);
+                //get a list of parents of the node
+                for( GraphNode child : gn.getChildren() ){
+                    StringBuffer edge = new StringBuffer();
+                    edge.append( "PARENT " ).append( " " ).append( gn.getID() ).append( " " ).
+                         append( "CHILD " ).append( child.getID() );
+                    printDagString( edge.toString() );
+                }
             }
 
             printDagString(this.mSeparator);
@@ -1206,8 +1238,8 @@ public class CondorGenerator extends Abstract {
        }
        else{
            //generate the prefix from the name of the dag
-           sb.append(dag.dagInfo.nameOfADag).append("-").
-               append(dag.dagInfo.index);
+           sb.append(dag.getLabel() ).append("-").
+               append(dag.getIndex() );
        }
        //append the suffix
        sb.append(".log");
@@ -1312,8 +1344,8 @@ public class CondorGenerator extends Abstract {
         }
         else{
             //generate the prefix from the name of the dag
-            sb.append(dag.dagInfo.nameOfADag).append("-").
-                append(dag.dagInfo.index);
+            sb.append( dag.getLabel() ).append("-").
+                append( dag.getIndex() );
         }
         //append the suffix
         sb.append(".dag.dagman.out");
@@ -1458,8 +1490,8 @@ public class CondorGenerator extends Abstract {
      * 
      * The default expression for periodic_release and periodic_remove is
      * <pre>
-     *   periodic_release = (NumSystemHolds <= releasevalue)
-     *   periodic_remove = (NumSystemHolds > removevalue)
+     *  periodic_release = False
+     *  periodic_remove = (JobStatus == 5) && ((CurrentTime - EnteredCurrentStatus) > 14400)
      * </pre>
      * where releasevalue is value of condor profile periodic_release
      * and   removevalue  is value of condor profile periodic_remove
@@ -1467,83 +1499,56 @@ public class CondorGenerator extends Abstract {
      * @param job  the job object.
      */
     public void populatePeriodicReleaseAndRemoveInJob( Job job ){
-
-        //Karan Oct 19, 2005. The values in property file
-        //should only be treated as default. Need to reverse
-        //below.
-
         //get the periodic release values always a default
         //value is got if not specified.
         String releaseval = (String) job.condorVariables.get( Condor.PERIODIC_RELEASE_KEY );
-        releaseval = (releaseval == null) ?
-            //put in default value
-            "3" :
-            //keep the one from profiles or dax
-            releaseval;
-        
 
-        String removeval = (String) job.condorVariables.get( Condor.PERIODIC_REMOVE_KEY );
-        removeval = (removeval == null) ?
-            //put in default value
-            "3" :
-            //keep the one from profiles or dax
-            removeval;
-
-
-        int removeint  = this.getNaturalNumberValue( removeval );
-        int releaseint = this.getNaturalNumberValue( releaseval );
-
-        if( removeint > 0 && releaseint > 0 ){
-            if( removeint > releaseint ){
-                removeval = releaseval;
-                //throw a warning down
-                mLogger.log(
-                    " periodic_remove > periodic_release " +
-                    "for job " + 
-                     ". Setting periodic_remove=periodic_release",
-                    LogManager.WARNING_MESSAGE_LEVEL);
+        if( releaseval == null ){
+            //construct default value
+           job.condorVariables.construct( Condor.PERIODIC_RELEASE_KEY, CondorGenerator.DEFAULT_PERIODIC_RELEASE_VALUE );
+        }
+        else{
+            //check if an integer value is specified PM-462
+            if ( isInteger( releaseval ) ){
+                mLogger.log( "Removing integer value " + releaseval + " for periodic_release for job " + job.getID(),
+                             LogManager.DEBUG_MESSAGE_LEVEL );
+                job.condorVariables.construct( Condor.PERIODIC_RELEASE_KEY, CondorGenerator.DEFAULT_PERIODIC_RELEASE_VALUE );
             }
         }
 
-    
-        String value = null;
-        if( releaseint > 0){
-            value = "(NumSystemHolds <= " + releaseint + ")";
-            job.condorVariables.construct( Condor.PERIODIC_RELEASE_KEY, value);
+        String removeval = (String) job.condorVariables.get( Condor.PERIODIC_REMOVE_KEY );
+        if( removeval == null ){
+            //construct default value
+           job.condorVariables.construct( Condor.PERIODIC_REMOVE_KEY, CondorGenerator.DEFAULT_PERIODIC_REMOVE_VALUE );
         }
         else{
-            job.condorVariables.construct( Condor.PERIODIC_RELEASE_KEY, releaseval );
+            //check if an integer value is specified PM-462
+            if ( isInteger( removeval ) ){
+                mLogger.log( "Removing integer value " + removeval + " for periodic_remove for job " + job.getID(),
+                             LogManager.DEBUG_MESSAGE_LEVEL );
+                job.condorVariables.construct( Condor.PERIODIC_REMOVE_KEY, CondorGenerator.DEFAULT_PERIODIC_REMOVE_VALUE );
+            }
         }
 
-        
-        if( removeint > 0){
-            value = "(NumSystemHolds > " + removeint + ")";
-            job.condorVariables.construct( Condor.PERIODIC_REMOVE_KEY, value );
-        }
-        else{
-            job.condorVariables.construct( Condor.PERIODIC_REMOVE_KEY, removeval );
-        }
-
-    
     }
     
     
     /**
-     * Returns a natural number value ( > 0 ) if the parameter passed is an integer
-     * and greater than zero, else -1
+     * Returns a boolean indicating whether the value represented is an
+     * integer or not.
      * 
      * @param value   the String passed
      * 
-     * @return
+     * @return true if an int else false
      */
-    protected int getNaturalNumberValue( String value ){
-        int result = -1;
+    protected boolean isInteger ( String value ){
+        boolean result = true;
 
         try{
-            result = Integer.parseInt(value);
+             Integer.parseInt(value);
         }
         catch( Exception e ){
-
+            result = false;
         }
         return result;
     }
