@@ -22,6 +22,7 @@ import edu.isi.pegasus.common.logging.LogManager;
 
 import edu.isi.pegasus.common.util.CondorVersion;
 import edu.isi.pegasus.common.util.FindExecutable;
+import edu.isi.pegasus.common.util.Separator;
 import edu.isi.pegasus.planner.classes.ADag;
 import edu.isi.pegasus.planner.classes.PlannerOptions;
 import edu.isi.pegasus.planner.classes.PegasusBag;
@@ -37,6 +38,9 @@ import edu.isi.pegasus.planner.common.RunDirectoryFilenameFilter;
 import edu.isi.pegasus.planner.client.CPlanner;
 
 import edu.isi.pegasus.planner.catalog.TransformationCatalog;
+import edu.isi.pegasus.planner.catalog.classes.Profiles;
+import edu.isi.pegasus.planner.catalog.site.classes.SiteCatalogEntry;
+import edu.isi.pegasus.planner.catalog.site.classes.SiteStore;
 import edu.isi.pegasus.planner.catalog.transformation.TransformationCatalogEntry;
 import edu.isi.pegasus.planner.catalog.transformation.classes.TCType;
 
@@ -44,6 +48,7 @@ import edu.isi.pegasus.planner.catalog.transformation.classes.TCType;
 import edu.isi.pegasus.planner.classes.DAXJob;
 import edu.isi.pegasus.planner.code.GridStartFactory;
 import edu.isi.pegasus.planner.code.generator.DAXReplicaStore;
+import edu.isi.pegasus.planner.code.gridstart.PegasusLite;
 import edu.isi.pegasus.planner.namespace.Condor;
 import edu.isi.pegasus.planner.namespace.ENV;
 import edu.isi.pegasus.planner.namespace.Pegasus;
@@ -51,10 +56,14 @@ import edu.isi.pegasus.planner.namespace.Pegasus;
 import edu.isi.pegasus.planner.partitioner.graph.Graph;
 import edu.isi.pegasus.planner.partitioner.graph.GraphNode;
 import java.io.BufferedReader;
+import java.io.BufferedWriter;
 import java.io.File;
+import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.OutputStream;
+import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 
 import java.text.NumberFormat;
@@ -70,7 +79,7 @@ import java.util.Set;
  * a SUBDAG with pegasus-plan as the appropriate prescript.
  *
  * @author Karan Vahi
- * @version $Revision: 4507 $
+ * @version $Revision$
  */
 public class SUBDAXGenerator{
 
@@ -190,10 +199,18 @@ public class SUBDAXGenerator{
      * on top down traversal during Code Generation.
      */
     private Map<String,String>mDAXJobIDToSubmitDirectoryCacheFile;
-    
-    private Graph mWorkflow;
+  
+    //PM-747 no need for conversion as ADag now implements Graph interface
+//    private Graph mWorkflow;
     private ADag mDAG;
     
+    private SiteStore mSiteStore;
+    
+    /**
+     * Cache file for the current DAG
+     */
+    private String mCurrentDAGCacheFile;
+    
     
     /**
      * The default constructor.
@@ -210,20 +227,23 @@ public class SUBDAXGenerator{
      *
      * @param bag  the bag of objects required for initialization
      * @param dag  the dag for which code is being generated
-     * @param workflow the graph representation of the dag
      * @param daxReplicaStore the dax replica store.
      * @param dagWriter  handle to the dag writer
      */
-    public void initialize( PegasusBag bag, ADag dag,Graph workflow, PrintWriter dagWriter ){
+    public void initialize( PegasusBag bag, ADag dag, PrintWriter dagWriter ){
         mBag = bag;
-        mWorkflow = workflow;
         mDAG = dag;
         mDAGWriter = dagWriter;
         mProps  = bag.getPegasusProperties();
         mLogger = bag.getLogger();
         mTCHandle = bag.getHandleToTransformationCatalog();
+        mSiteStore = bag.getHandleToSiteStore();
         this.mPegasusPlanOptions  = bag.getPlannerOptions();
         mCleanupScope = mProps.getCleanupScope();
+        
+        mCurrentDAGCacheFile = this.getCacheFile( mPegasusPlanOptions, 
+                                                  dag.getLabel(), 
+                                                  dag.getIndex());
         mDAXJobIDToSubmitDirectoryCacheFile = new HashMap();
                 
         mUser = mProps.getProperty( "user.name" ) ;
@@ -374,7 +394,16 @@ public class SUBDAXGenerator{
                          LogManager.DEBUG_MESSAGE_LEVEL  );
             cacheFiles.addAll( parentsTransientRCs  );
         }
-
+        
+        //we also add path to the cache file of the workflow 
+        //currently being planned i.e the one that has the dax job 
+        //for the sub workflow. this is to ensure that if we have a DAXA
+        //that has two jobs JOBA and DAXJobB in it, with JOBA parent of DAXJobB
+        //i.e JOBA -> DAXJobB, then whatever JOBA generates is accessible
+        //when we plan the DAXJob B. To ensure this we need to pass the cache
+        //file generated when planning DAXA to DAXJobB
+        //PM-736
+        cacheFiles.add( mCurrentDAGCacheFile );
 
         //do some sanitization of the path to the dax file.
         //if it is a relative path, then ???
@@ -482,7 +511,7 @@ public class SUBDAXGenerator{
             //point to the outer level workflow DAX replica store file
             inheritedRCFile.append( DAXReplicaStore.getDAXReplicaStoreFile( this.mPegasusPlanOptions,
                                                                             this.mDAG.getLabel(),
-                                                                            this.mDAG.dagInfo.index  )
+                                                                            this.mDAG.getIndex() )
                                                               );
             options.setInheritedRCFiles( inheritedRCFile.toString() );
         }
@@ -494,12 +523,12 @@ public class SUBDAXGenerator{
         StringBuffer log = new StringBuffer();
         log.append( mPegasusPlanOptions.getSubmitDirectory() ).append( File.separator ).
             append( job.getName() ).append( ".pre.log" );
-        String prescript = constructPegasusPlanPrescript( job, 
+        Job prescript = constructPegasusPlanPrescript( job, 
                                                           options,
                                                           mDAG.getRootWorkflowUUID(), 
                                                           propertiesFile,
                                                           log.toString() );
-        job.setPreScript( prescript );
+        //job.setPreScript( prescript );
         
         //determine the path to the dag file that will be constructed
         if( GENERATE_SUBDAG_KEYWORD ){
@@ -516,35 +545,100 @@ public class SUBDAXGenerator{
             return null;
         }
         else{
-            StringBuffer basenamePrefix = new StringBuffer();
-            if( options.getBasenamePrefix() == null ){
-                basenamePrefix.append( label );                
-                basenamePrefix.append( "-" ).append( index );
-            }else{
-                //add the prefix from options
-                basenamePrefix.append( options.getBasenamePrefix() );
-            }
-            
+            String basenamePrefix = this.getWorkflowFileBasenamePrefix(options, label, index);
+                    
             mLogger.log( "Basename prefix for the sub workflow is " + basenamePrefix,
                          LogManager.DEBUG_MESSAGE_LEVEL );
-            String subDAXCache = new File( options.getSubmitDirectory(),
-                                           basenamePrefix + CACHE_FILE_SUFFIX ).getAbsolutePath();
+            String subDAXCache = this.getCacheFile(options, label, index);
             mLogger.log( "Cache File for the sub workflow is " + subDAXCache,
                          LogManager.DEBUG_MESSAGE_LEVEL );
             mDAXJobIDToSubmitDirectoryCacheFile.put( job.getID(), subDAXCache);
             
+            //submit directory is the submit directory of the DAX that is currently
+            //being planned. The one that contains the DAX job.
+            File submitDirectory = new File( mPegasusPlanOptions.getSubmitDirectory() );
             Job dagJob = constructDAGJob( job,
-                                    new File( mPegasusPlanOptions.getSubmitDirectory() ),
-                                    new File( options.getSubmitDirectory()),
-                                    basenamePrefix.toString()
-                                  );
-            //set the prescript
-            dagJob.setPreScript( prescript );
+                                          submitDirectory,
+                                          new File( options.getSubmitDirectory()),
+                                          basenamePrefix.toString()
+                                        );
+            
+            
+            File wrapper = constructPlannerPrescriptWrapper( dagJob, 
+                                                             submitDirectory,
+                                                             job.getPreScriptPath(),
+                                                             job.getPreScriptArguments() );
+            
+            //set the prescript to the dag job
+            //the executable is the wrapper now PM-667
+//            dagJob.setPreScript( job.getPreScriptPath(), job.getPreScriptArguments() );
+            dagJob.setPreScript( wrapper.getAbsolutePath(), job.getPreScriptArguments() );
+            
             return dagJob;
         }
 
     }
     
+     
+    /**
+     * Construct a pegasus plan wrapper script that changes the directory in which
+     * pegasus-plan is launched. 
+     * 
+     * @param dagJob        the DAG job corresponding to which the prescript is associated.
+     * @param directory   the directory where the submit file for dagman job has
+     *                    to be written out to.
+     * @param executable    the path to the planner that needs to be called in the prescript
+     * @param arguments     the arguments with which the planner is called.
+     * 
+     * @return the wrapper script that gets called in the prescript for the dag job 
+     */
+    protected File constructPlannerPrescriptWrapper( Job dagJob, 
+                                                     File directory,
+                                                     String executable,
+                                                     String arguments ){
+        
+        //determine the basename for the wrapper
+        String basename = this.getBasename( dagJob.getName(), "_pre.sh" );
+        File wrapper = new File( directory, basename );
+        
+        try{
+            OutputStream ostream = new FileOutputStream( wrapper , true );
+            PrintWriter writer = new PrintWriter( new BufferedWriter(new OutputStreamWriter(ostream)) );
+
+            //determine the launch directory in which the pre script should be
+            //called. it is the scratch directory for local site
+            String launchDir = mSiteStore.getInternalWorkDirectory( dagJob );
+            
+            StringBuffer sb = new StringBuffer( );
+            sb.append( "#!/bin/bash" ).append( '\n' );
+            sb.append( "set -e" ).append( '\n' );
+            sb.append( "cd "  ).append( launchDir );
+            sb.append( '\n' );
+            sb.append( executable ).append( " ").append( "$@" );
+            sb.append( '\n' );
+    
+            writer.print( sb.toString() );
+            writer.flush();
+            
+            writer.close();
+            ostream.close();
+
+            //set the xbit on the shell script
+            //for 3.2, we will have 1.6 as the minimum jdk requirement
+            wrapper.setExecutable( true, false );
+ 
+        }
+        catch( IOException ioe ){
+            throw new RuntimeException( "Error while writing out prescript wrapper  " + wrapper 
+                                        + " for job " + dagJob.getName(), ioe );
+        }
+
+        
+        return wrapper;
+    
+    }
+
+    
     /**
      * Constructs a job that plans and submits the partitioned workflow,
      * referred to by a Partition. The main job itself is a condor dagman job
@@ -613,22 +707,30 @@ public class SUBDAXGenerator{
             entries = mTCHandle.lookup(job.namespace, job.logicalName,
                                                  job.version, job.getSiteHandle(),
                                                  TCType.INSTALLED);
+            if( entries == null ){
+                mLogger.log( Separator.combine( job.namespace, job.logicalName,  job.version ) + 
+                             "  not catalogued in the Transformation Catalog. Trying to construct from the Site Catalog",
+                              LogManager.DEBUG_MESSAGE_LEVEL );
+                entry = defaultTCEntry( "local" );
+            }
+            else{
+                entry = (TransformationCatalogEntry) entries.get(0);
+                mLogger.log( "Picked path to DAGMan from the Transformation Catalog " + entry.getPhysicalTransformation() ,
+                             LogManager.DEBUG_MESSAGE_LEVEL );
+            }
+            
+            /*
             entry = (entries == null) ?
                      defaultTCEntry( "local") ://construct from site catalog
                       //Gaurang assures that if no record is found then
                       //TC Mechanism returns null
                       (TransformationCatalogEntry) entries.get(0);
-            
+            */
             if( entry == null ){
-                mLogger.log( "DAGMan not catalogued in the TC. Trying to construct from the environment",
+                mLogger.log( "DAGMan not catalogued in the Transformation Catalog or the Site Catalog. Trying to construct from the environment",
                               LogManager.DEBUG_MESSAGE_LEVEL );
                 entry = constructTCEntryFromEnvironment( );
             }
-            else{
-                mLogger.log( "Picked path to DAGMan from the TC " + entry.getPhysicalTransformation() ,
-                             LogManager.DEBUG_MESSAGE_LEVEL );
-
-            }
 
             
         }
@@ -637,9 +739,9 @@ public class SUBDAXGenerator{
         }
         if(entry == null){
             //throw appropriate error
-            throw new RuntimeException("ERROR: Entry not found in tc for job " +
+            throw new RuntimeException("Unable to construct entry for  " +
                                         job.getCompleteTCName() +
-                                        " on site " + job.getSiteHandle());
+                                        " from the Transformation Catalog, Site Catalog or the Environment for site " + job.getSiteHandle());
         }
 
         //set the path to the executable and environment string
@@ -834,7 +936,22 @@ public class SUBDAXGenerator{
 
 
    
-
+    /**
+     * Returns the path to the cache file in a workflow's submit directory
+     * 
+     * @param options   the options for the  workflow.
+     * @param label     the label for the workflow.
+     * @param index     the index for the workflow.
+     *
+     * @return the path to the cache file
+     */
+    protected String getCacheFile( PlannerOptions options, String label , String index ){
+        
+        return new File( options.getSubmitDirectory(),
+                         this.getWorkflowFileName(options, label, index, CACHE_FILE_SUFFIX) ).getAbsolutePath();
+        
+    }
+    
     /**
      * Constructs the basename to the cache file that is to be used
      * to log the transient files. The basename is dependant on whether the
@@ -857,11 +974,32 @@ public class SUBDAXGenerator{
      * @param options   the options for the sub workflow.
      * @param label     the label for the workflow.
      * @param index     the index for the workflow.
+     * @param suffix    the suffix for the workfklow file.
      *
      * @return the name of the cache file
      */
     protected String getWorkflowFileName( PlannerOptions options, String label , String index, String suffix ){
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
+        
+        sb.append( this.getWorkflowFileBasenamePrefix(options, label, index) );
+        //append the suffix
+        sb.append( suffix );
+
+        return sb.toString();
+
+    }
+
+    /* Constructs the basename prefix for a workflow file.  This is dependant
+     * on whether the  basename prefix has been specified in options or not.
+     *
+     * @param options   the options for the sub workflow.
+     * @param label     the label for the workflow.
+     * @param index     the index for the workflow.
+     *
+     * @return the name of the cache file
+     */
+    protected String getWorkflowFileBasenamePrefix( PlannerOptions options, String label , String index ){
+        StringBuilder sb = new StringBuilder();
         String bprefix = options.getBasenamePrefix();
 
         if(bprefix != null){
@@ -871,16 +1009,11 @@ public class SUBDAXGenerator{
         else{
             //generate the prefix from the name of the dag
             sb.append( label ).append("-").
-           append( index );
+               append( index );
         }
-        //append the suffix
-        sb.append( suffix );
-
         return sb.toString();
-
     }
 
-
     /**
      * Returns a default TC entry to be used in case entry is not found in the
      * transformation catalog.
@@ -892,11 +1025,16 @@ public class SUBDAXGenerator{
      */
     private  TransformationCatalogEntry defaultTCEntry( String site ){
         //not implemented as we dont have handle to site catalog in this class
+        SiteCatalogEntry entry = mSiteStore.lookup( site );
+        if( entry != null ){
+            return constructTCEntryFromEnvProfiles( (ENV)entry.getProfiles().get(Profiles.NAMESPACES.env) );
+        }
+        
         return null;
     }
 
     /**
-     * Returns a tranformation catalog entry object constructed from the environment
+     * Returns a transformation catalog entry object constructed from the environment
      * 
      * An entry is constructed if either of the following environment variables
      * are defined
@@ -1037,12 +1175,12 @@ public class SUBDAXGenerator{
      * 
      * @return  the prescript
      */
-    public String constructPegasusPlanPrescript( Job job,
+    public Job constructPegasusPlanPrescript( Job job,
                                                  PlannerOptions options,
                                                  String rootUUID,
                                                  String properties,
                                                  String log ){
-        StringBuffer prescript = new StringBuffer();
+        //StringBuffer prescript = new StringBuffer();
 
     
         String site = job.getSiteHandle();
@@ -1091,12 +1229,9 @@ public class SUBDAXGenerator{
         //in case of deferred planning cleanup wont work
         //explicitly turn it off if the file cleanup scope if fullahead
         if( mCleanupScope.equals( PegasusProperties.CLEANUP_SCOPE.fullahead ) ){
-            options.setCleanup( false );
+            options.setCleanup( PlannerOptions.CLEANUP_OPTIONS.none );
         }
 
-        //we want monitoring to happen
-        options.setMonitoring( true );
-
         //construct the argument string.
         //add the jvm options and the pegasus options if any
         StringBuffer arguments = new StringBuffer();
@@ -1105,17 +1240,29 @@ public class SUBDAXGenerator{
                   append( " -D" ).append( PegasusProperties.ROOT_WORKFLOW_UUID_PROPERTY_KEY ).
                   append( "=" ).append( rootUUID ).
                   //add other jvm options that user may have specified
-                  append( options.toJVMOptions() ).
-                  append(" --conf ").append( properties ).
-                  //put in all the other options.
-                  append( options.toOptions());
+                  append( options.toJVMOptions() );
+        
+        //PM-667
+        //the dax jobs can have a conf option specified on the command line
+        //if that is the case then don't inherit for the sub workflow 
+        if( options.getConfFile() != null  ){
+            mLogger.log( "Not inheriting properties from the outer level workflow. DAX Job " + job.getID() + 
+                        " already has a conf option specified " + options.getConfFile(),
+                          LogManager.DEBUG_MESSAGE_LEVEL );
+        }
+        else{
+            arguments.append(" --conf ").append( properties );
+        }
+       
+        //put in all the other options.
+        arguments.append( options.toOptions());
         
         //add the --dax option explicitly in the end
         arguments.append( " --dax " ).append( options.getDAX() );
 
-        prescript.append( script ).append( " " ).append( arguments );
-        
-        return prescript.toString();
+        //prescript.append( script ).append( " " ).append( arguments );
+        job.setPreScript( script.toString(), arguments.toString() );
+        return job;
     }
 
 
@@ -1407,7 +1554,7 @@ public class SUBDAXGenerator{
         Set<String> s = new HashSet();
         
         //get the graph node corresponding to the jobs
-        GraphNode node = this.mWorkflow.getNode( job.getID() );
+        GraphNode node = this.mDAG.getNode( job.getID() );
         
         for( GraphNode parent : node.getParents() ){
             Job p = ( Job )parent.getContent();
@@ -1419,5 +1566,6 @@ public class SUBDAXGenerator{
         return s;
     }
 
+   
 
 }
