@@ -23,6 +23,7 @@ import edu.isi.pegasus.common.util.CommonProperties;
 import edu.isi.pegasus.common.util.Boolean;
 
 import edu.isi.pegasus.planner.catalog.classes.Profiles;
+import edu.isi.pegasus.planner.namespace.Dagman;
 import edu.isi.pegasus.planner.namespace.Namespace;
 import java.io.File;
 import java.io.IOException;
@@ -46,11 +47,11 @@ import java.util.Map;
  * @author Karan Vahi
  * @author Gaurang Mehta
  *
- * @version $Revision: 4935 $
+ * @version $Revision$
  *
  * @see org.griphyn.common.util.CommonProperties
  */
-public class PegasusProperties {
+public class PegasusProperties implements Cloneable {
 
     /**
      * the name of the property to disable invoke functionality
@@ -61,6 +62,23 @@ public class PegasusProperties {
 
     public static final String PEGASUS_TRANSFER_WORKER_PACKAGE_PROPERTY = "pegasus.transfer.worker.package";
 
+   
+    public static final String PEGASUS_TRANSFORMATION_CATALOG_PROPERTY = "pegasus.catalog.transformation";
+    
+    public static final String PEGASUS_TRANSFORMATION_CATALOG_FILE_PROPERTY = "pegasus.catalog.transformation.file";
+    
+    public static final String PEGASUS_REPLICA_CATALOG_PROPERTY = "pegasus.catalog.replica";
+    
+    public static final String PEGASUS_REPLICA_CATALOG_FILE_PROPERTY = "pegasus.catalog.replica.file";
+    
+    
+    public static final String PEGASUS_SITE_CATALOG_PROPERTY = "pegasus.catalog.site";
+    
+    public static final String PEGASUS_SITE_CATALOG_FILE_PROPERTY = "pegasus.catalog.site.file";
+    
+    public static final String PEGASUS_APP_METRICS_PREFIX = "pegasus.metrics.app";
+    
+    
     //Replica Catalog Constants
     public static final String DEFAULT_RC_COLLECTION = "GriphynData";
 
@@ -90,7 +108,7 @@ public class PegasusProperties {
     public static final String TC_DATA_FILE = "tc.data";
 
 
-    public static final String DEFAULT_POOL_MODE = "XML3";
+    public static final String DEFAULT_POOL_MODE = "XML";
 
     public static final String DEFAULT_CONDOR_BIN_DIR = "";
 
@@ -154,9 +172,15 @@ public class PegasusProperties {
      * The property key designated the root workflow uuid.
      */
     public static final String ROOT_WORKFLOW_UUID_PROPERTY_KEY = "pegasus.workflow.root.uuid";
-    
 
     
+    /**
+     * The default value to be assigned for dagman.maxpre .
+     */
+    public static final String DEFAULT_DAGMAN_MAX_PRE_VALUE = "1";
+
+
+
     /**
      * An enum defining The scope for cleanup algorithm
      */
@@ -168,10 +192,6 @@ public class PegasusProperties {
      */
     private static final String DEFAULT_DAX_CALLBACK = "DAX2Graph";
 
-    /**
-     * Ensures only one object is created always. Implements the Singleton.
-     */
-    private static PegasusProperties pegProperties = null;
 
 
     /**
@@ -184,15 +204,7 @@ public class PegasusProperties {
      */
     private CommonProperties mProps;
 
-    /**
-     * The Logger object.
-     */
-//    private LogManager mLogger;
 
-    /**
-     * The String containing the messages to be logged.
-     */
-    private String mLogMsg;
 
     /**
      * The default path to the transformation catalog.
@@ -360,7 +372,32 @@ public class PegasusProperties {
         }
         return profiles;
     }
-    
+
+
+    /**
+     * Returns the clone of the object.
+     *
+     * @return the clone
+     */
+    public Object clone(){
+        PegasusProperties props;
+        try{
+            //this will do a shallow clone for all member variables
+            //that is fine for the string variables
+            props = ( PegasusProperties ) super.clone();
+
+            //clone the CommonProperties
+            props.mProfiles =  ( this.mProfiles == null ) ? null :(Profiles) this.mProfiles.clone();
+            props.mProps =  ( this.mProps == null )  ?  null: (CommonProperties) this.mProps.clone();
+        }
+        catch( CloneNotSupportedException e ){
+            //somewhere in the hierarch chain clone is not implemented
+            throw new RuntimeException("Clone not implemented in the base class of " + this.getClass().getName(),
+                                       e );
+        }
+        return props;
+    }
+
     /**
      * Accessor to the bin directory of the Pegasus install
      *
@@ -477,12 +514,10 @@ public class PegasusProperties {
             //Karan April 27, 2011
             mProps = CommonProperties.nonSingletonInstance( confProperties );
         } catch ( IOException e ) {
-            mLogMsg = "unable to read property file: " + e.getMessage();
-            System.err.println( mLogMsg );
+            System.err.println( "unable to read property file: " + e.getMessage() );
             System.exit( 1 );
         } catch ( MissingResourceException e ) {
-            mLogMsg = "A required property is missing: " + e.getMessage();
-            System.err.println( mLogMsg );
+            System.err.println( "A required property is missing: " + e.getMessage() );
             System.exit( 1 );
         }
 
@@ -641,13 +676,13 @@ public class PegasusProperties {
      * Referred to by the "pegasus.dir.create.strategy" property.
      *
      * @return  the create dir classname if specified in the properties file,
-     *          else Tentacles.
+     *          else Minimal.
      */
     public String getCreateDirClass() {
         
         return getProperty( "pegasus.dir.create.strategy",
                             "pegasus.dir.create",
-                             "Tentacles" );
+                             "Minimal" );
     }
 
     /**
@@ -797,6 +832,31 @@ public class PegasusProperties {
         return mProps.getProperty( "pegasus.file.cleanup.impl", "Cleanup" );
                              
     }
+
+    /**
+     * Returns the maximum number of clean up jobs created per level of the workflow
+     * in case of InPlace cleanup.
+     *
+     * Referred to by the "pegasus.file.cleanup.clusters.num" property
+     *
+     * @return the value in the property file , else null
+     */
+    public String getMaximumCleanupJobsPerLevel() {
+        return mProps.getProperty( "pegasus.file.cleanup.clusters.num" );
+    }
+
+    /**
+     * Returns the fraction  of cleanup jobs clustered into a single clustered
+     * cleanup job.
+     *
+     * Referred to by the "pegasus.file.cleanup.clusters.size" property
+     *
+     * @return the value in the property file , else null
+     */
+    public String getClusterSizeCleanupJobsPerLevel() {
+        return mProps.getProperty( "pegasus.file.cleanup.clusters.size" );
+    }
+
     
     /**
      * Returns the scope for file cleanup. It is used to trigger cleanup in case
@@ -838,7 +898,7 @@ public class PegasusProperties {
      * @see #DEFAULT_TC_MODE
      */
     public String getTCMode() {
-        return mProps.getProperty( "pegasus.catalog.transformation", DEFAULT_TC_MODE );
+        return mProps.getProperty( PegasusProperties.PEGASUS_TRANSFORMATION_CATALOG_PROPERTY, DEFAULT_TC_MODE );
     }
 
     /**
@@ -852,7 +912,7 @@ public class PegasusProperties {
      * @see #mDefaultTC
      */
     public String getTCPath() {
-        return mProps.getProperty( "pegasus.catalog.transformation.file", mDefaultTC );
+        return mProps.getProperty( PegasusProperties.PEGASUS_TRANSFORMATION_CATALOG_FILE_PROPERTY, mDefaultTC );
     }
 
     /**
@@ -882,7 +942,7 @@ public class PegasusProperties {
      *         else null
      */
     public String getReplicaMode() {
-        return mProps.getProperty( "pegasus.catalog.replica" );
+        return mProps.getProperty( PEGASUS_REPLICA_CATALOG_PROPERTY );
     }
 
     /**
@@ -937,7 +997,7 @@ public class PegasusProperties {
     * @see #DEFAULT_POOL_MODE
     */
    public String getPoolMode() {
-       return mProps.getProperty( "pegasus.catalog.site", DEFAULT_POOL_MODE );
+       return mProps.getProperty( PegasusProperties.PEGASUS_SITE_CATALOG_PROPERTY, DEFAULT_POOL_MODE );
    }
 
    /**
@@ -951,7 +1011,7 @@ public class PegasusProperties {
     * @see #mDefaultPoolFile
     */
    public String getPoolFile() {
-       return mProps.getProperty( "pegasus.catalog.site.file",
+       return mProps.getProperty( PegasusProperties.PEGASUS_SITE_CATALOG_FILE_PROPERTY,
                                   mDefaultPoolFile );
    }
 
@@ -1013,13 +1073,13 @@ public class PegasusProperties {
      * Returns the sls transfer implementation that is to be used for constructing
      * the transfer jobs.
      *
-     * Referred to by the "pegasus.transfer.sls.*.impl" property.
+     * Referred to by the "pegasus.transfer.lite.*.impl" property.
      *
      * @return the transfer implementation
      * 
      */
     public String getSLSTransferImplementation(){
-        return getTransferImplementation( "pegasus.transfer.sls.*.impl" );
+        return getTransferImplementation( "pegasus.transfer.lite.*.impl" );
     }
 
 
@@ -1041,13 +1101,13 @@ public class PegasusProperties {
      * Returns a boolean indicating whether to stage sls files via Pegasus
      * First Level Staging or let Condor do it.
      *
-     * Referred to by the property "pegasus.transfer.stage.sls.file"
+     * Referred to by the property "pegasus.transfer.stage.lite.file"
      *
      * @return boolean value mentioned in the properties or else the default
      *         value which is true.
      */
     public boolean stageSLSFilesViaFirstLevelStaging( ){
-        return Boolean.parse( mProps.getProperty( "pegasus.transfer.stage.sls.file" ),
+        return Boolean.parse( mProps.getProperty( "pegasus.transfer.stage.lite.file" ),
                               false );
     }
 
@@ -1076,6 +1136,21 @@ public class PegasusProperties {
          return mProps.getProperty("pegasus.transfer.*.impl");
      }
 
+
+     /**
+      * Returns a boolean indicating whether to bypass first level staging of
+      * inputs. Useful in case of PegasusLite setup
+      *
+      * Referred to by the "pegasus.transfer.bypass.input.staging" property.
+      *
+      * @return boolean value specified , else false
+      */
+     public boolean bypassFirstLevelStagingForInputs( ){
+         return Boolean.parse(
+                    mProps.getProperty( "pegasus.transfer.bypass.input.staging" ),
+                    false );
+     }
+
      /**
       * Returns the default priority for the transfer jobs if specified in
       * the properties file.
@@ -1317,16 +1392,16 @@ public class PegasusProperties {
     }
     
     /**
-     * Returns the extra arguments with which the SLS transfer executables needs
-     * to be invoked.
+     * Returns the extra arguments with which the  transfer executable used in
+     * PegasusLite needs to be invoked.
      *
-     * Referred to by "pegasus.transfer.sls.arguments" property.
+     * Referred to by "pegasus.transfer.lite.arguments" property.
      *
      * @return the arguments specified in the properties file,
      *         else null if property is not specified.
      */
     public String getSLSTransferArguments() {
-	return mProps.getProperty("pegasus.transfer.sls.arguments");
+	return mProps.getProperty("pegasus.transfer.lite.arguments");
     }
     
 
@@ -1832,12 +1907,17 @@ public class PegasusProperties {
     /** 
      * Completely disable placing a symlink for Condor common log (indiscriminately). 
      * 
-     * @return true if we want a symlink for Condor common log. 
+     * Starting 4.2.1 this defaults to "false" .
+     * 
+     * Referred to by the "pegasus.condor.logs.symlink" property.
+     * 
+     * @return value specified by the property. Defaults to false.
     */ 
     public boolean symlinkCommonLog() { 
-        return Boolean.parse( mProps.getProperty( "pegasus.condor.logs.symlink"),
-                              true  ); 
-   } 
+        return Boolean.parse( mProps.getProperty( "pegasus.condor.logs.symlink" ),
+                              false  ); 
+    } 
+    
 
     /**
      * Returns a boolean indicating whether we want to Condor Quote the
@@ -1848,7 +1928,7 @@ public class PegasusProperties {
      * @return boolean
      */
     public boolean useCondorQuotingForArguments(){
-        return Boolean.parse( mProps.getProperty("pegasus.condor.arguments.quote"),
+        return Boolean.parse( mProps.getProperty( "pegasus.condor.arguments.quote" ),
                               true);
     }
 
@@ -2018,7 +2098,17 @@ public class PegasusProperties {
         return file;
     }
 
-    
+    /**
+     * Returns a boolean indicating whether to log JVM memory usage or not.
+     *
+     * Referred to by the "pegasus.log.memory.usage" property.
+     *
+     * @return boolean value specified in properties else false.
+     */
+    public boolean logMemoryUsage(){
+        return Boolean.parse( mProps.getProperty( "pegasus.log.memory.usage" ) ,
+                              false  ); 
+    }
 
     //SOME MISCELLANEOUS PROPERTIES
 
@@ -2028,26 +2118,26 @@ public class PegasusProperties {
      * 
      * Referred to by the "pegasus.job.priority.assign" property.
      *
-     * @return boolean value specified in properties else false.
+     * @return boolean value specified in properties else true.
      */
     public boolean assignDefaultJobPriorities() {
         return Boolean.parse( mProps.getProperty( "pegasus.job.priority.assign" ) ,
                               true  );
     }
 
-    
     /**
-     * Return returns the environment string specified for the local pool. If
-     * specified the registration jobs are set with these environment variables.
+     * Returns a boolean indicating whether we create registration jobs or not.
+     * 
+     * Referred to by the "pegasus.register" property.
      *
-     * Referred to by the "pegasus.local.env" property
-     *
-     * @return the environment string for local pool in properties file if
-     *         defined, else null.
+     * @return boolean value specified in properties else true.
      */
-    public String getLocalPoolEnvVar() {
-        return mProps.getProperty( "pegasus.local.env" );
+    public boolean createRegistrationJobs() {
+        return Boolean.parse( mProps.getProperty( "pegasus.register" ) ,
+                              true  );
     }
+
+    
     
     /**
      * Returns a boolean indicating whether to have jobs executing on worker
@@ -2087,6 +2177,20 @@ public class PegasusProperties {
                               false) ;
     }
 
+    /**
+     * Returns a boolean indicating whether to automatically
+     * add edges as a result of underlying data dependecnies between jobs. 
+     * 
+     * Referred to by the "pegasus.parser.dax.data.dependencies" property.
+     * 
+     * @return boolean value in the properties file, else true if not specified
+     *         or an invalid value specified.
+     */
+    public boolean addDataDependencies(){
+        return Boolean.parse( mProps.getProperty( "pegasus.parser.dax.data.dependencies" ),
+                              true) ;
+    }
+    
     /**
      * Returns the path to the wings properties file.
      * 
@@ -2415,6 +2519,16 @@ public class PegasusProperties {
             sanitizePathForProperty( properties, "pegasus.catalog.transformation.file" );
         }
 
+        //put in a sensible default for dagman maxpre for pegasus-run to
+        //pick up if not specified beforehand
+        StringBuffer buffer = new StringBuffer();
+        buffer.append( Dagman.NAMESPACE_NAME ).append( "." ).append( Dagman.MAXPRE_KEY.toLowerCase() );
+        String key = buffer.toString();
+        if( !properties.containsKey( key ) ){
+            //add defautl value
+            properties.put( key , DEFAULT_DAGMAN_MAX_PRE_VALUE );
+        }
+
         //create a temporary file in directory
         File f = File.createTempFile( "pegasus.", ".properties", dir );
 
