@@ -1,5 +1,5 @@
 /**
- *  Copyright 2007-2008 University Of Southern California
+ *  Copyright 2007-2012 University Of Southern California
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -28,7 +28,7 @@ import java.util.Set;
 /**
  *
  * @author gmehta
- * @version $Revision: 4301 $
+ * @version $Revision$
  */
 public class AbstractJob {
 
@@ -47,6 +47,7 @@ public class AbstractJob {
     protected static LogManager mLogger;
     private static final String ARG_DELIMITER = " ";
     private static final String FILE_DELIMITER = " ";
+    private static final String JOBTYPE="AbstractJob";
 
     protected AbstractJob() {
         mLogger = LogManagerFactory.loadSingletonInstance();
@@ -92,7 +93,7 @@ public class AbstractJob {
      * @return List
      */
     public List getArguments() {
-        return Collections.unmodifiableList(mArguments);
+        return mArguments;
     }
 
     /**
@@ -396,7 +397,7 @@ public class AbstractJob {
      * @see Profile
      */
     public AbstractJob addProfile(Profile profile) {
-        mProfiles.add(new Profile(profile));
+        mProfiles.add(profile);
         return this;
     }
 
@@ -406,10 +407,18 @@ public class AbstractJob {
      * @return
      */
     public AbstractJob addProfiles(List<Profile> profiles) {
-        mProfiles.addAll(Collections.unmodifiableCollection(profiles));
+        mProfiles.addAll(profiles);
         return this;
     }
 
+    /**
+     * Return the profile List. The List contains both {@link Profile} objects
+     * @return List
+     */
+    public List getProfiles() {
+        return mProfiles;
+    }
+
     /**
      * Get the STDIN file object
      * @return File
@@ -428,9 +437,6 @@ public class AbstractJob {
     public AbstractJob setStdin(File stdin) {
         File f = new File(stdin, File.LINK.INPUT);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -444,9 +450,6 @@ public class AbstractJob {
         File f = new File(stdin, File.LINK.INPUT);
         f.setTransfer(transfer);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -460,9 +463,6 @@ public class AbstractJob {
         File f = new File(stdin, File.LINK.INPUT);
         f.setRegister(register);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -479,9 +479,6 @@ public class AbstractJob {
         f.setTransfer(transfer);
         f.setRegister(register);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -499,9 +496,6 @@ public class AbstractJob {
         f.setRegister(register);
         f.setOptional(optional);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -513,9 +507,6 @@ public class AbstractJob {
     public AbstractJob setStdin(String stdin) {
         File f = new File(stdin, File.LINK.INPUT);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -529,9 +520,6 @@ public class AbstractJob {
         File f = new File(stdin, File.LINK.INPUT);
         f.setTransfer(transfer);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -545,9 +533,6 @@ public class AbstractJob {
         File f = new File(stdin, File.LINK.INPUT);
         f.setRegister(register);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -564,9 +549,6 @@ public class AbstractJob {
         f.setTransfer(transfer);
         f.setRegister(register);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -585,9 +567,6 @@ public class AbstractJob {
         f.setRegister(register);
         f.setOptional(optional);
         mStdin = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -607,9 +586,6 @@ public class AbstractJob {
     public AbstractJob setStdout(File stdout) {
         File f = new File(stdout, File.LINK.OUTPUT);
         mStdout = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -623,9 +599,6 @@ public class AbstractJob {
         File f = new File(stdout, File.LINK.OUTPUT);
         f.setTransfer(transfer);
         mStdout = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -639,9 +612,6 @@ public class AbstractJob {
         File f = new File(stdout, File.LINK.OUTPUT);
         f.setRegister(register);
         mStdout = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -658,9 +628,6 @@ public class AbstractJob {
         f.setTransfer(transfer);
         f.setRegister(register);
         mStdout = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -679,9 +646,6 @@ public class AbstractJob {
         f.setRegister(register);
         f.setOptional(optional);
         mStdout = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -693,9 +657,6 @@ public class AbstractJob {
     public AbstractJob setStdout(String stdout) {
         File f = new File(stdout, File.LINK.OUTPUT);
         mStdout = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -709,9 +670,6 @@ public class AbstractJob {
         File f = new File(stdout, File.LINK.OUTPUT);
         f.setTransfer(transfer);
         mStdout = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -725,9 +683,6 @@ public class AbstractJob {
         File f = new File(stdout, File.LINK.OUTPUT);
         f.setRegister(register);
         mStdout = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -763,9 +718,6 @@ public class AbstractJob {
         f.setRegister(register);
         f.setOptional(optional);
         mStdout = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -786,9 +738,6 @@ public class AbstractJob {
     public AbstractJob setStderr(File stderr) {
         File f = new File(stderr, File.LINK.OUTPUT);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -802,9 +751,6 @@ public class AbstractJob {
         File f = new File(stderr, File.LINK.OUTPUT);
         f.setTransfer(transfer);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -818,9 +764,6 @@ public class AbstractJob {
         File f = new File(stderr, File.LINK.OUTPUT);
         f.setRegister(register);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -837,9 +780,6 @@ public class AbstractJob {
         f.setTransfer(transfer);
         f.setRegister(register);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -858,9 +798,6 @@ public class AbstractJob {
         f.setRegister(register);
         f.setOptional(optional);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -872,9 +809,6 @@ public class AbstractJob {
     public AbstractJob setStderr(String stderr) {
         File f = new File(stderr, File.LINK.OUTPUT);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -888,9 +822,6 @@ public class AbstractJob {
         File f = new File(stderr, File.LINK.OUTPUT);
         f.setTransfer(transfer);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -904,9 +835,6 @@ public class AbstractJob {
         File f = new File(stderr, File.LINK.OUTPUT);
         f.setRegister(register);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -923,9 +851,6 @@ public class AbstractJob {
         f.setTransfer(transfer);
         f.setRegister(register);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -944,9 +869,6 @@ public class AbstractJob {
         f.setRegister(register);
         f.setOptional(optional);
         mStderr = f;
-        if (!mUses.contains(f)) {
-            mUses.add(f);
-        }
         return this;
     }
 
@@ -955,7 +877,7 @@ public class AbstractJob {
      * @return Set<File>
      */
     public Set<File> getUses() {
-        return Collections.unmodifiableSet(mUses);
+        return mUses;
     }
 
     /**
@@ -998,7 +920,49 @@ public class AbstractJob {
         }
         return this;
     }
+    
+    /**
+     *
+     * @param file
+     * @param link
+     * @param register
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(String file, File.LINK link, boolean register,String size) {
+        File f = new File(file, link);
+        f.setRegister(register);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion) + "already contains a file " + Separator.
+                    combine(f.mNamespace, f.mName, f.mVersion) + ". Ignoring",
+                    LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
+    }
 
+    /**
+     *
+     * @param file
+     * @param link
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(String file, File.LINK link, String size) {
+        File f = new File(file, link);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion) + "already contains a file " + Separator.
+                    combine(f.mNamespace, f.mName, f.mVersion) + ". Ignoring",
+                    LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
+    }
+    
     /**
      *
      * @param file
@@ -1019,6 +983,28 @@ public class AbstractJob {
         return this;
     }
 
+    /**
+     *
+     * @param file
+     * @param link
+     * @param transfer
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(String file, File.LINK link, File.TRANSFER transfer, String size) {
+        File f = new File(file, link);
+        f.setTransfer(transfer);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion) + "already contains a file " + Separator.
+                    combine(f.mNamespace, f.mName, f.mVersion) + ". Ignoring",
+                    LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
+    }
+    
     /**
      *
      * @param file
@@ -1041,6 +1027,31 @@ public class AbstractJob {
         }
         return this;
     }
+    
+    /**
+     *
+     * @param file
+     * @param link
+     * @param transfer
+     * @param register
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(String file, File.LINK link, File.TRANSFER transfer,
+            boolean register, String size) {
+        File f = new File(file, link);
+        f.setRegister(register);
+        f.setTransfer(transfer);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion) + "already contains a file " + Separator.
+                    combine(f.mNamespace, f.mName, f.mVersion) + ". Ignoring",
+                    LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
+    }
 
     /**
      * 
@@ -1069,6 +1080,36 @@ public class AbstractJob {
         }
         return this;
     }
+    
+    /**
+     * 
+     * @param file
+     * @param link
+     * @param transfer
+     * @param register
+     * @param optional
+     * @param executable
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(String file, File.LINK link, File.TRANSFER transfer,
+            boolean register, boolean optional, boolean executable, String size) {
+        File f = new File(file, link);
+        f.setRegister(register);
+        f.setOptional(optional);
+        f.setTransfer(transfer);
+        f.setExecutable(executable);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion)
+                    + "already contains a file " + Separator.combine(
+                    f.mNamespace, f.mName, f.mVersion)
+                    + ". Ignoring", LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
+    }
 
     /**
      *
@@ -1109,6 +1150,30 @@ public class AbstractJob {
         }
         return this;
     }
+    
+        /**
+     * 
+     * @param file
+     * @param link
+     * @param transfer
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(File file, File.LINK link, File.TRANSFER transfer,String size) {
+        File f = new File(file, link);
+        f.setTransfer(transfer);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion)
+                    + "already contains a file " + Separator.combine(
+                    f.mNamespace, f.mName, f.mVersion)
+                    + ". Ignoring", LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
+    }
+
 
     /**
      * 
@@ -1132,7 +1197,55 @@ public class AbstractJob {
 
 
     }
+    
+       /**
+     * 
+     * @param file
+     * @param link
+     * @param register
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(File file, File.LINK link, boolean register, String size) {
+        File f = new File(file, link);
+        f.setRegister(register);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion)
+                    + "already contains a file "
+                    + Separator.combine(f.mNamespace, f.mName, f.mVersion)
+                    + ". Ignoring", LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
 
+
+    }
+
+    /**
+     * 
+     * @param file
+     * @param link
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(File file, File.LINK link, String size) {
+        File f = new File(file, link);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion)
+                    + "already contains a file "
+                    + Separator.combine(f.mNamespace, f.mName, f.mVersion)
+                    + ". Ignoring", LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
+
+
+    }
+    
     /**
      *
      * @param file
@@ -1156,6 +1269,33 @@ public class AbstractJob {
         }
         return this;
     }
+    
+        
+    /**
+     *
+     * @param file
+     * @param link
+     * @param transfer
+     * @param register
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(File file, File.LINK link, File.TRANSFER transfer,
+            boolean register, String size) {
+        File f = new File(file, link);
+        f.setTransfer(transfer);
+        f.setRegister(register);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion)
+                    + "already contains a file " + Separator.combine(
+                    f.mNamespace, f.mName, f.mVersion)
+                    + ". Ignoring", LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
+    }
 
     /**
      *
@@ -1184,6 +1324,37 @@ public class AbstractJob {
         }
         return this;
     }
+    
+    
+    /**
+     *
+     * @param file
+     * @param link
+     * @param transfer
+     * @param register
+     * @param optional
+     * @param executable
+     * @param size
+     * @return AbstractJob
+     */
+    public AbstractJob uses(File file, File.LINK link, File.TRANSFER transfer,
+            boolean register, boolean optional, boolean executable, String size) {
+        File f = new File(file, link);
+        f.setTransfer(transfer);
+        f.setRegister(register);
+        f.setOptional(optional);
+        f.setExecutable(executable);
+        f.setSize(size);
+        if (!mUses.contains(f)) {
+            mUses.add(f);
+        } else {
+            mLogger.log("Job " + Separator.combine(mNamespace, mName, mVersion)
+                    + "already contains a file " + Separator.combine(
+                    f.mNamespace, f.mName, f.mVersion)
+                    + ". Ignoring", LogManager.WARNING_MESSAGE_LEVEL);
+        }
+        return this;
+    }
 
     /**
      *
@@ -1208,13 +1379,22 @@ public class AbstractJob {
     }
 
     /**
-     *
+     * Returns list of Invoke objects
      * @return List<Invoke>
      */
     public List<Invoke> getInvoke() {
-        return Collections.unmodifiableList(mInvokes);
+        return mInvokes;
     }
 
+    
+    /**
+     * Same as getInvoke(). Returns list of Invoke objects
+     * @return List<Invoke>
+     */
+    public List<Invoke> getNotification() {
+        return getInvoke();
+    }
+    
     /**
      * Add Notification to the job
      * @param when
@@ -1392,16 +1572,48 @@ public class AbstractJob {
         } //add profiles
         for (Profile p : mProfiles) {
             p.toXML(writer, indent + 1);
-        } //add stdin
+        } 
+        
+        //PM-708 add extra uses for stdout|stderr|stdin if not 
+        //specified by the user in the uses section
+        Set<File> addOnUses = new LinkedHashSet<File>();
+        
+        //add stdin
         if (mStdin != null) {
+            if( !mUses.contains( mStdin) ){
+                //add uses with default flags
+                File f = new File( mStdin, File.LINK.INPUT);
+                addOnUses.add(f);
+            }
             mStdin.toXML(writer, indent + 1, "stdin");
-        } //add stdout
+        } 
+        
+        //add stdout
         if (mStdout != null) {
+            if( !mUses.contains( mStdout) ){
+                //add uses with default flags
+                File f = new File( mStdout, File.LINK.OUTPUT);
+                addOnUses.add(f);
+            }
             mStdout.toXML(writer, indent + 1, "stdout");
-        } //add stderr
+        } 
+        
+        //add stderr
         if (mStderr != null) {
+            if( !mUses.contains( mStderr) ){
+                //add uses with default flags
+                File f = new File( mStderr, File.LINK.OUTPUT);
+                addOnUses.add(f);
+            }
             mStderr.toXML(writer, indent + 1, "stderr");
-        } //add uses
+        } 
+        
+        //print add on uses first
+        for (File f : addOnUses) {
+            f.toXML(writer, indent + 1, "uses");
+        }
+        
+        //add uses
         for (File f : mUses) {
             f.toXML(writer, indent + 1, "uses");
         } //add invoke
