@@ -17,55 +17,37 @@
 
 package edu.isi.pegasus.planner.refiner;
 
+import edu.isi.pegasus.common.logging.LogManager;
+import edu.isi.pegasus.common.util.Separator;
+import edu.isi.pegasus.planner.catalog.site.classes.FileServer;
+import edu.isi.pegasus.planner.catalog.site.classes.FileServerType.OPERATION;
 import edu.isi.pegasus.planner.catalog.site.classes.GridGateway;
 import edu.isi.pegasus.planner.catalog.site.classes.SiteCatalogEntry;
-
+import edu.isi.pegasus.planner.catalog.transformation.Mapper;
+import edu.isi.pegasus.planner.catalog.transformation.TransformationCatalogEntry;
+import edu.isi.pegasus.planner.catalog.transformation.classes.TCType;
+import edu.isi.pegasus.planner.catalog.transformation.classes.TransformationStore;
 import edu.isi.pegasus.planner.classes.ADag;
 import edu.isi.pegasus.planner.classes.FileTransfer;
-import edu.isi.pegasus.planner.classes.PegasusFile;
 import edu.isi.pegasus.planner.classes.Job;
 import edu.isi.pegasus.planner.classes.PegasusBag;
-
-
-
-
-import edu.isi.pegasus.common.logging.LogManager;
-
-import edu.isi.pegasus.planner.selector.SiteSelector;
-
-import edu.isi.pegasus.planner.selector.site.SiteSelectorFactory;
-
-import edu.isi.pegasus.planner.selector.TransformationSelector;
-
-import edu.isi.pegasus.planner.namespace.Hints;
-
-import edu.isi.pegasus.planner.provenance.pasoa.XMLProducer;
-import edu.isi.pegasus.planner.provenance.pasoa.producer.XMLProducerFactory;
-
-import edu.isi.pegasus.planner.provenance.pasoa.PPS;
-import edu.isi.pegasus.planner.provenance.pasoa.pps.PPSFactory;
-
-
-import edu.isi.pegasus.planner.catalog.transformation.TransformationCatalogEntry;
-
-
-import edu.isi.pegasus.planner.catalog.transformation.classes.TCType;
-
-import edu.isi.pegasus.planner.catalog.transformation.Mapper;
-
-import edu.isi.pegasus.common.util.Separator;
-
-import edu.isi.pegasus.planner.catalog.transformation.classes.TransformationStore;
+import edu.isi.pegasus.planner.classes.PegasusFile;
 import edu.isi.pegasus.planner.common.PegRandom;
+import edu.isi.pegasus.planner.namespace.Hints;
+import edu.isi.pegasus.planner.partitioner.graph.GraphNode;
+import edu.isi.pegasus.planner.provenance.pasoa.PPS;
+import edu.isi.pegasus.planner.provenance.pasoa.XMLProducer;
+import edu.isi.pegasus.planner.provenance.pasoa.pps.PPSFactory;
+import edu.isi.pegasus.planner.provenance.pasoa.producer.XMLProducerFactory;
+import edu.isi.pegasus.planner.selector.SiteSelector;
+import edu.isi.pegasus.planner.selector.TransformationSelector;
+import edu.isi.pegasus.planner.selector.site.SiteSelectorFactory;
 import edu.isi.pegasus.planner.transfer.SLS;
-
 import edu.isi.pegasus.planner.transfer.sls.SLSFactory;
-
 import java.io.File;
-
+import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
-import java.util.ArrayList;
 import java.util.Set;
 import java.util.Vector;
 /**
@@ -74,11 +56,16 @@ import java.util.Vector;
  *
  * @author Karan Vahi
  * @author Gaurang Mehta
- * @version $Revision: 5256 $
+ * @version $Revision$
  *
  */
 public class InterPoolEngine extends Engine implements Refiner {
 
+    /**
+     * The name of the refiner for purposes of error logging
+     */
+    public static final String REFINER_NAME = "InterPoolEngine";
+    
     /**
      * ADag object corresponding to the Dag whose jobs we want to schedule.
      *
@@ -231,10 +218,7 @@ public class InterPoolEngine extends Engine implements Refiner {
      *
      */
     public void determineSites() {
-        Job job;
-
         //at present we schedule the whole workflow at once
-        List jobs = convertToList( mDag.vJobSubInfos );
         List pools = convertToList( mExecPools );
 
         //going through all the jobs making up the Adag, to do the physical mapping
@@ -294,21 +278,22 @@ public class InterPoolEngine extends Engine implements Refiner {
         //Iterate through the jobs and hand them to
         //the site selector if required
         String site ;
-        for( Iterator it = dag.jobIterator(); it.hasNext(); i++ ){
-
-            Job job = ( Job ) it.next();
+        for( Iterator<GraphNode> it = dag.jobIterator(); it.hasNext(); i++ ){
+            GraphNode node = it.next();
+            Job job = ( Job )node.getContent();
             site  = job.getSiteHandle();
             mLogger.log( "Mapping Job "  + job.getName(), 
                          LogManager.DEBUG_MESSAGE_LEVEL );
 
 
             //check if the user has specified any hints in the dax
-            incorporateHint(job, Hints.JOBMANAGER_UNIVERSE_KEY  );
             if (incorporateHint(job, "executionPool")) {
                 //i++;
-                incorporateProfiles(job);
-                //set the staging site for the job
+                //the staging site needs to be set before any
+                //profiles are incorporated PM-618
                 job.setStagingSiteHandle( getStagingSite( job ) );
+
+                incorporateProfiles(job);
                 continue;
             }
 
@@ -409,6 +394,8 @@ public class InterPoolEngine extends Engine implements Refiner {
         return (ss == null) ? job.getSiteHandle(): ss;
     }
 
+    
+
     /**
      * Incorporates the profiles from the various sources into the job.
      * The profiles are incorporated in the order pool, transformation catalog,
@@ -511,43 +498,32 @@ public class InterPoolEngine extends Engine implements Refiner {
             //pool where it needs to be staged to
             //always creating a third party transfer URL
             //for the destination.
-            String stagedPath =  mSiteStore.getInternalWorkDirectory( job, true )
-                                + File.separator + job.getStagedExecutableBaseName();
+//            String externalStagedPath =  mSiteStore.getInternalWorkDirectory( job, true )
+//                                + File.separator + job.getStagedExecutableBaseName();
+
+            StringBuffer externalStagedPath = new StringBuffer();
+
+
+            //PM-590 Stricter checks
+//            String headnodeURLPrefix = this.selectHeadNodeScratchSharedFileServerURLPrefix( site );
+            FileServer headNodeScratchServer = site.selectHeadNodeScratchSharedFileServer( FileServer.OPERATION.put );
+
+            if( headNodeScratchServer == null ){
+                this.complainForHeadNodeURLPrefix( REFINER_NAME,  site.getSiteHandle() , FileServer.OPERATION.put, job );
+            }
+            externalStagedPath.append( headNodeScratchServer.getURLPrefix() ).
+                       append( mSiteStore.getExternalWorkDirectory(headNodeScratchServer, site.getSiteHandle() )).
+                       append( File.separator ).append(  job.getStagedExecutableBaseName());
 
             fTx.addDestination( stagingSiteHandle,
-                                site.getHeadNodeFS().selectScratchSharedFileServer().getURLPrefix() + stagedPath);
+                                externalStagedPath.toString() );
 
-            //added in the end now after dependant executables
-            //have been handled Karan May 31 2007
-            //job.addInputFile(fTx);
 
-            /* Karan April 27, 2012. No longer required
-            if( mWorkerNodeExecution ){
-                //do not specify the full path as we do not know worker
-                //node directory
+            //the internal path needs to be set for the executable
+            String internalStagedPath =  mSiteStore.getInternalWorkDirectory( job, true )
+                                + File.separator + job.getStagedExecutableBaseName();
 
-                
-                if( mSLS.doesCondorModifications() ){
-                    //we need to take the basename of the source url
-                    //as condor file transfer mech does not allow to
-                    //specify destination filenames
-                    job.setRemoteExecutable( new File( tcEntry.getPhysicalTransformation() ).getName() );
-                }
-                else{
-                  
-                    //do this only when kickstart executable existance check is fixed
-                    //Karan Nov 30 2007
-                    //job.setRemoteExecutable(job.getStagedExecutableBaseName());
-                    job.setRemoteExecutable(  stagedPath );
-                }
-            }
-            else{
-                //the jobs executable is the path to where
-                //the executable is going to be staged
-                job.executable = stagedPath;
-            }
-            */
-            job.setRemoteExecutable(  stagedPath );
+            job.setRemoteExecutable(  internalStagedPath );
             
             //setting the job type of the job to
             //denote the executable is being staged
@@ -633,10 +609,8 @@ public class InterPoolEngine extends Engine implements Refiner {
                         throw new RuntimeException(error.toString());
                     }
 
-                    //            tcEntry = (TransformationCatalogEntry) tcEntries.get(0);
+                    
                     if (tcEntry.getType().equals(TCType.STAGEABLE )) {
-//                        SiteInfo site = mPoolHandle.getPoolEntry(siteHandle,
-//                            "vanilla");
                         
                         SiteCatalogEntry site = mSiteStore.lookup( stagingSiteHandle );
                         //construct a file transfer object and add it
@@ -648,6 +622,7 @@ public class InterPoolEngine extends Engine implements Refiner {
                         FileTransfer fTx = new FileTransfer( basename,
                                                              job.jobName );
                         fTx.setType(FileTransfer.EXECUTABLE_FILE);
+                        
                         //the physical transformation points to
                         //guc or the user specified transfer mechanism
                         //accessible url
@@ -656,23 +631,30 @@ public class InterPoolEngine extends Engine implements Refiner {
                         //the destination url is the working directory for
                         //pool where it needs to be staged to
                         //always creating a third party transfer URL
-                        //for the destination.
-//                        String stagedPath = mPoolHandle.getExecPoolWorkDir(job)
-//                            + File.separator + basename;
-//                        fTx.addDestination(siteHandle,
-//                                           site.getURLPrefix(false) + stagedPath);
-                        
+                        //for the destination.                        
                         String stagedPath = mSiteStore.getInternalWorkDirectory( job, true )
                             + File.separator + basename;
-                        fTx.addDestination(siteHandle,
-                                           site.getHeadNodeFS().selectScratchSharedFileServer().getURLPrefix() + stagedPath);
 
+                        //PM-590 Stricter checks
+                        //PM-686 construct the URL using the externally accessible path for work directory
+                        StringBuffer externalStagedPath = new StringBuffer();
+                        FileServer headNodeScratchServer = site.selectHeadNodeScratchSharedFileServer( FileServer.OPERATION.put );
+
+                        if( headNodeScratchServer == null ){
+                            this.complainForHeadNodeURLPrefix( REFINER_NAME,  site.getSiteHandle() , FileServer.OPERATION.put, job );
+                        }
+                        externalStagedPath.append( headNodeScratchServer.getURLPrefix() ).
+                                           append( mSiteStore.getExternalWorkDirectory(headNodeScratchServer, site.getSiteHandle() )).
+                                           append( File.separator ).append(  basename );
+
+                        fTx.addDestination( stagingSiteHandle,
+                                            externalStagedPath.toString() );
 
                         dependantExecutables.add( fTx );
 
                         //the jobs executable is the path to where
                         //the executable is going to be staged
-                        //job.executable = stagedPath;
+                        //job.executable = externalStagedPath;
                         mLogger.log( "Dependant Executable " + input.getLFN() + " being staged from " +
                                      fTx.getSourceURL(), LogManager.DEBUG_MESSAGE_LEVEL );
 
@@ -780,32 +762,7 @@ public class InterPoolEngine extends Engine implements Refiner {
                     return true;
                 }
                 break;
-/*
-            case 'g':
-                if (key.equals("globusScheduler")) {
-                    if( job.hints.containsKey( Hints.GLOBUS_SCHEDULER_KEY ) ){
-                        //user specified the jobmanager on which they want 
-                        //the job to execute on
-                        job.globusScheduler = (String) job.hints.removeKey("globusScheduler");
-                    }
-                    else{
-                        //try to lookup in the site catalog
-                        SiteCatalogEntry s = mSiteStore.lookup( job.getSiteHandle() );
-                        if( s == null ){
-                            throw new RuntimeException( "Unable to find entry for site in site catalog " + job.getSiteHandle() );
-                        }
-                        GridGateway gw = s.selectGridGateway( 
-                                                    GridGateway.JOB_TYPE.valueOf(job.condorUniverse));
-                        if( gw == null ){
-                            throw new RuntimeException( 
-                                    "No GridGateway specified for compute jobs for site  " + job.getSiteHandle() );
-                        }
-                        job.globusScheduler =  gw.getContact();
-                    }
-                    return true;
-                }
-                break;
-*/
+
             
             case 'p':
 
@@ -820,7 +777,10 @@ public class InterPoolEngine extends Engine implements Refiner {
 
                 }
                 break;
-                
+
+            // the hint Hints.JOBMANAGER_UNIVERSE_KEY is handled in
+            // Job class getGridGatewayJobType
+            /*
             case 'j':
                 if (key.equals( Hints.JOBMANAGER_UNIVERSE_KEY  )) {
                  job.condorUniverse = job.hints.containsKey( Hints.JOBMANAGER_UNIVERSE_KEY  ) ?
@@ -831,7 +791,7 @@ public class InterPoolEngine extends Engine implements Refiner {
 
              }
              break;
-
+             */
 
             default:
                 break;
