@@ -17,8 +17,9 @@
 package edu.isi.pegasus.planner.refiner;
 
 
+import edu.isi.pegasus.common.credential.CredentialHandler;
+import edu.isi.pegasus.common.credential.CredentialHandlerFactory;
 import edu.isi.pegasus.common.logging.LoggingKeys;
-import edu.isi.pegasus.planner.catalog.classes.Profiles;
 
 import edu.isi.pegasus.planner.catalog.site.classes.GridGateway;
 import edu.isi.pegasus.planner.catalog.site.classes.SiteCatalogEntry;
@@ -35,19 +36,12 @@ import edu.isi.pegasus.planner.classes.PlannerOptions;
 import edu.isi.pegasus.common.logging.LogManager;
 import edu.isi.pegasus.planner.common.PegasusProperties;
 
-import edu.isi.pegasus.planner.namespace.ENV;
-
 import edu.isi.pegasus.planner.catalog.ReplicaCatalog;
-import edu.isi.pegasus.planner.catalog.transformation.TransformationCatalogEntry;
-
 import edu.isi.pegasus.planner.catalog.replica.ReplicaFactory;
 
-
-
+import edu.isi.pegasus.planner.catalog.transformation.TransformationCatalogEntry;
 import edu.isi.pegasus.planner.catalog.transformation.classes.TCType;
 
-import edu.isi.pegasus.common.util.Separator;
-
 import java.io.File;
 import java.io.FileWriter;
 
@@ -67,7 +61,7 @@ import edu.isi.pegasus.planner.namespace.Dagman;
  *
  * @author Karan Vahi
  * @author Gaurang Mehta
- * @version $Revision: 4736 $
+ * @version $Revision$
  *
  */
 public class ReplicaCatalogBridge
@@ -128,6 +122,18 @@ public class ReplicaCatalogBridge
      */
     public static final String CACHE_READ_ONLY_KEY = "read.only";
 
+    
+    /**
+     * The name of the Replica Catalog Implementer that serves as the source for
+     * cache files.
+     */
+    public static final String DIRECTORY_REPLICA_CATALOG_IMPLEMENTER = "Directory";
+
+    /**
+     * The name of the source key for Replica Catalog Implementer that serves as
+     * cache
+     */
+    public static final String DIRECTORY_REPLICA_CATALOG_KEY = "directory";
 
     /**
      * The name of the URL key for the replica catalog impelementer to be picked
@@ -167,6 +173,12 @@ public class ReplicaCatalogBridge
      * the cache replica catalogs.
      */
     private ReplicaStore mCacheStore;
+    
+    /**
+     * The replica store where we store all the results that are queried from
+     * the input directory specified at the command line.
+     */
+    private ReplicaStore mDirectoryReplicaStore;
 
 
     /**
@@ -187,12 +199,6 @@ public class ReplicaCatalogBridge
      */
     private boolean mTreatCacheAsRC;
 
-    /**
-     * The namespace object holding the environment variables for local
-     * pool.
-     */
-    private ENV mLocalEnv;
-
     /**
      * The default tc entry.
      */
@@ -218,8 +224,6 @@ public class ReplicaCatalogBridge
      *
      */
     public ReplicaCatalogBridge( ADag dag ,
-//                                 PegasusProperties properties,
-//                                 PlannerOptions options 
                                  PegasusBag bag ) {
         super( bag );
         this.initialize( dag, bag );
@@ -257,17 +261,14 @@ public class ReplicaCatalogBridge
         mRCDown = false;
         mCacheStore = new ReplicaStore();
         mInheritedReplicaStore = new ReplicaStore();
+        mDirectoryReplicaStore = new ReplicaStore();
         mTreatCacheAsRC = mProps.treatCacheAsRC();
         mDefaultTCRCCreated = false;
 
         //converting the Vector into vector of
         //strings just containing the logical
         //filenames
-        mSearchFiles = dag.dagInfo.getLFNs( options.getForce() );
-
-        //load the local environment variable
-        //from pool config and property file
-        mLocalEnv = loadLocalEnvVariables();
+        mSearchFiles = dag.getDAGInfo().getLFNs( options.getForce() );
 
         //only for windward for time being
         properties.setProperty( "pegasus.catalog.replica.dax.id", dag.getAbstractWorkflowName() );
@@ -279,10 +280,27 @@ public class ReplicaCatalogBridge
             //Karan May 1 2007
             mReplicaCatalog = null;
             if ( mSearchFiles != null && !mSearchFiles.isEmpty() ){
-               mReplicaCatalog = ReplicaFactory.loadInstance(properties);
 
-               //load all the mappings.
-               mReplicaStore = new ReplicaStore( mReplicaCatalog.lookup( mSearchFiles ) );
+                //need to clone before setting any read only properites
+                PegasusProperties props = (PegasusProperties) properties.clone();
+
+                //set the read only property for the file based rc
+                //we are connecting via PegasusProperties add the prefix
+                String name =  ReplicaCatalog.c_prefix + "." + ReplicaCatalogBridge.CACHE_READ_ONLY_KEY;
+                props.setProperty( name, "true" );
+                
+                String proxy = getPathToLocalProxy();
+                if( proxy != null ){
+                    mLogger.log( "Proxy used for Replica Catalog is " + proxy,
+                                 LogManager.CONFIG_MESSAGE_LEVEL );
+                    props.setProperty( ReplicaCatalog.c_prefix + "." + ReplicaCatalog.PROXY_KEY, 
+                                       proxy );
+                }
+                
+                mReplicaCatalog = ReplicaFactory.loadInstance( props );          
+
+                //load all the mappings.
+                mReplicaStore = new ReplicaStore( mReplicaCatalog.lookup( mSearchFiles ) );
             }
 
         } catch ( Exception ex ) {
@@ -296,7 +314,10 @@ public class ReplicaCatalogBridge
             //exit if there is no cache overloading specified.
             if ( options.getCacheFiles().isEmpty() &&       //no cache files specified
                  options.getInheritedRCFiles().isEmpty() && //no files locations inherited from outer level DAX
-                 this.mDAXReplicaStore.isEmpty() ) {        //no file locations in current DAX
+                 this.mDAXReplicaStore.isEmpty() &&         //no file locations in current DAX
+                 options.getInputDirectory() == null  && //no input directory specified on the command line
+                 dag.getDAGInfo().getLFNs( true ).size() > 0 //the number of raw input files is more than 1
+                    ){
                 mLogger.log( msg + ex.getMessage(),LogManager.ERROR_MESSAGE_LEVEL );
                 throw new RuntimeException( msg , ex );
             }
@@ -322,6 +343,12 @@ public class ReplicaCatalogBridge
                 mProps.setProperty( key, "1" );
             }
         }
+        
+        //incorporate all mappings from input directory if specified
+        String input = options.getInputDirectory();
+        if( input != null ){
+            mDirectoryReplicaStore = getReplicaStoreFromDirectory( input );
+        }
             
         //incorporate the caching if any
         if ( !options.getCacheFiles().isEmpty() ) {
@@ -372,6 +399,13 @@ public class ReplicaCatalogBridge
                     mSearchFiles.size(),
                     LogManager.DEBUG_MESSAGE_LEVEL);
 
+        //check if any exist in input directory
+        lfnsFound.addAll( this.mDirectoryReplicaStore.getLFNs( mSearchFiles ) );
+        mLogger.log(lfnsFound.size()  + " entries found in cache of total " +
+                    mSearchFiles.size(),
+                    LogManager.DEBUG_MESSAGE_LEVEL);
+
+        
         //check in the main replica catalog
         if ( this.mDAXReplicaStore.isEmpty() &&
                 ( mRCDown || mReplicaCatalog == null )) {
@@ -416,6 +450,11 @@ public class ReplicaCatalogBridge
                          " retrieved from cache" , LogManager.DEBUG_MESSAGE_LEVEL);
             return rl;
         }
+        
+        //we prefer location in Directory over the DAX entries
+        if( this.mDirectoryReplicaStore.containsLFN( lfn ) ){
+            return this.mDirectoryReplicaStore.getReplicaLocation(lfn);
+        }
 
 
         //we prefer location in DAX over the inherited replica store
@@ -773,15 +812,13 @@ public class ReplicaCatalogBridge
         StringBuffer sb = new StringBuffer();
         NameValue destURL = ft.getDestURL();
         sb.append( ft.getLFN() ).append( " " );
-        sb.append( destURL.getValue()  ).append( " " );
+        sb.append( ft.getURLForRegistrationOnDestination()  ).append( " " );
         sb.append( "pool=\"" ).append( destURL.getKey() ).append( "\"" );
         sb.append( "\n" );
         return sb.toString();
     }
 
-
-
-
+    
     /**
      * Retrieves a location from the cache table, that contains the contents
      * of the cache files specified at runtime.
@@ -869,29 +906,63 @@ public class ReplicaCatalogBridge
         mLogger.logEventCompletion();
         return store;
     }
+    
+    /**
+     * Loads the mappings from the input directory 
+     * 
+     * @param directory  the directory to load from
+     */
+    private ReplicaStore getReplicaStoreFromDirectory(String directory) {
+        ReplicaStore store = new ReplicaStore();
+        Properties properties = mProps.getVDSProperties().matchingSubset(
+                                                              ReplicaCatalog.c_prefix,
+                                                              false );
+
+        mLogger.logEventStart( LoggingKeys.EVENT_PEGASUS_LOAD_DIRECTORY_CACHE, 
+                               LoggingKeys.DAX_ID,
+                               mDag.getAbstractWorkflowName() );
+
+        ReplicaCatalog catalog = null;
+        
+        //set the appropriate property to designate path to file
+        properties.setProperty( ReplicaCatalogBridge.DIRECTORY_REPLICA_CATALOG_KEY, directory );
+
+            
+        mLogger.log("Loading  from directory: " + directory,  LogManager.DEBUG_MESSAGE_LEVEL);
+        try{
+            catalog = ReplicaFactory.loadInstance( DIRECTORY_REPLICA_CATALOG_IMPLEMENTER,
+                                                   properties );
+            
+            
+            store.add( catalog.lookup( mSearchFiles ) );
+        }
+        catch( Exception e ){
+            mLogger.log( "Unable to load from directory  " + directory,
+                             e,
+                             LogManager.ERROR_MESSAGE_LEVEL );
+        }
+        finally{
+            if( catalog != null ){
+                catalog.close();
+            }
+        }
+        
+        mLogger.logEventCompletion();
+        return store;
+    }
+
 
     /**
-     * Reads in the environment variables into memory from the properties file
-     * and the pool catalog.
-     *
-     * @return  the <code>ENV</code> namespace object holding the environment
-     *          variables.
+     * Returns path to the local proxy
+     * 
+     * @return path to the local proxy 
      */
-    private ENV loadLocalEnvVariables() {
-        //assumes that pool handle, and property handle are initialized.
-        ENV env = new ENV();
-
-        //load from the pool.config
-//        env.checkKeyInNS( mPoolHandle.getPoolProfile( "local", Profile.ENV ) );
-        SiteCatalogEntry local = mSiteStore.lookup( "local" );        
-        env.checkKeyInNS( local.getProfiles().get( Profiles.NAMESPACES.env ) );
-        //load from property file
-        env.checkKeyInNS( mProps.getLocalPoolEnvVar() );
-
-        // the new RC API has a different key. if that is specified use that.
-        //mProps.getProperty( ReplicaCatalog.c_prefix )
-
-        return env;
+    private String getPathToLocalProxy() {
+        //load and intialize the CredentialHandler Factory
+        CredentialHandlerFactory factory = new CredentialHandlerFactory();
+        factory.initialize( mBag );
+        CredentialHandler handler = factory.loadInstance(CredentialHandler.TYPE.x509);
+        return handler.getPath( "local" );
     }
 
 
