@@ -21,14 +21,11 @@ package edu.isi.pegasus.planner.selector.site;
 import edu.isi.pegasus.common.logging.LogManagerFactory;
 import edu.isi.pegasus.planner.catalog.site.classes.FileServer;
 import edu.isi.pegasus.planner.catalog.site.classes.SiteCatalogEntry;
-import edu.isi.pegasus.planner.catalog.site.impl.old.classes.GridFTPServer;
 import edu.isi.pegasus.planner.classes.PegasusFile;
 import edu.isi.pegasus.planner.classes.Job;
 
 import edu.isi.pegasus.common.logging.LogManager;
-import edu.isi.pegasus.planner.common.PegasusProperties;
-
-import edu.isi.pegasus.planner.catalog.site.impl.old.PoolInfoProvider;
+import edu.isi.pegasus.planner.catalog.site.classes.Directory;
 
 
 import java.io.BufferedReader;
@@ -44,7 +41,6 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.StringTokenizer;
-import edu.isi.pegasus.planner.partitioner.graph.Adapter;
 import edu.isi.pegasus.planner.classes.ADag;
 import edu.isi.pegasus.planner.classes.PegasusBag;
 
@@ -178,7 +174,7 @@ import edu.isi.pegasus.planner.classes.PegasusBag;
  * @author Karan Vahi
  * @author Jens Vöckler
  *
- * @version $Revision: 4507 $
+ * @version $Revision$
  *
  * @see java.lang.Runtime
  * @see java.lang.Process
@@ -314,7 +310,8 @@ public class NonJavaCallout extends AbstractPerJob {
      */
     public void mapWorkflow( ADag workflow, List sites ){
         mAbstractDag = workflow;
-        mapWorkflow( Adapter.convert( workflow ), sites );
+        //PM-747 no need for conversion as ADag now implements Graph interface
+        super.mapWorkflow( workflow , sites );
     }
 
 
@@ -549,26 +546,19 @@ public class NonJavaCallout extends AbstractPerJob {
                     pool = (String) i.next();
                     st = "resource.id=" + pool + " ";
 
-                    // get handle to pool config
-/*                    List l = mSCHandle.getGridFTPServers(pool);
-                    if (l == null || l.isEmpty()) {
-                        // FIXME: How hard should this error be?
-                        mLogger.log("Site " + pool +
-                                    " has no grid ftp" +
-                                    "servers associated with it",
-                                    LogManager.WARNING_MESSAGE_LEVEL);
-                        // append a NONE grid ftp server
-                        pw.println(st + "NONE");
-                    } else {
-                        for ( Iterator j=l.iterator(); j.hasNext(); ) {
-                            pw.println(st + ( (GridFTPServer) j.next()).
-                                       getInfo(GridFTPServer.GRIDFTP_URL));
-                        }
-                    }
- */
+
                     SiteCatalogEntry site = mSiteStore.lookup( pool );
+                    /*
                     for( Iterator it = site.getHeadNodeFS().getScratch().getSharedDirectory().getFileServersIterator(); it.hasNext();){
                         pw.println(st + ( (FileServer) it.next()).getURLPrefix() );
+                    }*/
+                    Directory d = site.getDirectory( Directory.TYPE.shared_scratch );
+                    if( d != null ){
+                        for( FileServer.OPERATION op : FileServer.OPERATION.values() ){
+                            for( Iterator it = d.getFileServersIterator(op); it.hasNext();){
+                                pw.println(st + ( (FileServer) it.next()).getURLPrefix() );
+                            }
+                        }
                     }
                 } // for
             }
@@ -579,8 +569,8 @@ public class NonJavaCallout extends AbstractPerJob {
 
                 // write workflow related metadata
             if ( this.mAbstractDag != null ) {
-                pw.println("wf.name=" + mAbstractDag.dagInfo.nameOfADag);
-                pw.println("wf.index=" + mAbstractDag.dagInfo.index);
+                pw.println("wf.name=" + mAbstractDag.getLabel() );
+                pw.println("wf.index=" + mAbstractDag.getIndex() );
                 // pw.println("workflow.time=" + mAbstractDag.dagInfo.time??);
                 // FIXME: Try File.lastModified() on the DAX file
 
