@@ -470,12 +470,9 @@ public class GUC extends AbstractMultipleFTPerXFERJob {
 
            //associate any credential required , both with destination
            // and the source urls
-           job.addCredentialType( source.getValue() );
-           job.addCredentialType( dest.getValue() );
+           job.addCredentialType( source.getKey(), source.getValue() );
+           job.addCredentialType( dest.getKey(), dest.getValue() );
        }
-
-
-
    }
 
    /**
