@@ -19,20 +19,16 @@ package edu.isi.pegasus.planner.transfer.refiner;
 
 
 import edu.isi.pegasus.planner.classes.ADag;
-import edu.isi.pegasus.planner.classes.PlannerOptions;
 import edu.isi.pegasus.planner.classes.Job;
 import edu.isi.pegasus.planner.classes.FileTransfer;
 
-import edu.isi.pegasus.planner.common.PegasusProperties;
 import edu.isi.pegasus.common.logging.LogManager;
 
-import edu.isi.pegasus.planner.catalog.classes.Profiles;
 import edu.isi.pegasus.planner.catalog.classes.Profiles.NAMESPACES;
 import edu.isi.pegasus.planner.catalog.site.classes.SiteCatalogEntry;
 import edu.isi.pegasus.planner.catalog.site.classes.SiteStore;
 import edu.isi.pegasus.planner.namespace.Pegasus;
 
-import edu.isi.pegasus.planner.catalog.transformation.TransformationCatalogEntry;
 
 import edu.isi.pegasus.planner.transfer.Refiner;
 
@@ -56,10 +52,10 @@ import java.util.LinkedList;
  * the number of transfer nodes per execution site for stagein and stageout.
  *
  * @author Karan Vahi
- * @version $Revision: 4642 $
+ * @version $Revision$
  */
 
-public class Bundle extends Default {
+public class Bundle extends Basic {
 
     /**
      * A short description of the transfer refinement.
@@ -73,7 +69,7 @@ public class Bundle extends Default {
      * that are being created per execution pool for stageing in data for
      * the workflow.
      */
-    public static final String DEFAULT_LOCAL_STAGE_IN_BUNDLE_FACTOR = "4";
+    public static final String DEFAULT_LOCAL_STAGE_IN_BUNDLE_FACTOR = "2";
 
     
     /**
@@ -81,14 +77,14 @@ public class Bundle extends Default {
      * that are being created per execution pool for stageing in data for
      * the workflow.
      */
-    public static final String DEFAULT_REMOTE_STAGE_IN_BUNDLE_FACTOR = "4";
+    public static final String DEFAULT_REMOTE_STAGE_IN_BUNDLE_FACTOR = "2";
 
     /**
      * The default bundling factor that identifies the number of transfer jobs
      * that are being created per execution pool for stageing out data for
      * the workflow.
      */
-    public static final String DEFAULT_LOCAL_STAGE_OUT_BUNDLE_FACTOR = "4";
+    public static final String DEFAULT_LOCAL_STAGE_OUT_BUNDLE_FACTOR = "2";
 
 
     /**
@@ -96,7 +92,7 @@ public class Bundle extends Default {
      * that are being created per execution pool for stageing out data for
      * the workflow.
      */
-    public static final String DEFAULT_REMOTE_STAGE_OUT_BUNDLE_FACTOR = "4";
+    public static final String DEFAULT_REMOTE_STAGE_OUT_BUNDLE_FACTOR = "2";
 
 
     /**
@@ -117,7 +113,7 @@ public class Bundle extends Default {
      * used to construct the relations that need to be added to workflow, once
      * the traversal is done.
      */
-    private Map mRelationsMap;
+    private Map mRelationsParentMap;
 
     
     /**
@@ -212,7 +208,7 @@ public class Bundle extends Default {
         mStageInLocalMap   = new HashMap( mPOptions.getExecutionSites().size());
         mStageInRemoteMap   = new HashMap( mPOptions.getExecutionSites().size());
         
-        mRelationsMap = new HashMap();
+        mRelationsParentMap = new HashMap();
         mSetupMap     = new HashMap();
         mCurrentSOLevel = -1;
         mJobPrefix    = mPOptions.getJobnamePrefix();
@@ -362,6 +358,10 @@ public class Bundle extends Default {
         String par = null;
         int bundle = -1;
 
+        int priority = getJobPriority( job );
+
+
+
         //to prevent duplicate dependencies
         Set tempSet = new HashSet();
 
@@ -372,6 +372,10 @@ public class Bundle extends Default {
             FileTransfer ft = (FileTransfer) it.next();
             String lfn = ft.getLFN();
 
+            //set the priority associated with the 
+            //compute job PM-622
+            ft.setPriority( priority );
+
             //get the key for this lfn and pool
             //if the key already in the table
             //then remove the entry from
@@ -488,12 +492,12 @@ public class Bundle extends Default {
 
         //add the temp set to the relations
         //relations are added to the workflow in the end.
-        if( mRelationsMap.containsKey( jobName )){
+        if( mRelationsParentMap.containsKey( jobName )){
             //the map already has some relations for the job
             //add those to temp set to 
-            tempSet.addAll( (Set) mRelationsMap.get( jobName ) );
+            tempSet.addAll( (Set) mRelationsParentMap.get( jobName ) );
         }
-        mRelationsMap.put(jobName,tempSet);
+        mRelationsParentMap.put(jobName,tempSet);
 
 
     }
@@ -535,12 +539,18 @@ public class Bundle extends Default {
 
         mLogMsg = "Adding stageout nodes for job " + jobName;
 
+        //PM-622
+        int priority = getJobPriority( job );
+
         //separate the files for transfer
         //and for registration
         List txFiles = new ArrayList();
         List regFiles = new ArrayList();
         for (Iterator it = files.iterator(); it.hasNext(); ) {
             FileTransfer ft = (FileTransfer) it.next();
+
+            ft.setPriority( priority );
+
             if (!ft.getTransientTransferFlag()) {
                 txFiles.add(ft);
             }
@@ -635,24 +645,29 @@ public class Bundle extends Default {
                      this.mTXStageInImplementation,
                      Job.STAGE_IN_JOB,
                      false );
+        
+        //reset the stageout map too
+        this.resetStageOutMaps();
        
         //adding relations that tie in the stagin
         //jobs to the compute jobs.
-        for(Iterator it = mRelationsMap.entrySet().iterator();it.hasNext();){
+        for(Iterator it = mRelationsParentMap.entrySet().iterator();it.hasNext();){
             Map.Entry entry = (Map.Entry)it.next();
             String key   = (String)entry.getKey();
-            mLogger.log("Adding relations for job " + key,
+            mLogger.log("Adding stagein relations for job " + key,
                         LogManager.DEBUG_MESSAGE_LEVEL);
             for(Iterator pIt = ((Collection)entry.getValue()).iterator();
                                                               pIt.hasNext();){
                 String value = (String)pIt.next();
-                addRelation( value, key );
+                mLogger.log("Adding Edge " + value + " -> " + key,
+                        LogManager.DEBUG_MESSAGE_LEVEL);
+                this.mDAG.addEdge( value, key );
             }
         }
-
         
-        //reset the stageout map too
-        this.resetStageOutMaps();
+        //PM-747 add the edges in the very end
+        super.done();
+        
     }
     
     /**
@@ -836,6 +851,8 @@ public class Bundle extends Default {
     }
 
     
+
+    
     /**
      * A container class for storing the name of the transfer job, the list of
      * file transfers that the job is responsible for.
