@@ -20,6 +20,7 @@ package edu.isi.pegasus.planner.transfer.sls;
 import edu.isi.pegasus.planner.catalog.site.classes.SiteStore;
 
 import edu.isi.pegasus.common.logging.LogManager;
+import edu.isi.pegasus.common.util.PegasusURL;
 
 import edu.isi.pegasus.planner.catalog.transformation.classes.TCType;
 
@@ -29,18 +30,22 @@ import edu.isi.pegasus.planner.transfer.SLS;
 
 import edu.isi.pegasus.planner.catalog.TransformationCatalog;
 import edu.isi.pegasus.planner.catalog.transformation.TransformationCatalogEntry;
-import edu.isi.pegasus.planner.catalog.ReplicaCatalog;
 
 
+import edu.isi.pegasus.planner.catalog.replica.ReplicaCatalogEntry;
+import edu.isi.pegasus.planner.catalog.site.classes.FileServer;
+import edu.isi.pegasus.planner.catalog.site.classes.FileServerType.OPERATION;
 import edu.isi.pegasus.planner.code.gridstart.PegasusLite;
 import edu.isi.pegasus.planner.classes.PegasusBag;
 import edu.isi.pegasus.planner.classes.FileTransfer;
 import edu.isi.pegasus.planner.classes.Job;
 import edu.isi.pegasus.planner.classes.PegasusFile;
+import edu.isi.pegasus.planner.classes.PlannerCache;
 import edu.isi.pegasus.planner.classes.Profile;
 import edu.isi.pegasus.planner.namespace.ENV;
 
 import edu.isi.pegasus.planner.common.PegasusProperties;
+import edu.isi.pegasus.planner.namespace.Pegasus;
 
 import java.io.File;
 
@@ -56,7 +61,7 @@ import java.util.LinkedList;
  * second level staging.
  *
  * @author Karan Vahi
- * @version $Revision: 4780 $
+ * @version $Revision$
  */
 public class Transfer   implements SLS {
 
@@ -90,6 +95,11 @@ public class Transfer   implements SLS {
      * The derivation version number for the transfer job.
      */
     public static final String DERIVATION_VERSION = "1.0";
+    
+    /**
+     * The default number of threads pegasus-transfer uses
+     */
+    public static final int DEFAULT_NUMBER_OF_THREADS = 1;
 
     /**
      * A short description of the transfer implementation.
@@ -120,19 +130,7 @@ public class Transfer   implements SLS {
      * The handle to the logging manager.
      */
     protected LogManager mLogger;
-
    
-
-    /**
-     * The local url prefix for the submit host.
-     */
-    protected String mLocalURLPrefix;
-    
-    /**
-     * The handle to the transient replica catalog.
-     */
-    protected ReplicaCatalog mTransientRC;
-
     /**
      * Any extra arguments that need to be passed ahead to the s3 client invocation.
      */
@@ -148,6 +146,24 @@ public class Transfer   implements SLS {
      */
     protected boolean mSeqExecGridStartUsed;
 
+    /**
+     * A boolean indicating whether to bypass first level staging for inputs
+     */
+    private boolean mBypassStagingForInputs;
+
+    /**
+     * A SimpleFile Replica Catalog, that tracks all the files that are being
+     * materialized as part of workflow execution.
+     */
+    private PlannerCache mPlannerCache;
+    
+    /**
+     * This member variable if set causes the destination URL for the symlink jobs
+     * to have symlink:// url if the pool attributed associated with the pfn
+     * is same as a particular jobs execution pool. 
+     */
+    protected boolean mUseSymLinks;
+    
 
     /**
      * The default constructor.
@@ -166,13 +182,12 @@ public class Transfer   implements SLS {
         mLogger     = bag.getLogger();
         mSiteStore  = bag.getHandleToSiteStore();
         mTCHandle   = bag.getHandleToTransformationCatalog();
-
-
-        mLocalURLPrefix = mSiteStore.lookup( "local" ).getHeadNodeFS().selectScratchSharedFileServer().getURLPrefix( );
-        mTransientRC = bag.getHandleToTransientReplicaCatalog();
         mExtraArguments = mProps.getSLSTransferArguments();
         mStageSLSFile = mProps.stageSLSFilesViaFirstLevelStaging();
         mSeqExecGridStartUsed = mProps.getGridStart().equals( PegasusLite.CLASSNAME );
+        mBypassStagingForInputs = mProps.bypassFirstLevelStagingForInputs();
+        mPlannerCache = bag.getHandleToPlannerCache();
+        mUseSymLinks = mProps.getUseOfSymbolicLinks();
     }
 
     /**
@@ -209,11 +224,25 @@ public class Transfer   implements SLS {
         
         invocation.append( executable );
 
+        //get the value if any that is present in the associated compute job
+        //that should take care of the properties also
+        int threads = Transfer.DEFAULT_NUMBER_OF_THREADS;
+        if(job.vdsNS.containsKey(Pegasus.TRANSFER_THREADS_KEY )){
+            try{
+                threads = Integer.parseInt( job.vdsNS.getStringValue( Pegasus.TRANSFER_THREADS_KEY ) );
+            }
+            catch( Exception e ){
+                mLogger.log( "Invalid value picked up for Pegasus profile " + Pegasus.TRANSFER_THREADS_KEY + " PegasusLite job " + job.getID(),
+                             LogManager.ERROR_MESSAGE_LEVEL );
+            }
+        }
+        invocation.append( " --threads ").append( threads ).append( " " );
+        
 
         //append any extra arguments set by user
-        //in properties
-        if( mExtraArguments != null ){
-            invocation.append( " " ).append( mExtraArguments );
+        String args = job.vdsNS.getStringValue( Pegasus.TRANSFER_SLS_ARGUMENTS_KEY );
+        if( args != null ){
+            invocation.append( " " ).append( args );
         }
 
 
@@ -231,11 +260,6 @@ public class Transfer   implements SLS {
                 invocation.append( slsFile.getName() );
             }
         }
-
-
-
-       
-
         return invocation.toString();
 
     }
@@ -306,7 +330,8 @@ public class Transfer   implements SLS {
      *
      * @param job           job for which the file is being created
      * @param fileName      name of the file that needs to be written out.
-     * @param submitDir     submit directory where it has to be written out.
+     * @param stagingSiteServer    the file server on the staging site to be used
+     *                             for retrieval of files i.e the get operation
      * @param stagingSiteDirectory    directory on the head node of the staging site.
      * @param workerNodeDirectory  worker node directory
      *
@@ -317,7 +342,7 @@ public class Transfer   implements SLS {
      */
     public Collection<FileTransfer>  determineSLSInputTransfers( Job job,
                                       String fileName,
-                                      String submitDir,
+                                      FileServer stagingSiteServer,
                                       String stagingSiteDirectory,
                                       String workerNodeDirectory ) {
 
@@ -334,20 +359,11 @@ public class Transfer   implements SLS {
 //      To handle for null conditions?
 //        File sls = null;
         Collection<FileTransfer> result = new LinkedList();
-
-        //figure out the remote site's headnode gridftp server
-        //and the working directory on it.
-        //the below should be cached somehow
-        String sourceURLPrefix = mSiteStore.lookup( job.getStagingSiteHandle() ).getHeadNodeFS().selectScratchSharedFileServer().getURLPrefix( );
-        String sourceDir = stagingSiteDirectory;
         String destDir = workerNodeDirectory;
-
-
-        PegasusFile pf;
-
+        
         //To do. distinguish the sls file from the other input files
         for( Iterator it = files.iterator(); it.hasNext(); ){
-            pf = ( PegasusFile ) it.next();
+            PegasusFile pf = ( PegasusFile ) it.next();
             String lfn = pf.getLFN();
 
             if( lfn.equals( ENV.X509_USER_PROXY_KEY ) ){
@@ -356,30 +372,64 @@ public class Transfer   implements SLS {
                 continue;
             }
 
-            //check if the input file is in the transient RC
-            //all files in the DAX should be in the transient RC
-            String transientPFN =  mTransientRC.lookup( lfn, job.getSiteHandle() );
             FileTransfer ft = new FileTransfer();
-            if( transientPFN == null ){
-                //create the default path from the directory
-                //on the head node
-                StringBuffer url = new StringBuffer();
-                url.append( sourceURLPrefix ).append( File.separator );
-                url.append( sourceDir ).append( File.separator );
-                url.append( lfn );
-                ft.addSource( job.getStagingSiteHandle(), url.toString() );
+
+            //create the default path from the directory
+            //on the head node
+            StringBuffer url = new StringBuffer();
+
+            ReplicaCatalogEntry cacheLocation = null;
+            boolean symlink = false;
+            if( mBypassStagingForInputs ){
+                //we retrieve the URL from the Planner Cache as a get URL
+                //bypassed URL's are stored as GET urls in the cache and
+                //associated with the compute site
+                //we need a GET URL. we don't know what site is associated with
+                //the source URL. Get the first matching one
+                //PM-698
+                cacheLocation = mPlannerCache.lookup( lfn, OPERATION.get );
+            }
+            if( cacheLocation == null ){
+                String stagingSite = job.getStagingSiteHandle();
+                //construct the location with respect to the staging site
+                if( mUseSymLinks && //specified in configuration
+                    stagingSite.equals( job.getSiteHandle() ) ){ //source URL logically on the same site where job is to be run
+                    //we can symlink . construct the source URL as a file url
+                    symlink = true;
+                    url.append( PegasusURL.FILE_URL_SCHEME ).append( "//" ).append( stagingSiteDirectory );
+                    if( pf.isExecutable() ){
+                        //PM-734 for executable files we can have the source url as file url 
+                        //but we cannot have the destination URL as symlink://
+                        //as we want to do chmod on the local copy on the worker node
+                        symlink = false;
+                    }
+                }
+                else{
+                    url.append( mSiteStore.getExternalWorkDirectoryURL(stagingSiteServer, stagingSite ));
+                }
+                url.append( File.separator ).append( lfn );
+                ft.addSource( stagingSite, url.toString() );
             }
             else{
-                //use the location specified in
-                //the transient replica catalog
-//                    input.write( transientPFN );
-                ft.addSource( job.getStagingSiteHandle(), transientPFN );
-            }
+                //construct the URL wrt to the planner cache location
+                url.append( cacheLocation.getPFN() );
+                ft.addSource( cacheLocation.getResourceHandle(), url.toString() );
                 
+                symlink = ( mUseSymLinks && //specified in configuration
+                            !pf.isExecutable() && //can only do symlinks for data files . not executables
+                            ft.getSourceURL().getKey().equals( job.getSiteHandle()) && //source URL logically on the same site where job is to be run
+                            url.toString().startsWith( PegasusURL.FILE_URL_SCHEME ) ); //source URL is a file URL
+            }
+            
+            //if the source URL is already present at the compute site
+            //and is a file URL, then the destination URL has to be a symlink
+            String destURLScheme = ( symlink )?
+                                   PegasusURL.SYMLINK_URL_SCHEME:
+                                   PegasusURL.FILE_URL_SCHEME;//default is file URL
 
             //destination
-            StringBuffer url = new StringBuffer();
-            url.append( "file://" ).append( destDir ).append( File.separator ).
+            url = new StringBuffer();
+            url.append( destURLScheme ).append( "//" ).append( destDir ).append( File.separator ).
                 append( pf.getLFN() );
             ft.addDestination( job.getSiteHandle(), url.toString() );
 
@@ -394,9 +444,9 @@ public class Transfer   implements SLS {
      *
      * @param job the job for which the file is being created
      * @param fileName the name of the file that needs to be written out.
-     * @param submitDir the submit directory where it has to be written out.
-     * @param stagingSiteDirectory the directory on the head node of the
-     *   staging site.
+     * @param stagingSiteServer    the file server on the staging site to be used
+     *                             for retrieval of files i.e the put operation
+     * @param stagingSiteDirectory the directory on the head node of the  staging site.
      * @param workerNodeDirectory the worker node directory
      *
      * @return a Collection of FileTransfer objects listing the transfers that
@@ -406,7 +456,7 @@ public class Transfer   implements SLS {
      */
     public Collection<FileTransfer>  determineSLSOutputTransfers( Job job,
                                        String fileName,
-                                       String submitDir,
+                                       FileServer stagingSiteServer,
                                        String stagingSiteDirectory,
                                        String workerNodeDirectory ) {
 
@@ -421,14 +471,8 @@ public class Transfer   implements SLS {
         //      To handle for null conditions?
 //        File sls = null;
         Collection<FileTransfer> result = new LinkedList();
-
         Set files = job.getOutputFiles();
 
-        //figure out the remote site's headnode gridftp server
-        //and the working directory on it.
-        //the below should be cached somehow
-        String destURLPrefix = mSiteStore.lookup( job.getStagingSiteHandle() ).getHeadNodeFS().selectScratchSharedFileServer().getURLPrefix();
-        String destDir = stagingSiteDirectory;
         String sourceDir = workerNodeDirectory;
 
         PegasusFile pf;
@@ -446,9 +490,14 @@ public class Transfer   implements SLS {
 
             //destination
             url = new StringBuffer();
+/*
             url.append( destURLPrefix ).append( File.separator );
             url.append( destDir ).append( File.separator );
-            url.append( pf.getLFN() );
+ */
+            //on the head node
+            url.append( mSiteStore.getExternalWorkDirectoryURL(stagingSiteServer, job.getStagingSiteHandle() ));
+            url.append( File.separator ).append( pf.getLFN() );
+            
             ft.addDestination( job.getStagingSiteHandle(), url.toString() );
 
             result.add(ft);
@@ -462,103 +511,6 @@ public class Transfer   implements SLS {
     }
 
 
-    /**
-     * Modifies a job for the first level staging to headnode.This is to add
-     * any files that needs to be staged to the head node for a job specific
-     * to the SLS implementation. If any file needs to be added, a <code>FileTransfer</code>
-     * object should be created and added as an input or an output file.
-     *
-     *
-     * @param job           the job
-     * @param submitDir     the submit directory
-     * @param slsInputLFN   the sls input file if required, that is used for
-     *                      staging in from the head node to worker node directory.
-     * @param slsOutputLFN  the sls output file if required, that is used
-     *                      for staging in from the head node to worker node directory.
-     * @return boolean
-     */
-    public boolean modifyJobForFirstLevelStaging( Job job,
-                                                  String submitDir,
-                                                  String slsInputLFN,
-                                                  String slsOutputLFN ) {
-
-        String separator = File.separator;
-
-        //holds the externally accessible path to the directory on the staging site
-        String externalWorkDirectoryURL = mSiteStore.getExternalWorkDirectoryURL( job.getStagingSiteHandle() );
-        
-
-        //sanity check
-        if( !this.mStageSLSFile ){
-
-            //add condor file transfer keys if input and output lfs are not null
-            if( slsInputLFN != null ){
-                job.condorVariables.addIPFileForTransfer( submitDir + File.separator + slsInputLFN );
-            }
-            if( slsOutputLFN != null ){
-                job.condorVariables.addIPFileForTransfer( submitDir + File.separator + slsOutputLFN );
-            }
-
-            return true;
-        }
-
-        
-        //incorporate the sls input file if required
-        if( slsInputLFN != null ){
-
-            FileTransfer ft = new FileTransfer( slsInputLFN, job.getName());
-
-            //the source sls is to be sent across from the local site
-            //using the grid ftp server at local site.
-            StringBuffer sourceURL = new StringBuffer();
-            sourceURL.append( mLocalURLPrefix ).append( separator ).
-                append( submitDir ).append(separator).
-                append( slsInputLFN );
-            ft.addSource("local", sourceURL.toString());
-
-            //the destination URL is the working directory on the filesystem
-            //on the head node where the job is to be run.
-            StringBuffer destURL = new StringBuffer();
-            destURL.append( externalWorkDirectoryURL ).
-                    append( separator ).
-                    append( slsInputLFN );
-            ft.addDestination( job.getStagingSiteHandle(), destURL.toString() );
-
-            //add this as input file for the job
-            job.addInputFile( ft );
-        }
-
-        //add the sls out file as input to the job
-        if( slsOutputLFN != null ){
-            FileTransfer ft = new FileTransfer( slsOutputLFN, job.getName() );
-
-            //the source sls is to be sent across from the local site
-            //using the grid ftp server at local site.
-            StringBuffer sourceURL = new StringBuffer();
-            sourceURL.append( mLocalURLPrefix ).append( separator ).
-                      append( submitDir ).append( separator ).
-                      append( slsOutputLFN );
-
-            ft.addSource( "local" , sourceURL.toString() );
-
-            //the destination URL is the working directory on the filesystem
-            //on the head node where the job is to be run.
-            StringBuffer destURL = new StringBuffer();
-            destURL.append( externalWorkDirectoryURL )
-                    .append( separator ).
-                    append( slsOutputLFN );
-
-            ft.addDestination( job.getStagingSiteHandle(), destURL.toString()  );
-
-            //add this as input file for the job
-            job.addInputFile( ft );
-        }
-
-        
-
-       return true;
-
-    }
 
     /**
      * Modifies a compute job for second level staging. The only modification
@@ -781,5 +733,33 @@ public class Transfer   implements SLS {
         return Transfer.EXECUTABLE_BASENAME;
     }
 
+    /**
+     * Complains for head node url prefix not specified
+     *
+     * @param site   the site handle
+     *
+     * @throws RuntimeException when URL Prefix cannot be determined for various reason.
+     */
+    protected void complainForHeadNodeURLPrefix( String site ) {
+         this.complainForHeadNodeURLPrefix( null, site );
+    }
+
+    /**
+     * Complains for head node url prefix not specified
+     *
+     * @param job    the related job if any
+     * @param site   the site handle
+     *
+     * @throws RuntimeException when URL Prefix cannot be determined for various reason.
+     */
+    protected void complainForHeadNodeURLPrefix(Job job, String site ) {
+        StringBuffer error = new StringBuffer();
+        if( job != null ){
+            error.append( "[SLS Transfer] For job (" ).append( job.getID() ).append( ")." );
+        }
+        error.append( "Unable to determine URL Prefix for the FileServer for scratch shared file system on site: " ).
+              append( site );
+        throw new RuntimeException( error.toString() );
+    }
 
 }
