@@ -39,8 +39,7 @@ extern char** environ;
 #endif // LINUX
 #endif // MACHINE_SPECIFIC
 
-static const char* RCS_ID =
-  "$Id: pegasus-keg.cc 4536 2011-09-27 01:25:24Z voeckler $";
+#include "version.h"
 
 static char output[4096];
 static char pattern[] = 
@@ -461,12 +460,12 @@ identify( char* result, size_t size, const char* arg0,
 	  line, ms+1, hours, minutes, start, that-start );
 
   // phase 1: Say hi
-#ifdef HAS_SVNVERSION
-  append( result, size, "Applicationname: %s [v%s] @ %s\n", 
-	  arg0, HAS_SVNVERSION, hostname );
+#ifdef KEG_VERSION
+  append( result, size, "Applicationname: %s [%s] @ %s\n", 
+	  arg0, KEG_VERSION, hostname );
 #else
   append( result, size, "Applicationname: %s @ %s\n", arg0, hostname );
-#endif // HAS_SVNVERSION
+#endif
 
   if ( getcwd( line, linsize ) == 0 ) strcpy( line, "(n.a.)" );
   append( result, size, "Current Workdir: %s\n", line );
@@ -478,7 +477,7 @@ identify( char* result, size_t size, const char* arg0,
   pegasus_loadavg( result, size ); 
   pegasus_meminfo( result, size ); 
   pegasus_statfs( result, size ); 
-#endif // MACHINE_SPECIFIC
+#endif
 
   if ( condor ) { 
     for ( char** p = environ; *p; p++ ) {
@@ -521,7 +520,6 @@ void
 helpMe( const char* ptr, unsigned long timeout, unsigned long spinout, 
 	const char* prefix )
 {
-  puts( RCS_ID ); 
   printf( "Usage:\t%s [-a appname] [(-t|-T) thinktime] [-l fn] [-o fn [..]]\n"
 	  "\t[-i fn [..] | -G size] [-e env [..]] [-p p [..]] [-P ps]\n",
 	  ptr );
