@@ -25,6 +25,7 @@
 #include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
+#include <limits.h>
 
 #include "rwio.h"
 #include "debug.h"
@@ -33,6 +34,8 @@
 #include "mylist.h"
 #include "invoke.h"
 #include "tools.h"
+#include "version.h"
+#include "ptrace.h"
 
 /* truely shared globals */
 int isExtended = 1;     /* timestamp format concise or extended */
@@ -45,9 +48,6 @@ static int doFlush = 0; /* apply fsync() on kickstart's stdout if true */
 static AppInfo appinfo; /* sigh, needs to be global for atexit handler */
 static volatile sig_atomic_t global_no_atexit;
 
-static const char* RCS_ID =
-"$Id: pegasus-kickstart.c 4561 2011-10-04 16:21:37Z voeckler $";
-
 static
 int
 obtainStatusCode( int raw ) 
@@ -108,10 +108,10 @@ initStatFromList( mylist_p list, size_t* size )
 
     if ( (result = (StatInfo*) calloc( sizeof(StatInfo), *size )) ) {
       while ( item && i < *size ) {
-	initStatInfoFromName( result+i, item->pfn, O_RDONLY, 0 );
-	if ( item->lfn != NULL ) addLFNToStatInfo( result+i, item->lfn );
-	item = item->next;
-	++i;
+        initStatInfoFromName( result+i, item->pfn, O_RDONLY, 0 );
+        if ( item->lfn != NULL ) addLFNToStatInfo( result+i, item->lfn );
+        item = item->next;
+        ++i;
       }
     }
   }
@@ -156,35 +156,35 @@ helpMe( const AppInfo* run )
   if ( p ) ++p;
   else p=run->argv[0];
 
-  fprintf( stderr, "%s\n", RCS_ID );
   fprintf( stderr, 
 "Usage:\t%s [-i fn] [-o fn] [-e fn] [-l fn] [-n xid] [-N did] \\\n"
 "\t[-w|-W cwd] [-R res] [-s [l=]p] [-S [l=]p] [-X] [-H] [-L lbl -T iso] \\\n" 
-"\t[-B sz] [-F] (-I fn | app [appflags])\n", p );
+"\t[-B sz] [-F] [-f] (-I fn | app [appflags])\n", p );
   fprintf( stderr, 
 " -i fn\tConnects stdin of app to file fn, default is \"%s\".\n", 
-	   xlate(&run->input) );
+           xlate(&run->input) );
   fprintf( stderr, 
 " -o fn\tConnects stdout of app to file fn, default is \"%s\".\n",
-	   xlate(&run->output) );
+           xlate(&run->output) );
   fprintf( stderr, 
 " -e fn\tConnects stderr of app to file fn, default is \"%s\".\n", 
-	   xlate(&run->error) );
+           xlate(&run->error) );
   fprintf( stderr, 
 " -l fn\tProtocols invocation record into file fn, default is \"%s\".\n",
-	   xlate(&run->logfile) );
+           xlate(&run->logfile) );
 
   fprintf( stderr, 
 " -n xid\tProvides the TR name, default is \"%s\".\n"
 " -N did\tProvides the DV name, default is \"%s\".\n" 
 " -R res\tReflects the resource handle into record, default is \"%s\".\n"
 " -B sz\tResizes the data section size for stdio capture, default is %zu.\n",
-	   show(run->xformation), show(run->derivation), 
-	   show(run->sitehandle), data_section_size );
+           show(run->xformation), show(run->derivation), 
+           show(run->sitehandle), data_section_size );
   fprintf( stderr,
 " -L lbl\tReflects the workflow label into record, no default.\n"
 " -T iso\tReflects the workflow time stamp into record, no default.\n"
-" -H\tOmit <?xml ...?> header and <resource|environment> from record.\n"
+" -H\tOmit <?xml ...?> header and <machine> from record. This is used\n"
+"   \tin clustered jobs to supress duplicate information.\n"
 " -I fn\tReads job and args from the file fn, one arg per line.\n"
 " -V\tDisplays the version and exit.\n"
 " -X\tMakes the application executable, no matter what.\n"
@@ -195,6 +195,12 @@ helpMe( const AppInfo* run )
 " -s l=p\tProvides filename pairs to stat before exit, multi-option.\n"
 " \tIf the arg is prefixed with '@', it is a list-of-filenames file.\n"
 " -F\tAttempt to fsync kickstart's stdout at exit (should not be necessary).\n"
+" -f\tPrint full information including <resource>, <environment> and \n"
+"   \t<statcall>. If the job fails, then -f is implied.\n"
+" -q\tOmit <data> for <statcall> (stdout, stderr) if the job succeeds.\n"
+#ifdef HAS_PTRACE
+" -t\tEnable resource usage tracing\n"
+#endif
  );
 
   /* avoid printing of results in exit handler */
@@ -218,15 +224,11 @@ finish( void )
 
   /* PM-466 debugging */
   if ( doFlush ) {
-    struct timeval start, final;
-    int status;
-
-    now(&start);
-    status = fsync( STDOUT_FILENO ); 
-    now(&final); 
-    debugmsg( "# fsync(%d)=%d (errno=%d) in %.3f s\n", 
-	      STDOUT_FILENO, status, errno,
-	      mymaketime(final)-mymaketime(start) ); 
+    int status = fsync(STDOUT_FILENO);
+    if (status != 0) {
+      debugmsg("WARNING: fsync(%d)=%d (errno=%d, strerror=%s)\n",
+               STDOUT_FILENO, status, errno, strerror(errno));
+    }
   }
 
   nfs_sync( STDERR_FILENO, DEFAULT_SYNC_IDLE );
@@ -241,7 +243,7 @@ show_args( const char* prefix, char** argv, int argc )
   debugmsg( "argc=%d\n", argc );
   for ( i=0; i<argc; ++i )
     debugmsg( "%s%2d: %s\n", (prefix ? prefix : ""), i, 
-	      (argv[i] ? argv[i] : "(null)" ) );
+              (argv[i] ? argv[i] : "(null)" ) );
 }
 #endif
 
@@ -310,29 +312,6 @@ handleOutputStream( StatInfo* stream, const char* temp, int std_fileno )
 
 extern char** environ;
 
-static
-int
-areWeSane( const char* what )
-/* purpose: count the number of occurances of a specific environment variable
- * paramtr: what (IN): environment variable name
- * returns: the count. 
- * warning: Produces a warning on stderr if count==0
- */
-{
-  size_t len = strlen(what);
-  int count = 0;
-  char** s = environ;
-
-  while ( s && *s ) {
-    if ( strncmp( *s, what, len ) == 0 && (*s)[len] == '=' ) count++;
-    ++s;
-  }
-
-  if ( ! count ) 
-    debugmsg( "Warning! Did not find %s in environment!\n", what );
-  return count;
-}
-
 static
 char*
 noquote( char* s )
@@ -357,37 +336,20 @@ noquote( char* s )
 int
 main( int argc, char* argv[] )
 {
-  size_t m, cwd_size = getpagesize();
+  size_t cwd_size = getpagesize();
   int status, result;
   int i, j, keeploop;
   int createDir = 0;
-#if 0
-  long fsflags = -1; 
-#endif
   const char* temp;
   const char* workdir = NULL;
   mylist_t initial;
   mylist_t final;
- 
+
   /* premature init with defaults */
   if ( mylist_init( &initial ) ) return 43;
   if ( mylist_init( &final ) ) return 43;
   initAppInfo( &appinfo, argc, argv );
 
-#if 0
-  debugmsg( "# appinfo=%d, jobinfo=%d, statinfo=%d, useinfo=%d\n",
-	    sizeof(AppInfo), sizeof(JobInfo), sizeof(StatInfo),
-	    sizeof(struct rusage) );
-#endif
-
-#if 0
-  /* NEW: 2011-08-19: PM-466 -- best effort add O_SYNC flag to stdout */
-  /* Handled differently in finish() above using fsync() on stdout */
-  if ( (fsflags=fcntl( STDOUT_FILENO, F_GETFL )) != -1 ) { 
-    fcntl( STDOUT_FILENO, F_SETFL, ( fsflags | O_SYNC ) ); 
-  }
-#endif
-
   /* register emergency exit handler */
   if ( atexit( finish ) == -1 ) {
     appinfo.application.status = -1;
@@ -403,28 +365,40 @@ main( int argc, char* argv[] )
 
   /*
    * read commandline arguments
-   * DO NOT use getopt to avoid cluttering flags to the application 
+   * DO NOT use getopt to avoid cluttering flags to the application
    */
   for ( keeploop=i=1; i < argc && argv[i][0] == '-' && keeploop; ++i ) {
     j = i;
     switch ( argv[i][1] ) {
     case 'B':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg("ERROR: -B argument missing\n");
+          return 127;
+      }
       temp = argv[i][2] ? &argv[i][2] : argv[++i];
-      m = strtoul( temp, 0, 0 );
-      /* limit max <data> size to 64 MB for each. */
-      if ( m < 67108863ul ) data_section_size = m;
+
+      /* The special value 'all' means that we echo all the output */
+      if (strcmp(temp, "all") == 0) {
+          data_section_size = ULONG_MAX;
+          break;
+      }
+
+      /* Otherwise, we expect a unsigned long value */
+      size_t m = strtoul(temp, 0, 0);
+      if (m == 0 || m == ULONG_MAX) {
+        debugmsg("ERROR: Invalid -B argument: %s\n", temp);
+        return 127;
+      }
+      data_section_size = m;
+
       break;
-#if 0
-    case 'c':
-      if ( appinfo.channel.source != IS_INVALID )
-	deleteStatInfo( &appinfo.channel );
-      temp = argv[i][2] ? &argv[i][2] : argv[++i];
-      initStatInfoAsFifo( &appinfo.channel, temp, "GRIDSTART_CHANNEL" );
-      break;
-#endif
     case 'e':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -e argument missing\n");
+          return 127;
+      }
       if ( appinfo.error.source != IS_INVALID )
-	deleteStatInfo( &appinfo.error );
+        deleteStatInfo( &appinfo.error );
       temp = ( argv[i][2] ? &argv[i][2] : argv[++i] );
       handleOutputStream( &appinfo.error, temp, STDERR_FILENO );
       break;
@@ -433,100 +407,171 @@ main( int argc, char* argv[] )
       helpMe( &appinfo );
       break; /* unreachable */
     case 'V':
-      puts( RCS_ID );
+      puts( PEGASUS_VERSION );
       appinfo.isPrinted=1;
       return 0;
     case 'i':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -i argument missing\n");
+          return 127;
+      }
       if ( appinfo.input.source != IS_INVALID )
-	deleteStatInfo( &appinfo.input );
+        deleteStatInfo( &appinfo.input );
       temp = argv[i][2] ? &argv[i][2] : argv[++i];
       if ( temp[0] == '-' && temp[1] == '\0' )
-	initStatInfoFromHandle( &appinfo.input, STDIN_FILENO );
+        initStatInfoFromHandle( &appinfo.input, STDIN_FILENO );
       else
-	initStatInfoFromName( &appinfo.input, temp, O_RDONLY, 2 );
+        initStatInfoFromName( &appinfo.input, temp, O_RDONLY, 2 );
       break;
     case 'H':
       appinfo.noHeader++;
       break;
+    case 'f':
+      appinfo.fullInfo++;
+      break;
     case 'F':
       doFlush++;
       break;
     case 'I':
+      /* XXX We expect exactly 1 argument after -I. If we see more,
+       * then it is considered to be an error. This should be fixed
+       * to work in a more sensible fashion.
+       */
+      if (argc > i+2) {
+          debugmsg("ERROR: No arguments allowed after -I fn\n");
+          return 127;
+      }
+
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -I argument missing\n");
+          return 127;
+      }
+
       /* invoke application and args from given file */
       temp = argv[i][2] ? &argv[i][2] : argv[++i];
       if ( readFromFile( temp, &argv, &argc, &i, j ) == -1 ) {
-	int saverr = errno;
-	debugmsg( "ERROR: While parsing -I %s: %d: %s\n",
-		  temp, errno, strerror(saverr) );
-	appinfo.application.prefix = strerror(saverr);
-	appinfo.application.status = -1;
-	return 127;
+        int saverr = errno;
+        debugmsg( "ERROR: While parsing -I %s: %d: %s\n",
+                  temp, errno, strerror(saverr) );
+        appinfo.application.prefix = strerror(saverr);
+        appinfo.application.status = -1;
+        return 127;
       }
       keeploop = 0;
       break;
     case 'l':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -l argument missing\n");
+          return 127;
+      }
       if ( appinfo.logfile.source != IS_INVALID )
-	deleteStatInfo( &appinfo.logfile );
+        deleteStatInfo( &appinfo.logfile );
       temp = argv[i][2] ? &argv[i][2] : argv[++i];
       if ( temp[0] == '-' && temp[1] == '\0' )
-	initStatInfoFromHandle( &appinfo.logfile, STDOUT_FILENO );
+        initStatInfoFromHandle( &appinfo.logfile, STDOUT_FILENO );
       else
-	initStatInfoFromName( &appinfo.logfile, temp, O_WRONLY | O_CREAT | O_APPEND, 2 );
+        initStatInfoFromName( &appinfo.logfile, temp, O_WRONLY | O_CREAT | O_APPEND, 2 );
       break;
     case 'L':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -L argument missing\n");
+          return 127;
+      }
       appinfo.wf_label = noquote( argv[i][2] ? &argv[i][2] : argv[++i] );
       break;
     case 'n':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -n argument missing\n");
+          return 127;
+      }
       appinfo.xformation = noquote( argv[i][2] ? &argv[i][2] : argv[++i] );
       break;
     case 'N':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -N argument missing\n");
+          return 127;
+      }
       appinfo.derivation = noquote( argv[i][2] ? &argv[i][2] : argv[++i] );
       break;
     case 'o':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -o argument missing\n");
+          return 127;
+      }
       if ( appinfo.output.source != IS_INVALID )
-	deleteStatInfo( &appinfo.output );
+        deleteStatInfo( &appinfo.output );
       temp = ( argv[i][2] ? &argv[i][2] : argv[++i] );
       handleOutputStream( &appinfo.output, temp, STDOUT_FILENO );
       break;
+    case 'q':
+      appinfo.omitData = 1;
+      break;
     case 'R':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -R argument missing\n");
+          return 127;
+      }
       appinfo.sitehandle = noquote( argv[i][2] ? &argv[i][2] : argv[++i] );
       break;
     case 'S':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -S argument missing\n");
+          return 127;
+      }
       temp = argv[i][2] ? &argv[i][2] : argv[++i];
       if ( temp[0] == '@' ) {
-	/* list-of-filenames file */
-	if ( (result=mylist_fill( &initial, temp+1 )) )
-	  debugmsg( "ERROR: initial %s: %d: %s\n", 
-		    temp+1, result, strerror(result) );
+        /* list-of-filenames file */
+        if ( (result=mylist_fill( &initial, temp+1 )) )
+          debugmsg( "ERROR: initial %s: %d: %s\n", 
+                    temp+1, result, strerror(result) );
       } else {
-	/* direct filename */
-	if ( (result=mylist_add( &initial, temp )) )
-	  debugmsg( "ERROR: initial %s: %d: %s\n", 
-		    temp, result, strerror(result) );
+        /* direct filename */
+        if ( (result=mylist_add( &initial, temp )) )
+          debugmsg( "ERROR: initial %s: %d: %s\n", 
+                    temp, result, strerror(result) );
       }
       break;
     case 's':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -s argument missing\n");
+          return 127;
+      }
       temp = argv[i][2] ? &argv[i][2] : argv[++i];
       if ( temp[0] == '@' ) {
-	/* list-of-filenames file */
-	if ( (result=mylist_fill( &final, temp+1 )) )
-	  debugmsg( "ERROR: final %s: %d: %s\n", 
-		   temp+1, result, strerror(result) );
+        /* list-of-filenames file */
+        if ( (result=mylist_fill( &final, temp+1 )) )
+          debugmsg( "ERROR: final %s: %d: %s\n", 
+                   temp+1, result, strerror(result) );
       } else {
-	/* direct filename */
-	if ( (result=mylist_add( &final, temp )) )
-	  debugmsg( "ERROR: final %s: %d: %s\n", 
-		   temp, result, strerror(result) );
+        /* direct filename */
+        if ( (result=mylist_add( &final, temp )) )
+          debugmsg( "ERROR: final %s: %d: %s\n", 
+                   temp, result, strerror(result) );
       }
       break;
     case 'T':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -T argument missing\n");
+          return 127;
+      }
       appinfo.wf_stamp = noquote( argv[i][2] ? &argv[i][2] : argv[++i] );
       break;
+    case 't':
+      appinfo.enableTracing++;
+      break;
     case 'w':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -w argument missing\n");
+          return 127;
+      }
       workdir = noquote( argv[i][2] ? &argv[i][2] : argv[++i] );
       createDir = 0;
       break;
     case 'W':
+      if (!argv[i][2] && argc <= i+1) {
+          debugmsg( "ERROR: -W argument missing\n");
+          return 127;
+      }
       workdir = noquote( argv[i][2] ? &argv[i][2] : argv[++i] );
       createDir = 1;
       break;
@@ -543,9 +588,6 @@ main( int argc, char* argv[] )
     }
   }
 
-  /* sanity check -- for FNAL/ATLAS */
-  areWeSane("GRIDSTART_CHANNEL");
-
   /* initialize app info and register CLI parameters with it */
   if ( argc-i > 0 ) {
     /* there is an application to run */
@@ -556,14 +598,14 @@ main( int argc, char* argv[] )
       appinfo.application.status = -1;
       switch ( appinfo.application.isValid ) { 
       case 2: /* permissions? */
-	appinfo.application.saverr = EACCES;
-	break; 
+        appinfo.application.saverr = EACCES;
+        break; 
       default: /* no such file? */
-	appinfo.application.saverr = ENOENT; 
-	break;
+        appinfo.application.saverr = ENOENT; 
+        break;
       }
       fputs( "FATAL: The main job specification is invalid or missing.\n",
-	     stderr );
+             stderr );
       return 127;
     }
   } else {
@@ -579,14 +621,14 @@ main( int argc, char* argv[] )
       createDir = 0; /* once only */
 
       if ( mkdir( workdir, 0777 ) == 0 ) {
-	/* If this causes an infinite loop, your file-system is
-	 * seriously whacked out -- run fsck or equivalent. */
-	goto REDIR;
+        /* If this causes an infinite loop, your file-system is
+         * seriously whacked out -- run fsck or equivalent. */
+        goto REDIR;
       }
       /* else */
       appinfo.application.saverr = errno;
       debugmsg( "Unable to mkdir %s: %d: %s\n", 
-	       workdir, errno, strerror(errno) );
+               workdir, errno, strerror(errno) );
       appinfo.application.prefix = "Unable to mkdir: ";
       appinfo.application.status = -1;
       return 127;
@@ -595,7 +637,7 @@ main( int argc, char* argv[] )
     /* unable to use alternate workdir */
     appinfo.application.saverr = errno;
     debugmsg( "Unable to chdir %s: %d: %s\n", 
-	     workdir, errno, strerror(errno) );
+             workdir, errno, strerror(errno) );
     appinfo.application.prefix = "Unable to chdir: ";
     appinfo.application.status = -1;
     return 127;
@@ -627,7 +669,7 @@ main( int argc, char* argv[] )
   if ( prepareSideJob( &appinfo.setup, getenv("GRIDSTART_SETUP") ) )
     mysystem( &appinfo, &appinfo.setup, environ );
 
-  /* possible prae job */
+  /* possible pre job */
   result = 0;
   if ( prepareSideJob( &appinfo.prejob, getenv("GRIDSTART_PREJOB") ) ) {
     /* there is a prejob to be executed */
@@ -660,6 +702,9 @@ main( int argc, char* argv[] )
   appinfo.final = initStatFromList( &final, &appinfo.fcount );
   mylist_done( &final );
 
+  /* Record final result */
+  appinfo.status = result;
+
   /* append results to log file */
   printAppInfo( &appinfo );
 
@@ -667,16 +712,6 @@ main( int argc, char* argv[] )
   global_no_atexit = 1; /* disable atexit handler */
   deleteAppInfo( &appinfo );
 
-  /* force NFS sync for gatekeeper */
-#if 0
-  /* FIXME: No locking on stdout, because printAppInfo will have done so */
-  nfs_sync( STDOUT_FILENO, DEFAULT_SYNC_IDLE );
-#endif
-#if 0
-  /* FIXME: No locking on stderr, because atexit-handler finish() does it. */
-  nfs_sync( STDERR_FILENO, DEFAULT_SYNC_IDLE );
-#endif
-
   /* done */
   return result;
 }
