@@ -18,26 +18,12 @@
 #include <sys/types.h>
 #include <sys/resource.h>
 
-#ifdef SOLARIS
-#ifndef HAS_USAGE_MEM
-#define HAS_USAGE_MEM 1
-#endif
-#ifndef HAS_USAGE_IO
-#define HAS_USAGE_IO 1
-#endif
-#ifndef HAS_USAGE_MSG
-#define HAS_USAGE_MSG 1
-#endif
-#endif /* SOLARIS */
-
 extern
 int
-printXMLUseInfo( char* buffer, size_t size, size_t* len, size_t indent,
-		 const char* id, const struct rusage* use );
-/* purpose: format the rusage record into the given buffer as XML.
- * paramtr: buffer (IO): area to store the output in
- *          size (IN): capacity of character area
- *          len (IO): current position within area, will be adjusted
+printXMLUseInfo(FILE *out, int indent, const char* id, 
+                const struct rusage* use);
+/* purpose: format the rusage record into the given stream as XML.
+ * paramtr: out (IO): the stream
  *          indent (IN): indentation level
  *          id (IN): object identifier to use as element tag name.
  *          use (IN): struct rusage info
